// |$Id: bcnv.c,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $          |
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              |
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS    http://cgiss.boisestate.edu/~pm     | 
   | FUNCTION NAME: bcnv.c                                      |
   | DIRECTORY: bsu-2.0.2/src/C/bsegy                           |
   |------------------------------------------------------------|
   | purpose: convert SEGY with reel header to BSEGY            |
   |          without reel header.                              |
   |------------------------------------------------------------|
   |  NOTICE:                                                   |
   |  Copyright (C) 2017 Paul Michaels                          |
   |  <pm@cgiss.boisestate.edu>                                 |
   |  This program is free software; you can                    |
   |  redistribute it and/or modify it under the terms          |
   |  of the GNU General Public License as published            |
   |  by the Free Software Foundation; either version           |
   |  3 of the License, or (at your option) any later           |
   |  version.  This program is distributed in the              |
   |  hope that it will be useful, but                          |
   |  WITHOUT ANY WARRANTY; without even the implied            |
   |  warranty of MERCHANTABILITY or FITNESS FOR A              |
   |  PARTICULAR PURPOSE.  See the GNU General Public           |
   |  License for more details.                                 |
   |  You should have received a copy of the GNU                |
   |  General Public License along with this program;           |
   |  if not, write to the Free Software Foundation,            |
   |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
   |------------------------------------------------------------|
   |INLINE FUNCTIONS:                                           |
   |  getparm    get parameters from command line or interactive|
   |  outlst     echo check of input parameters to listing file |
   |  BSEGY --> SEG-Y Conversion:                               |
   |  seg2sgy_step1 scan file for maximum value, xmax           |
   |  seg2sgy_step2 write ASCII portion of reel header          |
   |  seg2sgy_step3 write BINARY portion of reel header         |
   |  seg2sgy_step4 dynamic allocation of output trace memory   |
   |  seg2sgy_step5 scale microvolts to seismograph value       |
   |  bsgyout       write a SEG-Y trace header and data         |
   |  sgyformat converts BSEGY to SEG-Y format as needed        |
   |         (to IBM Floats, Big Endian, Byte swap header, data)|
   |  Note: Estep* versions of above are for SEGY compliant     |
   |                                                            |
   |  SEG-Y --> BSEGY Conversion:                               |
   |  sgy2seg_step1 read ASCII portion of reel header           |
   |  sgy2seg_step2 read BINARY portion of reel header          |
   |  sgy2seg_step3 dynamic allocation of trace memory          |
   |  bsgyin        read SEG-Y trace header and data            |
   |  sgy2seg_step4 convert seismograph value to microvolts     |
   |  segformat converts SEG-Y to native host format as needed  |
   |       (from IBM Floats, Big Endian, Byte swap header, data)|
   |  Note: Estep* versions of above are for SEGY compliant     |
   |                                                            |
   |  USED BY EITHER DIRECTION:                                 |
   |  swap4  4 byte swap                                        |
   |  swap2  2 byte swap                                        |
   |                                                            |
   |------------------------------------------------------------|
   |sublibC4 functions:                                         |
   |  c_bsegin   read seismic trace                             |
   |  c_bsegout  write a seismic trace                          |
   |  in_chk     scan input file for samples, number traces etc.|
   |  bargrid    write progress bar grid to screen              |
   |  exbar      write progress bar to screen                   |
   |  cr_labl    write copyright label to screen                |
   |  fcr_labl   write copyright label to file                  |
   |  names      create an output file name from input file name|
   |                                                            |
   |------------------------------------------------------------|
*/
      #include <sys/types.h>
      #include <stdio.h>
      #include <stdlib.h>
      #include <float.h>
      #include <math.h>
      #include <string.h>
      #include <time.h>
      #include "../include/c_bsegy.h"

//...parameter definitions

//...function prototypes
      #include "../include/sub4.h"

	void *
	ebcdic2ascii(void *dest, const void *srce, int count);

	void *
	ascii2ebcdic(void *dest, const void *srce, int count);

      int  getparm(int argc,char *argv[],
         int *endian, int *compliance, int *idirec, int *idfc, int *iunits,
	 char title[], char hedfil[], char infile[],
         char ofile[], char lstfile[],char pid[]);

      void outlst(int endian, int compliance, int idirec, int idfc, int iunits,
	 char title[], char hedfil[], char infile[],
         char pid[], char ofile[], char lstfile[], FILE *h3);

      int seg2sgy_step1(int *ntraces, int *npts,float *fsamin,
	  float *s1, float *xmax, struct header *hd,  FILE *h1 );

      int seg2sgy_step2(char title[],char hedfil[],  FILE *h2,
	  FILE *h3);

      int seg2sgy_Estep2(char title[],char hedfil[], FILE *h2,
	  FILE *h3);

      int seg2sgy_step3(float xmax, int ntraces, int npts, 
	short *twfhld,  float fsamin, float *factor, int idfc,
	int iunits,  struct header hd,  FILE *h2, FILE *h3 );

      int seg2sgy_Estep3(float xmax, int ntraces, int npts, 
	short *twfhld,  float fsamin, float *factor, int idfc,
	int iunits,  struct header hd,  FILE *h2, FILE *h3 );

      void *  seg2sgy_step4(int idfc, int npts );

      void  seg2sgy_step5(int npts, float *s1, void *s2, int idfc, 
        	 struct header hd  );

      int  sgy2seg_step1(char title[], FILE *h1, FILE *h3);

      int  sgy2seg_step2(int *npts, int *idfc, int *ntraces,
	   int *iunits,	float *fsamin, FILE *h1, FILE *h3 );

      int  sgy2seg_Estep1(char title[], char infile[], FILE *h3);

      int  sgy2seg_Estep2(int *npts, int *idfc, int *ntraces,
	   int *iunits,	float *fsamin, FILE *h1, FILE *h3 );

      void * sgy2seg_step3(int npts, int idfc);

      void  sgy2seg_step4(int idfc, int npts, void *s2, void *s3, 
		struct header hd );

      void ConvertIEEEToFloat(void *src, void *dst);

      void ConvertFloatToIEEE(void  *src, void  *dst);

      void swap4(void *x4);

      void swap2(void *x2);

      void sgyformat(int idfc, int endian, int compliance, void  *s2,
      int npts, struct header *hd, FILE *h2);

      void segformat(int idfc, int endian, int compliance, void  *s2,
      int npts, struct header *hd, FILE *h2);

      int bsgyin(int idfc, int jrec, void  *s2, int npts,
        struct header *hd, FILE *h1);

      int bsgyout(int idfc, int jrec, void  *s2, int npts,
        struct header *hd, FILE *h1);

      int main(int argc, char *argv[])
     {

//...declare variables
      FILE              *h1, *h2, *h3;
      int               jrec,
                        npts,
                        ntraces,
                        ibar,
                        invbar;
      float             fsamin,
                        bar,
                        *s1;
      void              *s2,
      			*s3;
      char              pid[5]="bcnv",   //process id name
                        ofile[13],
                        lstfile[13],
                        infile[14];
      struct header     hd;

//...specific to this application
      char              title[82];
      char              hedfil[13];
      int               idirec,
                        idfc,
			endian=0, /* endian=0 Little Endian,
				 PC endian=1 Big Endian */
         		compliance=1, /* 1=Compliance with SEGY Standard 
			         EBCDIC header, IBM Float, BigEndian
				 0=Native Format extension of SEGY
				 ASCII header, Native Float, LittleEndian */
                        iunits;
      short		twfhld;
      float		xmax;
      float		factor;

      
//...get parameters
      if(getparm(argc,argv,&endian,&compliance, &idirec,&idfc,&iunits,title,
      hedfil,infile,ofile,lstfile,pid)!=0)
      {
       goto quick_exit;
      }

//...open output listing file
        if( (h3=fopen(lstfile,"w"))==NULL )
        {
          fprintf(stderr,"\n ABORT !! Error opening file %s \n",lstfile);
          goto quick_exit;
        }

//...echo check to listing file
      outlst(endian,compliance,idirec,idfc,iunits,title,hedfil,infile,
         pid,ofile,lstfile,h3);
	fflush(h3);


//------------BRANCH According to Direction----------------
//idirec==0  SEG-->SGY  (BSEGY to SEG-Y)===================
	if(idirec==0) {
//...open input file
        if( (h1=fopen(infile,"rb"))==NULL )
        {
          fprintf(stderr,"\n ABORT !! Error opening file %s \n",infile); 
          goto exit_3;
        }

//...open output file
        if( (h2=fopen(ofile,"wb"))==NULL )
        {
          fprintf(stderr,"\n ABORT !! Error opening file %s \n",ofile);
          goto exit_1;
        }

		//...check input file size
          if(in_chk(&ntraces,&npts,&fsamin,hd,h1)!=0)
         {
          goto exit_2;
         }

//...print input file data to lstfile
        fprintf(h3,"\n     Number of samples= %d",npts);
        fprintf(h3,"\n     Sample Interval = %5.4f sec.",fsamin);
        fprintf(h3,"\n     Number of traces = %d\n",ntraces);
        fflush(h3);

//...dynamic allocation of seimic trace memory
        s1=calloc(npts,sizeof(float));
          if(s1==NULL)
         {
          fprintf(stderr,"ABORT:s1 can't allocate enough trace memory\n");
          fprintf(stderr," npts = %d",npts);
          goto exit_2;
         }

//...scan for maximum value to scale if type conversions
        if(seg2sgy_step1(&ntraces,&npts,&fsamin,s1,&xmax,&hd,h1)!=0)
	{ goto quit_it;  };

	if (compliance == 1) {
//...Write EBCDIC Portion of REEL Header
         if (seg2sgy_Estep2(title,hedfil,h2,h3)!=0)
	 { goto quit_it;}
	}
	else {
//...Write ASCII Portion of REEL Header
         if (seg2sgy_step2(title,hedfil,h2,h3)!=0)
	 { goto quit_it;}
	} //endif

if (endian == 0 && compliance ==1)  {
//...write binary portion of REEL Header BYTE SWAP
	if(seg2sgy_Estep3(xmax,ntraces,npts,&twfhld,fsamin,&factor,
	       idfc,iunits,hd,h2,h3)!=0)
	{fprintf(stderr,"ABORT--write error in seg2sgy_step3\n");
		goto quit_it; }
}
else {
// if endian=1 and any compliance, or if endian=0 and compliance=0
//...write binary portion of REEL Header
	if(seg2sgy_step3(xmax,ntraces,npts,&twfhld,fsamin,&factor,
	       idfc,iunits,hd,h2,h3)!=0)
	{fprintf(stderr,"ABORT--write error in seg2sgy_step3\n");
		goto quit_it; }
}

//...dynamic allocation of output trace memory
	s2=seg2sgy_step4(idfc,npts);
	if(s2==NULL)
         {
          fprintf(stderr,"ABORT:s2 can't allocate enough trace memory");
          fprintf(stderr," npts = %d",npts);
          goto exit_s1;
         }
		
//...display progress bar grid

      bargrid(ntraces,&bar,&ibar,&invbar);
	fflush(h2);
	
//...begin trace header/data sequence++		
	for (jrec=0;jrec<ntraces;jrec++)
	{

//...read trace
        if(c_bsegin(jrec,s1,npts,&hd,h1)!=0)
       {
        fprintf(stderr,"ABORT--c_bsegin");
        goto quit_it;
       }

//...load output trace vector with correct data type
	 hd.twf=twfhld;
         seg2sgy_step5(npts,s1,s2,idfc,hd); 

//convert to segy format, endian IBM if float, etc.
	sgyformat(idfc,endian,compliance,s2,npts,&hd,h2);

//...write trace to file
         if( bsgyout(idfc,jrec,s2,npts,&hd,h2)!=0)
	{ fprintf(stderr,"Abort on bsgyout");
		goto quick_exit;
	}
		 
//...display progress bar
      exbar(ntraces,bar,ibar,invbar,jrec+1);

	 
	}  //endfor  header/data/sequence++


	} //------end of SEG-->SGY  =======================        
		else
//BRANCH 
//idirec==1  SGY-->SEG  (SEG-Y to BSEGY)===================
       {

//..READ REEL HEADER  
	if (compliance ==1 ) {
	       //EBCDIC PART 0 to 3199 byte
        if(sgy2seg_Estep1(title,infile,h3)!=0)
	{fprintf(stderr," ABORT- error reading EBCDIC reel header \n");
		goto quit_it; }
	fflush(h3);

//...open input file
        if( (h1=fopen(infile,"rb"))==NULL )
        {
          fprintf(stderr,"\n ABORT !! Error opening file %s \n",infile); 
          goto exit_3;
        }

//...open output file
        if( (h2=fopen(ofile,"wb"))==NULL )
        {
          fprintf(stderr,"\n ABORT !! Error opening file %s \n",ofile);
          goto exit_1;
        }
	
		//BINARY PART 3200 to 3599 byte
        if(sgy2seg_Estep2(&npts,&idfc,&ntraces,&iunits,&fsamin,h1,h3)!=0)
	{goto quit_it; }

	}
	else {
//...open input file
        if( (h1=fopen(infile,"rb"))==NULL )
        {
          fprintf(stderr,"\n ABORT !! Error opening file %s \n",infile); 
          goto exit_3;
        }

//...open output file
        if( (h2=fopen(ofile,"wb"))==NULL )
        {
          fprintf(stderr,"\n ABORT !! Error opening file %s \n",ofile);
          goto exit_1;
        }

	       //ASCII PART 0 to 3199 byte
        if(sgy2seg_step1(title,h1,h3)!=0)
	{fprintf(stderr," ABORT- error reading ACII reel header \n");
		goto quit_it; }
	fflush(h3);
	
		//BINARY PART 3200 to 3599 byte
        if(sgy2seg_step2(&npts,&idfc,&ntraces,&iunits,&fsamin,h1,h3)!=0)
	{goto quit_it; }

	}  //endif


//...file pointer at byte 3600

//...dynamic allocation of seimic trace memory
        s3=sgy2seg_step3(npts,idfc);
	if(s3==NULL)
	{ fprintf(stderr,"ABORT- error trace memory sgy2seg_step3");
		goto quit_it;}

//...dynamic allocation of bsegy output trace
 //        (float *) s2=calloc(npts,sizeof(float));
          s2=calloc(npts,sizeof(float));
          if(s2==NULL)
         {
          fprintf(stderr,"ABORT:s2 can't allocate enough trace memory");
          fprintf(stderr," npts = %d",npts);
          goto exit_s1;
         }

//-----------------main loop---------------------------
//...display progress bar grid

      bargrid(ntraces,&bar,&ibar,&invbar);

//...trace loop ++++++++

      for (jrec=0;jrec<ntraces;jrec++)
     {

//...read trace
         if(bsgyin(idfc,jrec,s3,npts,&hd,h1)!=0)
        {
         fprintf(stderr,"ABORT--bsgyin");
         goto quit_it;
        }

//convert back to bsegy format if needed
	segformat(idfc,endian,compliance,s3,npts,&hd,h2);

	 if(jrec==0) {fprintf(h3,
	 "\nTrace Weight Factor: hd.twf=%d",hd.twf); }

	if (compliance == 1) {	 
	//convert from seismograph number to microvolts
       sgy2seg_step4(idfc,npts,s2,s3,hd);
	}
	else {
       sgy2seg_step4(idfc,npts,s2,s3,hd);
	} //endif 

//...output trace
        if(c_bsegout(jrec,s2,npts,&hd,h2)!=0)
       {
        fprintf(stderr,"ABORT--c_bsegout");
        goto quit_it;
       }

//...display progress bar
      exbar(ntraces,bar,ibar,invbar,jrec+1);

     } //  end trace loop +++++++++

   }  //end BRANCH IF BLOCK============================ 

      printf("\n   Listing ====>%s",lstfile);
      printf("\n   Output  ====>%s",ofile);
      printf("\n   Processed %d traces",ntraces);
      return(0); 
quit_it:
      return(0);
        puts(" ");

        free(s2);
exit_s1:
        free(s1);
exit_2:
        fclose(h2);
exit_1:
        fclose(h1);
exit_3:
        fclose(h3);

quick_exit:
	return(1);
     } //  end of main

/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: getparm.c                                   |
   | Copyright (C) 2016 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: gets input parameters from command                |
   |          line or prompts user from consol.                 |
   |                                                            |
   |                                                            |
   |------------------------------------------------------------|
*/
      int  getparm(int argc,char *argv[],
         int *endian, int *compliance, int *idirec, int *idfc, int *iunits,
	 char title[], char hedfil[], char infile[],
         char ofile[], char lstfile[],char pid[])
     {
      char              sfx1[4]="seg",
                        sfx2[4]="lst",
                        sfx3[4]="sgy";
      char		help[3]="-h";
      char              msg1[46],msg2[46];


//...test for command line help ?
      if(argc==2 && !strcmp(argv[1],help))
     {
//...screen header and copyright notice
      cr_labl(pid);

//...help pannel
      sprintf(msg1,"Conversion Program  segy <----> BSU bsegy    ");
      sprintf(msg2,"                    *.sgy        *.seg       ");
      hlp_labl(msg1,msg2);
      puts(" ");

//...help display
      puts(" ");
      printf("  %s  infile endian compliance idirec idfc iunits hedfil \n",pid);
      puts(" ");
      puts("     infile   =  input file name");
      puts("     endian  0= Little Endian host (Linux PC)   ");
      puts("             1= Big    Endian host (IBM Mainframe)");
      puts("  compliance 1= SEGY Compliant (EBCDIC, IBM Float, BigEndian) ");
      puts("             0= (ASCII Reel Header, Float and endian of host) ");
      puts("     idirec  0= BSEGY ==>SEG-Y (new *.sgy)");
      puts("             1= SEG-Y ==>BSEGY (new *.seg)");
      puts("     idfc    1= floating point 4 byte output");
      puts("             2= long integer   4 byte output");
      puts("             3= short integer  2 byte output");
      puts("             (uses reel header if SEG-Y input data)");
      puts("     iunits  1= meters");
      puts("             2= feet");
      puts("             (uses reel header if SEG-Y input data)");
      puts(" ");
      puts("NOTE: ");
      puts(" hedfil only input if idirec=0  BSEGY --> SEG-Y ");
      puts(" (hedfil contains 3200 bytes, 40 records, 80char each");
      puts(" If hedfil='none', then blank lines after C used ");
      puts(" ");
      puts(" ");
      return(1);
     }
      else
     {
//...get parameters------------------------------
//...pars off character parameters
      if(argc>1)
       strncpy(infile,argv[1],13);
       else
      {
       puts(" Enter INPUT file name ");
       scanf(" %13s",infile);
      }
	if (namecheck(12,infile) != 0 ) return (1);

//...convert string to integer
      if(argc>2)
       *idirec=atoi(argv[2]);
       else
      {
       puts(" Enter endian  (int)");
       puts("     0= Little Endian host (Linux PC)   ");
       puts("     1= Big    Endian host (IBM Mainframe)");
       scanf(" %d",idirec);
      }

//...convert string to integer
      if(argc>3)
       *compliance=atoi(argv[3]);
       else
      {
       puts(" Enter compliance  (int)");
      puts("  compliance 1= SEGY Compliant (EBCDIC, IBM Float, BigEndian) ");
      puts("             0= (ASCII Reel Header, Float and endian of host) ");
       scanf(" %d",compliance);
      }

//...convert string to integer
      if(argc>4)
       *idirec=atoi(argv[4]);
       else
      {
       puts(" Enter idirec  (int)");
       puts("             0= BSEGY ==>SEGY (new *.sgy)");
       puts("             1= SEGY ==>BSEGY (new *.seg)");
       scanf(" %d",idirec);
      }

//--------ONLY GET REMAINING PARAMETERS IF idirec=0
       if(*idirec==0)
       {
//...convert string to int 
      if(argc>5)
       *idfc=atoi(argv[5]);
       else
      {
       puts(" Enter idfc (int)");
       puts("             1= floating point 4 byte output");
       puts("             2= long integer   4 byte output");
       puts("             3= short integer  2 byte output");
       scanf(" %d",idfc);
      }

//...convert string to int
      if(argc>6)
       *iunits=atoi(argv[6]);
       else
      {
       puts(" Enter iunits (int)");
       puts("             1= meters");
       puts("             2= feet");
       scanf(" %d",iunits);
      }

     if(*idirec==0)
    {

//...pars off character parameters
      if(argc>7)
       strcpy(hedfil,argv[7]);
       else
      {
       puts(" Enter INPUT hedfil (ascii header file name) ");
       puts(" (contains 40 records, 80 char each record, ASCII");
       puts("  3200 bytes");
       scanf(" %13s",hedfil);
      }
	if (namecheck(12,hedfil) != 0 ) return (1);
     }  //endif idirec==0

       }//endif  ONLY IF idirec=0

//...name output files
      if(*idirec==1)
      {names(infile,pid,ofile,sfx1);}
      else
      {names(infile,pid,ofile,sfx3);}
      names(infile,pid,lstfile,sfx2);
       return(0);
     }  //  endif bhelp test
      } // end getparm

/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: outlst.c                                    |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: prints copyright header, file names, and          |
   |          echo check of input parameters in newly           |
   |          opened output listing file.                       |
   |                                                            |
   |------------------------------------------------------------|
*/
      void outlst(int endian, int compliance, int idirec, int idfc,
	 int iunits, char title[], char hedfil[], char infile[],
         char pid[], char ofile[], char lstfile[], FILE *h3)
     {

//...print copyright label to outlst and screen

      fcr_labl(pid,h3);

//...echo check
      fputs("\n",h3);                      //blank line
      fprintf(h3,"\n     PROGRAM: %s.c ",pid);
      fputs("\n",h3);                      //blank line
      fputs("\n     FILES:       ",h3);
      fprintf(h3,"\n     Input   File= %s",infile);
      fprintf(h3,"\n     Output  File= %s",ofile);
      fprintf(h3,"\n     Listing File= %s",lstfile);
      if(idirec==0) fprintf(h3,"\n     Reel Header File= %s",hedfil);
      fputs("\n",h3);                      //blank line
      fputs("\n     INPUT PARAMETERS: ",h3);
      fprintf(h3,"\n     idirec= %5.5d ",idirec);
      fprintf(h3,"\n     endian= %5.5d ",endian);
      fprintf(h3,"\n     compliance= %5.5d ",compliance);
      if(idirec==0)
      {
      fprintf(h3,"\n     idfc= %5.5d ",idfc);
      if(idfc==1) fprintf(h3," (FLOATING POINT 4 byte)");
      if(idfc==2) fprintf(h3," (FIXED POINT 4 byte)");
      if(idfc==3) fprintf(h3," (FIXED POINT 2 byte)");
      fprintf(h3,"\n     iunits= %5.5d ",iunits);
      if(iunits!=1) fprintf(h3," (UNITS=FEET)");
      if(iunits==1) fprintf(h3," (UNITS=METERS)");
      }  //endif
      fputs("\n",h3);                      //blank line
     }  //  end outlst

/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: bsgyin.c                                    |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   | purpose: input the jrec-th trace from input data           |
   |          set in bsegy format.                              |
   |                                                            |
   | input:  h1      =pointer to input file                     |
   |         jrec    =trace number to be input                  |
   |         npts    =number of points in a trace               |
   |         idfc    =data format type                          |
   |                1=floating point 4 byte                     |
   |                2=integer        4 byte                     |
   |                3=integer        2 byte                     | 
   |                                                            |
   | output: hd      =pointer to header structure               |
   |         s2      =pointer to data array                     |
   |                                                            |
   |------------------------------------------------------------|
*/

      int bsgyin(int idfc, int jrec, void  *s2, int npts,
        struct header *hd, FILE *h1)
     {
      int          offset;

      switch (idfc)
     { 
         case 1:
        {
         offset=(240+(int) npts*sizeof(float))*(jrec)+3600;
         break;
        }  //esac 1
         case 2:
        {
         offset=(240+(int) npts*sizeof(int))*(jrec)+3600;
         break;
        }  //esac 2
         case 3:
        {
         offset=(240+(int) npts*sizeof(short))*(jrec)+3600;
         break;
        }  //esac 3
         default:
        {
         offset=(240+(int) npts*sizeof(int))*(jrec)+3600;
         break;
        } //esac default

     }  // endswitch
        if(fseek(h1,offset,SEEK_SET) !=0)
       {
        fputs("ABORT--BSGYIN: can't position indicator",stderr);
        return(1);
       }

//...read header
        if(fread(hd,sizeof(char),240,h1)!=240)
            {
             fputs("ABORT--BSGYIN: Error reading input header ",stderr);
             return(1);
            }

//...read data
      switch (idfc)
     { 
         case 1:
        {
         if(fread(s2,sizeof(float),npts,h1)!=npts)
            {
             fputs("ABORT--BSGYIN: Error reading input data ",stderr);
             return(1);
            }
         break;
        }  //esac 1
         case 2:
        {
         if(fread(s2,sizeof(int),npts,h1)!=npts)
            {
             fputs("ABORT--BSGYIN: Error reading input data ",stderr);
             return(1);
            }
         break;
        }  //esac 2
         case 3:
        {
         if(fread(s2,sizeof(short),npts,h1)!=npts)
            {
             fputs("ABORT--BSGYIN: Error reading input data ",stderr);
             return(1);
            }
         break;
        } //esac 3
         default:
        {
         if(fread(s2,sizeof(int),npts,h1)!=npts)
            {
             fputs("ABORT--BSGYIN: Error reading input data ",stderr);
             return(1);
            }
         break;
        }  //esac default

     }   //endswitch
      return(0);
     }  //  end bsgyin

/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: bsgyout.c                                   |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   | purpose: output the jrec-th trace from data                |
   |          set in SEGY  format.                              |
   |                                                            |
   | input:  h1      =pointer to input file                     |
   |         jrec    =trace number to be input                  |
   |         npts    =number of points in a trace               |
   |         idfc    =data format type                          |
   |                1=floating point 4 byte                     |
   |                2=integer        4 byte                     |
   |                3=integer        2 byte                     | 
   |                                                            |
   | output: hd      =pointer to header structure               |
   |         s2      =pointer to data array                     |
   |                                                            |
   |------------------------------------------------------------|
*/

      int bsgyout(int idfc, int jrec, void  *s2, int npts,
        struct header *hd, FILE *h2)
     {
      int         offset;

      switch (idfc)
     { 
         case 1:
        {
         offset=(240+(int) npts*sizeof(float))*(jrec)+3600;
         break;
        }  //esac 1
         case 2:
        {
         offset=(240+(int) npts*sizeof(int))*(jrec)+3600;
         break;
        }  //esac 2
         case 3:
        {
         offset=(240+(int) npts*sizeof(short))*(jrec)+3600;
         break;
        }  //esac 3
         default:
        {
         offset=(240+(int) npts*sizeof(int))*(jrec)+3600;
         break;
        } //esac default

     }  // endswitch
        if(fseek(h2,offset,SEEK_SET) !=0)
       {
        fputs("ABORT--BSGYOUT: can't position indicator",stderr);
        return(1);
       }

//...write header
        if(fwrite(hd,sizeof(char),240,h2)!=240)
            {
             fputs("ABORT--BSGYOUT: Error writing header ",stderr);
             return(1);
            }

//...write data
      switch (idfc)
     { 
         case 1:
        {
         if(fwrite(s2,sizeof(float),npts,h2)!=npts)
            {
             fputs("ABORT--BSGYOUT: Error writing data ",stderr);
             return(1);
            }
         break;
        }  //esac 1
         case 2:
        {
         if(fwrite(s2,sizeof(int),npts,h2)!=npts)
            {
             fputs("ABORT--BSGYOUT: Error writing data ",stderr);
             return(1);
            }
         break;
        }  //esac 2
         case 3:
        {
         if(fwrite(s2,sizeof(short),npts,h2)!=npts)
            {
             fputs("ABORT--BSGYOUT: Error writing data ",stderr);
             return(1);
            }
         break;
        } //esac 3
         default:
        {
         if(fwrite(s2,sizeof(int),npts,h2)!=npts)
            {
             fputs("ABORT--BSGYOUT: Error writing data ",stderr);
             return(1);
            }
         break;
        }  //esac default

     }   //endswitch
      return(0);
     }  //  end bsgyout
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: seg2sgy_step1.c                             |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose:  Convert SEG-->SGY Step 1, scan the bsegy data    |
   |           set and find max/min values for scaling to avoid |
   |           amplitude scaling problems if data conversion    |
   |           from float to integer is performed at output.    |
   |                                                            |
   |------------------------------------------------------------|
*/
	int seg2sgy_step1(int *ntraces, int *npts,float *fsamin,
	  float *s1, float *xmax, struct header *hd,  FILE *h1   )
{

      int		jrec,jt;
      float		s1max,s1min;
      float		stackf,gaink;
      float		value;

//...trace loop  Find Max and Min in data set
      s1max=-9.99E+30;
      s1min=+9.99E+30;
      
      for (jrec=0;jrec<*ntraces;jrec++)
     {
//...read trace
        if(c_bsegin(jrec,s1,*npts,hd,h1)!=0)
       {
        fprintf(stderr,"ABORT--c_bsegin");
        return(1);
       }
//...convert from microvolts to seismograph number
	if((*hd).gidb>=0)
	{
		gaink=10^((*hd).gidb/20);
	}
	else
	{
		gaink=10^(abs((*hd).gidb)/20);
		gaink=1.0/gaink;
	} //endif
	stackf=(float)(*hd).nvs;

//...find max and min after gain factors applied
	for (jt=0;jt<*npts;jt++)
	{
		value=s1[jt]*gaink*stackf;
		if(value>s1max) s1max=value;
		if(value<s1min) s1min=value;
	} //end for
	*xmax=fabs(s1max);
	if(fabs(s1min)>*xmax) *xmax=fabs(s1min);
	//xmax now has biggest number after regain

     }  //end trace loop
	return(0);
}  //end seg2sgy_step1

/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: seg2sgy_Estep2.c                            |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: write EBCDIC Portion   SEG-Y header to *.sgy file |
   |                                                            |
   |      bytes 0 to 3199                                       |
   |    compliance =1 SEGY STANDARD                             |
   |------------------------------------------------------------|
*/
	int seg2sgy_Estep2(char title[], char hedfil[], 
	  FILE *h2, FILE *h3)
{
	int		jt,
			jrec,
			nchar;
	char		nogo1[5]="none",
	    		nogo2[5]="NONE",
			buff=' ';
	char		cmd[81];
	char		tmpfile[]="bcnv_tmp";
	char		syscmd[81];
//	char		ebcbuf[3201];
	FILE		*h4;
	FILE		*h5; //temp file
//	FILE    	*pipefp; //pipe pointer

//...Write ASCII Portion of REEL Header
	fprintf(h3,"\n ASCII TRANSLATION OF REEL HEADER:\n");
	if( !strcmp(hedfil,nogo1) || !strcmp(hedfil,nogo2) )
	{	
  	  for (jt=0;jt<80;jt++) title[jt]='.';
	  title[0]='C';
	  title[80]='\0';

//...open output temp file
          if( (h5=fopen(tmpfile,"w"))==NULL )
          {
            fprintf(stderr,"\n ABORT !! Error opening file bcnv_tmp \n");
            return(1);
          }

	  for (jrec=0;jrec<40;jrec++)
	  { 
	  if(fwrite(title,sizeof(char),80,h5)!=80)
	  { fprintf(stderr,"ABORT- error writing EBCDIC reel header"); 
	    return(1); }
	    fprintf(h3,"%s\n",title);  //echo check to listing file
	  }  //endfor
	fclose(h5);
	fflush(h5);

	} 
	else
	{
//...open output listing file
          if( (h4=fopen(hedfil,"r"))==NULL )
          {
            fprintf(stderr,"\n ABORT !! Error opening file %s \n",hedfil);
            return(1);
          }
//...open temp file
          if( (h5=fopen(tmpfile,"w"))==NULL )
          {
            fprintf(stderr,"\n ABORT !! Error opening file %s \n",tmpfile);
            return(1);
          }
//..READ REEL HEADER 
//...read input ascii header (40 records, 80char/record)
          for(jrec=0;jrec<40;jrec++)
         {
          nchar=0;
          for (jt=0;jt<81;jt++)
	  {fscanf(h4,"%1c",&title[jt]);
		  if(title[jt]=='\n')
		  {nchar=jt; goto escp;}
	  }//next jt
escp:
	  if(nchar==0)
	  {fprintf(stderr,
	   "WARNING: hedfil lines length>80char jrec=%d\n",jrec+1);
	     while (buff!='\n') fscanf(h4,"%c",&buff); //finish line
	  } //endif
	  for (jt=nchar;jt<80;jt++) title[jt]='.';
         title[80]='\0';
	 title[0]='C';

	  if(fwrite(title,sizeof(char),80,h5)!=80)
	  { fprintf(stderr,"ABORT- error writing EBCDIC reel header"); 
	    return(1); }

	    fprintf(h3,"%s\n",title);  //echo check to listing file
         }   //next jrec
	fclose(h4);
	fclose(h5);
	fflush(h5);
	} //endif

/*
//open pipe and convert ascii to ebcdic header
        sprintf(cmd,"dd if=%s conv=ebcdic cbs=3200 obs=3200", tmpfile);
	if ( (pipefp=popen(cmd,"r") ) == NULL )
 	{
  	fprintf(stderr,"Error opening pipe \n");
  	exit(1);
	}
        fread(ebcbuf,1,3200,pipefp);
        pclose(pipefp);

// Write EBCDIC reel header to ofile
	fwrite(ebcbuf,sizeof(char),3200,h2);
*/
//...open temp file
          if( (h5=fopen(tmpfile,"r"))==NULL )
          {
            fprintf(stderr,"\n ABORT !! Error opening file %s \n",tmpfile);
            return(1);
          }
//scan tmpfile (which is in ascii, no Line Feed A0 characters)
	for(jrec=0; jrec<40; jrec++) {
	fscanf(h5,"%80c",title);
	ascii2ebcdic(cmd,title,80);  //convert to ebcdic 
// Write EBCDIC reel header to ofile
	fwrite(cmd,sizeof(char),80,h2);
	} //next jrec
	fclose(h5);

// remove temp file
	sprintf(syscmd,"rm -rf %s",tmpfile);
	system(syscmd);

	fflush(h2);
	return(0);

} //end seg2sgy_step2


/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: seg2sgy_step2.c                             |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: write ascii portion of SEG-Y header to *.sgy file |
   |                                                            |
   |      bytes 0 to 3199                                       |
   |   h2=binary output file                                    |
   |------------------------------------------------------------|
*/
	int seg2sgy_step2(char title[], char hedfil[],  FILE *h2,
	  FILE *h3)
{
	int		jt,
			jrec,
			nchar;
	char		nogo1[5]="none",
	    		nogo2[5]="NONE",
			buff=' ';
	FILE		*h4;

//...Write ASCII Portion of REEL Header
	fprintf(h3,"\n ASCII REEL HEADER:\n");
	if( !strcmp(hedfil,nogo1) || !strcmp(hedfil,nogo2) )
	{	
  	  for (jt=0;jt<80;jt++) title[jt]=' ';
	  title[0]='C';
	  title[80]='\0';

	  for (jrec=0;jrec<40;jrec++)
	  { 
	  if(fwrite(title,sizeof(char),80,h2)!=80)
	  { fprintf(stderr,"ABORT- error writing ASCII reel header"); 
	    return(1); }
	    fprintf(h3,"%s\n",title);  //echo check to listing file
	  }  //endfor
	} 
	else
	{
//...open output listing file
          if( (h4=fopen(hedfil,"r"))==NULL )
          {
            fprintf(stderr,"\n ABORT !! Error opening file %s \n",hedfil);
            return(1);
          }
//..READ REEL HEADER 
//...read input ascii header (40 records, 80char/record)
          for(jrec=0;jrec<40;jrec++)
         {
          nchar=0;
          for (jt=0;jt<81;jt++)
	  {fscanf(h4,"%1c",&title[jt]);
		  if(title[jt]=='\n')
		  {nchar=jt; goto escp;}
	  }//next jt
escp:
	  if(nchar==0)
	  {fprintf(stderr,
	   "WARNING: hedfil lines length>80char jrec=%d\n",jrec+1);
	     while (buff!='\n') fscanf(h4,"%c",&buff); //finish line
	  } //endif
	  for (jt=nchar;jt<80;jt++) title[jt]=' ';
         title[80]='\0';
	 title[0]='C';

	  if(fwrite(title,sizeof(char),80,h2)!=80)
	  { fprintf(stderr,"ABORT- error writing ASCII reel header"); 
	    return(1); }

	    fprintf(h3,"%s\n",title);  //echo check to listing file
         }   //next jrec
	fclose(h4);
	} //endif
	fflush(h2);
	return(0);

} //end seg2sgy_step2

/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: seg2sgy_Estep3.c                            |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: write binary reel header                          |
   |                                                            |
   |      bytes 3200 to 3599                                    |
   |    compliance =1 version Big Endian   BYTE SWAP (endian=0) |
   |------------------------------------------------------------|
*/
	int seg2sgy_Estep3(float xmax, int ntraces, int npts, 
	short *twfhld,  float fsamin, float *factor, int idfc,
	int iunits,  struct header hd,  FILE *h2, FILE *h3 )
{
	int		jrec,jt;
	short		s1int2;
	char		nullbyte='\0';
     	char            line[5];

	 jrec=3200;

loopbin:	
	if (jrec<3600)
	{
		
	  if(jrec==3216)
	  {   //sample interval in microseconds this data
		s1int2=(short)(fsamin*1.0E+6+.5);
		swap2(&s1int2);
		if(fwrite(&s1int2,sizeof(short),1,h2)!=1) {return(1);}
		jrec=jrec+2;
	  } //endif
	  
	  if(jrec==3218)
	  {   //sample interval in microseconds field data
		s1int2=(short)(fsamin*1.0E+6+.5);
		swap2(&s1int2);
		if(fwrite(&s1int2,sizeof(short),1,h2)!=1) {return(1);}
		jrec=jrec+2;
	  } //endif

	  if(jrec==3220)
	  {   //number of samples per trace, this data    
		s1int2=(short)(npts);              
		swap2(&s1int2);
		if(fwrite(&s1int2,sizeof(short),1,h2)!=1) {return(1);}
		jrec=jrec+2;
	  } //endif
	  
	  if(jrec==3222)
	  {   //number of samples per trace, field data   
		s1int2=(short)(npts);              
		swap2(&s1int2);
		if(fwrite(&s1int2,sizeof(short),1,h2)!=1) {return(1);}
		jrec=jrec+2;
	  } //endif
	  
	  if(jrec==3224)
	  {   /*data format code:  1= 4 byte float
	      	                   2= 4 byte integer
	                           3= 2 byte integer */
		s1int2=(short)(idfc);              
		swap2(&s1int2);
		if(fwrite(&s1int2,sizeof(short),1,h2)!=1) {return(1);}
		jrec=jrec+2;
	  } //endif
	  
	  if(jrec==3212)
	  {   //number of traces per record               
		s1int2=(short)(ntraces);            
		swap2(&s1int2);
		if(fwrite(&s1int2,sizeof(short),1,h2)!=1) {return(1);}
		jrec=jrec+2;
	  } //endif
	  
	  if(jrec==3204)
	  {   //Line ID                                   
		 for(jt=0;jt<4;jt++) line[jt]=hd.lineid[jt];
		swap4(&line);
		if(fwrite(&line,sizeof(char),4,h2)!=4) {return(1);}
		jrec=jrec+4;
	  } //endif
	  
	  if(jrec==3254)
	  {   /*survey units        1=meters  
                                    2=feet */		
		s1int2=(short)(iunits);            
		swap2(&s1int2);
		if(fwrite(&s1int2,sizeof(short),1,h2)!=1) {return(1);}
		jrec=jrec+2;
	  } //endif
	  

	if(fwrite(&nullbyte,sizeof(char),1,h2)!=1) {return(1);}
	jrec=jrec+1;
	goto loopbin;
	
	}  /*endif 
	    
	     END OF LOOPBIN
	   */  
	
//...scale factor if 2 byte integer output
	if(idfc==3)
	{
		*factor=32767./xmax;
	}
	else
	{
		*factor=2147480000./xmax;
		if(*factor>1.0) {*factor=1.0;}
	} //endif

//...set trace weighting factor
	*twfhld=(short) (1.0/(*factor));
	*factor=1.0/( (float) *twfhld);
	fprintf(h3," Scale Factor=%14.7e \n",*factor);
	fprintf(h3," TRACE WEIGHT FACTOR HEADER:  hd.twf=%d \n",*twfhld);
	fflush(h3);
	fflush(h2);
	return(0);
} //end seg2sgy_Estep3


/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: seg2sgy_step3.c                             |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: write binary reel header                          |
   |                                                            |
   |      bytes 3200 to 3599                                    |
   |                                                            |
   |------------------------------------------------------------|
*/
	int seg2sgy_step3(float xmax, int ntraces, int npts, 
	short *twfhld,  float fsamin, float *factor, int idfc,
	int iunits,  struct header hd,  FILE *h2, FILE *h3 )
{
	int		jrec,jt;
	short		s1int2;
	char		nullbyte='\0';
     	char            line[5];

	 jrec=3200;

loopbin:	
	if (jrec<3600)
	{
		
	  if(jrec==3216)
	  {   //sample interval in microseconds this data
		s1int2=(short)(fsamin*1.0E+6+.5);
		if(fwrite(&s1int2,sizeof(short),1,h2)!=1) {return(1);}
		jrec=jrec+2;
	  } //endif
	  
	  if(jrec==3218)
	  {   //sample interval in microseconds field data
		s1int2=(short)(fsamin*1.0E+6+.5);
		if(fwrite(&s1int2,sizeof(short),1,h2)!=1) {return(1);}
		jrec=jrec+2;
	  } //endif

	  if(jrec==3220)
	  {   //number of samples per trace, this data    
		s1int2=(short)(npts);              
		if(fwrite(&s1int2,sizeof(short),1,h2)!=1) {return(1);}
		jrec=jrec+2;
	  } //endif
	  
	  if(jrec==3222)
	  {   //number of samples per trace, field data   
		s1int2=(short)(npts);              
		if(fwrite(&s1int2,sizeof(short),1,h2)!=1) {return(1);}
		jrec=jrec+2;
	  } //endif
	  
	  if(jrec==3224)
	  {   /*data format code:  1= 4 byte float
	      	                   2= 4 byte integer
	                           3= 2 byte integer */
		s1int2=(short)(idfc);              
		if(fwrite(&s1int2,sizeof(short),1,h2)!=1) {return(1);}
		jrec=jrec+2;
	  } //endif
	  
	  if(jrec==3212)
	  {   //number of traces per record               
		s1int2=(short)(ntraces);            
		if(fwrite(&s1int2,sizeof(short),1,h2)!=1) {return(1);}
		jrec=jrec+2;
	  } //endif
	  
	  if(jrec==3204)
	  {   //Line ID                                   
		 for(jt=0;jt<4;jt++) line[jt]=hd.lineid[jt];
		if(fwrite(&line,sizeof(char),4,h2)!=4) {return(1);}
		jrec=jrec+4;
	  } //endif
	  
	  if(jrec==3254)
	  {   /*survey units        1=meters  
                                    2=feet */		
		s1int2=(short)(iunits);            
		if(fwrite(&s1int2,sizeof(short),1,h2)!=1) {return(1);}
		jrec=jrec+2;
	  } //endif
	  

	if(fwrite(&nullbyte,sizeof(char),1,h2)!=1) {return(1);}
	jrec=jrec+1;
	goto loopbin;
	
	}  /*endif 
	    
	     END OF LOOPBIN
	   */  
	
//...scale factor if 2 byte integer output
	if(idfc==3)
	{
		*factor=32767./xmax;
	}
	else
	{
		*factor=2147480000./xmax;
		if(*factor>1.0) {*factor=1.0;}
	} //endif

//...set trace weighting factor
	*twfhld=(short) (1.0/(*factor));
	*factor=1.0/( (float) *twfhld);
	fprintf(h3," Scale Factor=%14.7e \n",*factor);
	fprintf(h3," TRACE WEIGHT FACTOR HEADER:  hd.twf=%d \n",*twfhld);
	fflush(h3);
	fflush(h2);
	return(0);
} //end seg2sgy_step3

/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: seg2sgy_step4.c                             |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose:  dynamic allocation of trace memory for user      |
   |           specified data type                              |
   |                                                            |
   |------------------------------------------------------------|
*/
//...dynamic allocation of seimic trace memory
	void *  seg2sgy_step4(int idfc, int npts )
{
	void		*s2;
	      switch (idfc)
     { 
         case 1:
        {
	//return((float *)s2=calloc(npts,sizeof(float)) );
	return(s2=calloc(npts,sizeof(float)) );
         break;
        }  //esac 1
         case 2:
        {
	//return((int *)s2=calloc(npts,sizeof(int)) );
	return(s2=calloc(npts,sizeof(int)) );
         break;
        }  //esac 2
         case 3:
        {
	//return((short *)s2=calloc(npts,sizeof(short)) );
	return(s2=calloc(npts,sizeof(short)) );
         break;
        }  //esac 3
	default:
	{
		return(NULL);
	}

     }  //end switch
}  //end seg2sgy_step4

/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: seg2sgy_step5.c                             |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: load output trace with correct data conversion    |
   |                                                            |
   |------------------------------------------------------------|
*/
	void seg2sgy_step5(int npts, float *s1, void *s2, int idfc, 
        	 struct header hd  )
{
	int		jt;
	float		gaink,
			stackf,
			factor,
			twfactor;

	//convert from microvolts to seismograph number
	if(hd.gidb>=0)
	{
		gaink=10^(hd.gidb/20);
	}
	else
	{
		gaink=10^(abs(hd.gidb)/20);
		gaink=1.0/gaink;
	} //endif
	stackf=(float)hd.nvs;
	twfactor=(float)hd.twf;
	if (idfc != 1 ) {
	factor=(gaink*stackf)/twfactor; }
	else {
	factor=1.0; }

	//scale data for output
	for (jt=0;jt<npts;jt++)
	{
		s1[jt]=s1[jt]*factor;
	}  //endfor trace

//...load output trace	
	      switch (idfc)
     { 
         case 1:
        {
		for(jt=0;jt<npts;jt++) {((float *)s2)[jt]=(float) s1[jt]; }
                break;
        }  //esac 1

         case 2:
        {
		for(jt=0;jt<npts;jt++) {((int *)s2)[jt]=(int) s1[jt]; }
                break;
        }  //esac 2
  
         case 3:
        {
		for(jt=0;jt<npts;jt++) {((short *)s2)[jt]=(short) s1[jt]; }
                break;
        }  //esac 3
     }  //endswitch
}  //end seg2sgy_step5

/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: sgy2seg_Estep1.c                            |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: Read the ASCII portion of the reel header         |
   |                                                            |
   |------------------------------------------------------------|
*/
	int  sgy2seg_Estep1(char title[], char sgyfile[], FILE *h3)
{
	int		j;
	char		cmd[82];
	FILE    	*fp; // pointer
	
//...read input ascii header (40 records, 80char/record)
//   (bytes 1 - 3200) position indicator runs from 0 to n-1
//open stream and convert ascii to ebcdic header

/*  	sprintf(cmd,"dd ibs=1 if=%s conv=ascii count=3200 1>tmpfile 2>/dev/null \0",sgyfile);
	system(cmd);
	fp=fopen("tmpfile","r");
*/
	fp=fopen(sgyfile,"r");
        fputs("\n ASCII Translation of EBCDIC REEL HEADER:\n",h3);
        for(j=0;j<40;j++)
       {
        if(fscanf(fp,"%80c",title)!=1) {return(1);}
	ebcdic2ascii(cmd,title,80);
        cmd[80]='\0';
        fprintf(h3,"%s\n",cmd);
       }   //next j
 //	 pclose(pipefp);
	fclose(fp);
//	system("rm -r -f tmpfile");
	return(0);
}  //end sgy2seg_Estep1

/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: sgy2seg_Estep2.c                            |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose:  read binary reel header of SEG-Y data            |
   |                                                            |
   |------------------------------------------------------------|
*/
	int  sgy2seg_Estep2(int *npts, int *idfc, int *ntraces,
	   int *iunits,	float *fsamin, FILE *h1, FILE *h3 )
{
	long int	fpos;
        short		i2byte;
	int		nbyte;

//...read input binary header (bytes 3201 - 3600)
//...Read number of samples / trace
        if( fseek(h1,3220,SEEK_SET)!=0)
        { fputs("ABORT..Can't Position pointer to 3220byte",stderr);
          return(1);
        }
        fread(&i2byte,sizeof(short),1,h1);
	swap2(&i2byte);
        *npts=i2byte;
        fprintf(h3,"\n  Number of samples / trace = %d",*npts);

//...Read data format type (integer or floating point)
        if( fseek(h1,3224,SEEK_SET)!=0)
        { fputs("ABORT..Can't Position pointer to 3224byte",stderr);
          return(1);
        }
        fread(&i2byte,sizeof(short),1,h1);
	swap2(&i2byte);
        nbyte=4;
        if(i2byte==1) fprintf(h3,"\n Floating Point 4 byte data");
        if(i2byte==2) fprintf(h3,"\n Fixed Point    4 byte data");
        if(i2byte==3)
       {   fprintf(h3,"\n Fixed Point    2 byte data");
           nbyte=2;
       }  //endif
        *idfc=i2byte;
	fprintf(h3,"\n Data Format Code: idfc  =%d ",*idfc  );

//...Determine the length of the file in bytes
        fseek(h1,0,SEEK_END);
        fpos=ftell(h1);
        fprintf(h3,"\n bytes in file= %ld",fpos);

//...Compute number of traces
        *ntraces=(fpos-3600)/(240+nbyte*(*npts));
        fprintf(h3,"\n number of traces = %d",*ntraces); 
//...Read units
        if( fseek(h1,3254,SEEK_SET)!=0)
        { fputs("ABORT..Can't Position pointer to 3254byte",stderr);
          return(1);   
        }
        fread(&i2byte,sizeof(short),1,h1);
	swap2(&i2byte);
        *iunits=i2byte;
	fprintf(h3,"\n iunits=%d ",*iunits);
	if(*iunits==1) 
		fprintf(h3,"  METERS");
	else
		fprintf(h3,"  FEET  ");

//...Read sample interval
        if( fseek(h1,3216,SEEK_SET)!=0)
        { fputs("ABORT..Can't Position pointer to 3216byte",stderr);
          return(1);   
        }
        fread(&i2byte,sizeof(short),1,h1);
	swap2(&i2byte);
        *fsamin=(float)i2byte * 1.E+06;
        fprintf(h3,"\n Sample Interval = %14.3e microsec",*fsamin);

//...position at byte 3601
        if( fseek(h1,3600,SEEK_SET)!=0)
        { fputs("ABORT..Can't Position pointer to 3600byte",stderr);
          return(1);   
        }
	return(0);
}  //end sgy2seg_Estep2
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: sgy2seg_step1.c                             |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: Read the ASCII portion of the reel header         |
   |                                                            |
   |------------------------------------------------------------|
*/
	int  sgy2seg_step1(char title[], FILE *h1, FILE *h3)
{
	int		j;
//...read input ascii header (40 records, 80char/record)
//   (bytes 1 - 3200) position indicator runs from 0 to n-1
        fputs("\n ASCII REEL HEADER:\n",h3);
        for(j=0;j<40;j++)
       {
        if(fscanf(h1,"%80c",title)!=1) {return(1);}
        title[80]='\0';
        fprintf(h3,"%s\n", title);
       }   //next j
	return(0);
}  //end sgy2seg_step1

/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: sgy2seg_step2.c                             |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose:  read binary reel header of SEG-Y data            |
   |                                                            |
   |------------------------------------------------------------|
*/
	int  sgy2seg_step2(int *npts, int *idfc, int *ntraces,
	   int *iunits,	float *fsamin, FILE *h1, FILE *h3 )
{
	long int	fpos;
        short		i2byte;
	int		nbyte;

//...read input binary header (bytes 3201 - 3600)
//...Read number of samples / trace
        if( fseek(h1,3220,SEEK_SET)!=0)
        { fputs("ABORT..Can't Position pointer to 3220byte",stderr);
          return(1);
        }
        fread(&i2byte,sizeof(short),1,h1);
        *npts=i2byte;
        fprintf(h3,"\n  Number of samples / trace = %d",*npts);

//...Read data format type (integer or floating point)
        if( fseek(h1,3224,SEEK_SET)!=0)
        { fputs("ABORT..Can't Position pointer to 3224byte",stderr);
          return(1);
        }
        fread(&i2byte,sizeof(short),1,h1);
        nbyte=4;
        if(i2byte==1) fprintf(h3,"\n Floating Point 4 byte data");
        if(i2byte==2) fprintf(h3,"\n Fixed Point    4 byte data");
        if(i2byte==3)
       {   fprintf(h3,"\n Fixed Point    2 byte data");
           nbyte=2;
       }  //endif
        *idfc=i2byte;
	fprintf(h3,"\n Data Format Code: idfc  =%d ",*idfc  );

//...Determine the length of the file in bytes
        fseek(h1,0,SEEK_END);
        fpos=ftell(h1);
        fprintf(h3,"\n bytes in file= %ld",fpos);

//...Compute number of traces
        *ntraces=(fpos-3600)/(240+nbyte*(*npts));
        fprintf(h3,"\n number of traces = %d",*ntraces); 
//...Read units
        if( fseek(h1,3254,SEEK_SET)!=0)
        { fputs("ABORT..Can't Position pointer to 3254byte",stderr);
          return(1);   
        }
        fread(&i2byte,sizeof(short),1,h1);
        *iunits=i2byte;
	fprintf(h3,"\n iunits=%d ",*iunits);
	if(*iunits==1) 
		fprintf(h3,"  METERS");
	else
		fprintf(h3,"  FEET  ");

//...Read sample interval
        if( fseek(h1,3216,SEEK_SET)!=0)
        { fputs("ABORT..Can't Position pointer to 3216byte",stderr);
          return(1);   
        }
        fread(&i2byte,sizeof(short),1,h1);
        *fsamin=(float)i2byte * 1.E+06;
        fprintf(h3,"\n Sample Interval = %14.3e microsec",*fsamin);

//...position at byte 3601
        if( fseek(h1,3600,SEEK_SET)!=0)
        { fputs("ABORT..Can't Position pointer to 3600byte",stderr);
          return(1);   
        }
	return(0);
}  //end sgy2seg_step2


/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: sgy2seg_step3.c                             |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: dynamic allocation of trace memory by             |
   |          data type from reel header                        |
   |                                                            |
   |------------------------------------------------------------|
*/
	void * sgy2seg_step3(int npts, int idfc)
{
	void *s3;
//...dynamic allocation of seimic trace memory
	      switch (idfc)
     { 
         case 1:
        {
	//return((float *)s3=calloc(npts,sizeof(float)));
	return(s3=calloc(npts,sizeof(float)));
         break;
        }  //esac 1
         case 2:
        {
	//return((int *)s3=calloc(npts,sizeof(int)));
	return(s3=calloc(npts,sizeof(int)));
         break;
        }  //esac 2
         case 3:
        {
	//return((short *)s3=calloc(npts,sizeof(short)));
	return(s3=calloc(npts,sizeof(short)));
         break;
        }  //esac 3
	 default:
	{ return(NULL); }

     }  //end switch

} //end sgy2seg_step3

/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: sgy2seg_step4.c                             |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose:  convert from seismograph number to microvolts    |
   |                                                            |
   |------------------------------------------------------------|
*/
	void  sgy2seg_step4(int idfc, int npts, void *s2, void *s3, 
		struct header hd )
{
	float		gaink,
			stackf,
			factor;
	int		jt;

	//convert from seismograph number to microvolts
	if(hd.gidb>=0)
	{
		gaink=10^(hd.gidb/20);
	}
	else
	{
		gaink=10^(abs(hd.gidb)/20);
		gaink=1.0/gaink;
	} //endif
	stackf=(float)hd.nvs;

//...copy data to output trace
	if (idfc !=1 ) {
	factor=(gaink*stackf)/((float)hd.twf); }
	else {
	factor=1.0; }
        if (idfc==1)
        for(jt=0;jt<npts;jt++)
       	{((float *)s2)[jt]=((float) ((float *)s3)[jt])/factor; }
        if (idfc==2)
        for(jt=0;jt<npts;jt++)
       	{((float *)s2)[jt]=((float) ((int   *)s3)[jt])/factor; }
        if (idfc==3)
        for(jt=0;jt<npts;jt++)
       	{((float *)s2)[jt]=((float) ((short *)s3)[jt])/factor; }
}  //end sgy2seg_step4

/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: swap2.c  Swap a 2 byte variable             |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | Byte swap for endian conversion                            |
   |------------------------------------------------------------|
*/
/* 2 byte  swap  PM */
void swap2(void *x2)
{
unsigned char *tmp=(unsigned char *) x2;
unsigned char x;
x=tmp[0];
tmp[0]=tmp[1];
tmp[1]=x;
} //end swap2

/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: swap4.c  Swap a 4 byte variable             |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | Byte swap for endian conversion                            |
   |------------------------------------------------------------|
*/
/* 4 byte  swap  PM */
void swap4(void *x4)
{
unsigned char *tmp=(unsigned char *) x4;
unsigned char x;
x=tmp[0];
tmp[0]=tmp[3];
tmp[3]=x;
x=tmp[1];
tmp[1]=tmp[2];
tmp[2]=x;
} //end swap4
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: sgyformat.c                                 |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   | purpose: output the jrec-th trace from data                |
   |          set in SEGY  format.                              |
   |                                                            |
   | input:  h1      =pointer to input file                     |
   |         endian  =0 lendian 1=bendian                       |
   |         compliance  1=Strict SEGY, 0=ASCII and native fmt  |
   |         npts    =number of points in a trace               |
   |         idfc    =data format type                          |
   |                1=floating point 4 byte                     |
   |                2=integer        4 byte                     |
   |                3=integer        2 byte                     | 
   |                                                            |
   | output: hd      =pointer to header structure               |
   |         s2      =pointer to data array                     |
   |                                                            |
   |------------------------------------------------------------|
*/

      void sgyformat(int idfc, int endian, int compliance, void  *s2,
      int npts, struct header *hd, FILE *h2)
     {
	int		i4;
	short int	i2;
	void		*ptr;
	int		j;
	unsigned int	dst;
//	int		j4;

// Format 240 byte header
	if(endian==0 && compliance==1) {
//initialize pointer
	ptr=hd;

//loop through header and swap
	/* 4 byte section*/
	for (j=0; j<28; j=j+4) {
	memcpy(&i4,(ptr + j),4);
	swap4(&i4);	
	memcpy((ptr+j),&i4,4);
	} //next j
	/* 2 byte section*/
	for (j=28; j<36; j=j+2) {
	memcpy(&i2,(ptr + j),2);
	swap2(&i2);	
	memcpy((ptr+j),&i2,2);
	} //next j
	/* 4 byte section*/
	for (j=36; j<68; j=j+4) {
	memcpy(&i4,(ptr + j),4);
	swap4(&i4);	
	memcpy((ptr+j),&i4,4);
	} //next j
	/* 2 byte section*/
	for (j=68; j<72; j=j+2) {
	memcpy(&i2,(ptr + j),2);
	swap2(&i2);	
	memcpy((ptr+j),&i2,2);
	} //next j
	/* 4 byte section*/
	for (j=72; j<88; j=j+4) {
	memcpy(&i4,(ptr + j),4);
	swap4(&i4);	
	memcpy((ptr+j),&i4,4);
	} //next j
	/* 2 byte section*/
	for (j=88; j<180; j=j+2) {
	memcpy(&i2,(ptr + j),2);
	swap2(&i2);	
	memcpy((ptr+j),&i2,2);
	} //next j
	/* 4 byte section*/
	for (j=180; j<184; j=j+4) {
	memcpy(&i4,(ptr + j),4);
	swap4(&i4);	
	memcpy((ptr+j),&i4,4);
	} //next j
	/* 2 byte section*/
	for (j=184; j<208; j=j+2) {
	memcpy(&i2,(ptr + j),2);
	swap2(&i2);	
	memcpy((ptr+j),&i2,2);
	} //next j
	j=208;
	memcpy(&i4,(ptr + j),4);
	swap4(&i4);	
	memcpy((ptr+j),&i4,4);
// end of header swap bsegy

// Convert data trace according to type (float, int4, int2)
	switch (idfc) {
	case 1: {
		for (j=0; j<npts; j++) {
		ptr=s2+j*4;
		ConvertIEEEToFloat(ptr, &dst) ;
		swap4(&dst);
		memcpy(ptr,&dst,4);
		} //next j
	break;
	} //esac 1
	case 2: {
		for (j=0; j<npts; j++) {
		ptr=s2+j*4;
		i4=*(int *)ptr ;
		swap4(&i4);
		memcpy(ptr,&i4,4);
		} //next j
	break;
	} //esac 2

	case 3: {
		for (j=0; j<npts; j++) {
		ptr=s2+j*2;
		i2=*(int *)ptr ;
		swap2(&i2);
		memcpy(ptr,&i2,2);
		} //next j
	break;
	} //esac 3


	} //end switch

	} //endif



     } //end sgyformat
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: segformat.c                                 |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   | purpose: output the jrec-th trace from data                |
   |          set in BSEGY  format.                             |
   |                                                            |
   | input:  h1      =pointer to input file                     |
   |         endian  =0 lendian 1=bendian                       |
   |         compliance  1=Strict SEGY, 0=ASCII and native fmt  |
   |         npts    =number of points in a trace               |
   |         idfc    =data format type                          |
   |                1=floating point 4 byte                     |
   |                2=integer        4 byte                     |
   |                3=integer        2 byte                     | 
   |                                                            |
   | output: hd      =pointer to header structure               |
   |         s2      =pointer to data array                     |
   |                                                            |
   |------------------------------------------------------------|
*/

      void segformat(int idfc, int endian, int compliance, void  *s2,
      int npts, struct header *hd, FILE *h2)
     {
	int		i4;
	short int	i2;
	void		*ptr;
	int		j;
	unsigned int	dst;
//	int		j4;

// Format 240 byte header
	if(endian==0 && compliance==1) {
//initialize pointer
	ptr=hd;

//loop through header and swap
	/* 4 byte section*/
	for (j=0; j<28; j=j+4) {
	memcpy(&i4,(ptr + j),4);
	swap4(&i4);	
	memcpy((ptr+j),&i4,4);
	} //next j
	/* 2 byte section*/
	for (j=28; j<36; j=j+2) {
	memcpy(&i2,(ptr + j),2);
	swap2(&i2);	
	memcpy((ptr+j),&i2,2);
	} //next j
	/* 4 byte section*/
	for (j=36; j<68; j=j+4) {
	memcpy(&i4,(ptr + j),4);
	swap4(&i4);	
	memcpy((ptr+j),&i4,4);
	} //next j
	/* 2 byte section*/
	for (j=68; j<72; j=j+2) {
	memcpy(&i2,(ptr + j),2);
	swap2(&i2);	
	memcpy((ptr+j),&i2,2);
	} //next j
	/* 4 byte section*/
	for (j=72; j<88; j=j+4) {
	memcpy(&i4,(ptr + j),4);
	swap4(&i4);	
	memcpy((ptr+j),&i4,4);
	} //next j
	/* 2 byte section*/
	for (j=88; j<180; j=j+2) {
	memcpy(&i2,(ptr + j),2);
	swap2(&i2);	
	memcpy((ptr+j),&i2,2);
	} //next j
	/* 4 byte section*/
	for (j=180; j<184; j=j+4) {
	memcpy(&i4,(ptr + j),4);
	swap4(&i4);	
	memcpy((ptr+j),&i4,4);
	} //next j
	/* 2 byte section*/
	for (j=184; j<208; j=j+2) {
	memcpy(&i2,(ptr + j),2);
	swap2(&i2);	
	memcpy((ptr+j),&i2,2);
	} //next j
	j=208;
	memcpy(&i4,(ptr + j),4);
	swap4(&i4);	
	memcpy((ptr+j),&i4,4);
// end of header swap bsegy

// Convert data trace according to type (float, int4, int2)
	switch (idfc) {
	case 1: {
		for (j=0; j<npts; j++) {
		ptr=s2+j*4;
		swap4(ptr);
		ConvertFloatToIEEE(ptr, &dst) ;
		memcpy(ptr,&dst,4);
		} //next j
	break;
	} //esac 1
	case 2: {
		for (j=0; j<npts; j++) {
		ptr=s2+j*4;
		i4=*(int *)ptr ;
  		swap4(&i4);
		memcpy(ptr,&i4,4);
		} //next j
	break;
	} //esac 2

	case 3: {
		for (j=0; j<npts; j++) {
		ptr=s2+j*2;
		i2=*(int *)ptr ;
		swap2(&i2);
		memcpy(ptr,&i2,2);
		} //next j
	break;
	} //esac 3


	} //end switch

	} //endif

     } //end segformat
