// |$Id: bdat.c,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $          |
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              |
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS    http://cgiss.boisestate.edu/~pm     | 
   | FUNCTION NAME: bdat.c                                      |
   | DIRECTORY: bsu-2.0.2/src/C/bsegy                           |
   |------------------------------------------------------------|
   | purpose: datuming for refraction data                      |
   |                                                            |
   | sz-sd=shot elevation = datum                               |
   | rz=receiver elevation                                      |
   | vw=weathered zone velocity                                 |
   | static to add =(sz-sd-rz)/vw                               |
   | NOTE: This is to be applied to a shot record only          |
   |       (local datum is the ground surface at the shot, sz)  |
   |       Shot + Geophone static adjusts traces as though the  |
   |       geophone and charge were both at sz.  Can make pick  |
   |       of first arrivals easier.                            |
   |       First break picks are adjusted by the static shift.  |
   |       (so you can pick shifted data, then unshift, and     |
   |        picks will be corrected back to the original datum).|
   |                                                            |
   | iapply  +1=apply static  -1=remove static                  |
   |------------------------------------------------------------|
   |  NOTICE:                                                   |
   |  Copyright (C) 2017 Paul Michaels                          |
   |  <pm@cgiss.boisestate.edu>                                 |
   |  This program is free software; you can                    |
   |  redistribute it and/or modify it under the terms          |
   |  of the GNU General Public License as published            |
   |  by the Free Software Foundation; either version           |
   |  3 of the License, or (at your option) any later           |
   |  version.  This program is distributed in the              |
   |  hope that it will be useful, but                          |
   |  WITHOUT ANY WARRANTY; without even the implied            |
   |  warranty of MERCHANTABILITY or FITNESS FOR A              |
   |  PARTICULAR PURPOSE.  See the GNU General Public           |
   |  License for more details.                                 |
   |  You should have received a copy of the GNU                |
   |  General Public License along with this program;           |
   |  if not, write to the Free Software Foundation,            |
   |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
   |------------------------------------------------------------|
   |INLINE FUNCTIONS:                                           |
   |  getparm    get parameters from command line or interactive|
   |  outlst     echo check of input parameters to listing file |
   |  valxx      interpolate a sample at a time of interest     |
   |  vshft      calls valxx to apply static shift to a trace   |
   |                                                            |
   |------------------------------------------------------------|
   |sublibC4 functions:                                         |
   |  c_bsegin   read seismic trace                             |
   |  c_bsegout  write a seismic trace                          |
   |  in_chk     scan input file for samples, number traces etc.|
   |  bargrid    write progress bar grid to screen              |
   |  exbar      write progress bar to screen                   |
   |  cr_labl    write copyright label to screen                |
   |  fcr_labl   write copyright label to file                  |
   |  names      create an output file name from input file name|
   |                                                            |
   |------------------------------------------------------------|
*/
      #include <stdio.h>
      #include <stdlib.h>
      #include <float.h>
      #include <math.h>
      #include <string.h>
      #include <time.h>
      #include "../include/c_bsegy.h"

//...parameter definitions

//...function prototypes
      #include "../include/sub4.h"

      int  getparm(int argc,char *argv[],
         float *vel, int *iapply, char infile[],
         char ofile[], char lstfile[], char datfile[], char pid[]);

      void outlst(float vel, int iapply, char infile[],
        char pid[], char ofile[], char lstfile[], FILE *h3);

      float valxx(float tpic, float s1[], float fsamin);

      void vshft(int npts, float s1[], float s2[], float stat,
	float fsamin);

      int  main(int argc, char *argv[])
     {

//...declare variables
      FILE              *h1, *h2, *h3, *h4;
      int               jrec,
                        npts,
                        ntraces,
                        ibar,
                        invbar;
      float             fsamin,
                        bar,
                        *s1,
                        *s2;
      char              pid[5]="bdat",   //process id name
                        ofile[13],
                        lstfile[13],
			datfile[13],
                        infile[14];
      struct header     hd;

//...specific to this application
      int               iapply;		//switch
      float             vel;		//datuming velocity
      float		fbrk;		//first break time
      float		sx,sy,sz,sd,
			rx,ry,rz,range;
      float		stat,nstat;
      char		rcvn[5]="    ",
			shtn[5]="    ";

//...get parameters
      if(getparm(argc,argv,&vel,
         &iapply,infile,ofile,lstfile,datfile,pid)!=0)
      {
       goto quick_exit;
      }

//...open output listing file
        if( (h3=fopen(lstfile,"w"))==NULL )
        {
          fprintf(stderr,"\n ABORT !! Error opening file %s",lstfile);
          goto quick_exit;
        }

//...open output dat file
        if( (h4=fopen(datfile,"w"))==NULL )
        {
          fprintf(stderr,"\n ABORT !! Error opening file %s",lstfile);
          goto quick_exit;
        }

//...echo check to listing file
      outlst(vel,iapply,infile,pid,ofile,lstfile,h3);

//...open input file
        if( (h1=fopen(infile,"rb"))==NULL )
        {
          fprintf(stderr,"\n ABORT !! Error opening file %s",infile);
          goto exit_3;
        }

//...open output file
        if( (h2=fopen(ofile,"wb"))==NULL )
        {
          fprintf(stderr,"\n ABORT !! Error opening file %s",ofile);
          goto exit_1;
        }


//...check input file size
          if(in_chk(&ntraces,&npts,&fsamin,hd,h1)!=0)
         {
          goto exit_2;
         }

//...print input file data to lstfile
        fprintf(h3,"\n     Number of samples= %d",npts);
        fprintf(h3,"\n     Sample Interval = %6.5f sec.",fsamin);
        fprintf(h3,"\n     Number of traces = %d",ntraces);

//...dynamic allocation of seimic trace memory
        s1=calloc(npts,sizeof(float));
          if(s1==NULL)
         {
          fprintf(stderr,"ABORT:s1 can't allocate enough trace memory");
          fprintf(stderr," npts = %d",npts);
          goto exit_2;
         }
        s2=calloc(npts,sizeof(float));
          if(s2==NULL)
         {
          fprintf(stderr,"ABORT:s2 can't allocate enough trace memory");
          fprintf(stderr," npts = %d",npts);
          goto exit_s1;
         }
//-----------------main loop---------------------------
//...display progress bar grid

      bargrid(ntraces,&bar,&ibar,&invbar);

//...print column headers
	fprintf(h3,"\n");
	fprintf(h3,"     Static (s)   Elev_Rec  Elev_Sht  Rec_Name  Sht_Name\n");

//...trace loop

      for (jrec=0;jrec<ntraces;jrec++)
     {

//...read trace
        if(c_bsegin(jrec,s1,npts,&hd,h1)!=0)
       {
        fprintf(stderr,"ABORT--c_bsegin");
        goto quit_it;
       }

//...compute statics
	findxyz(&sx,&sy,&sz,&sd,&rx,&ry,&rz,&range,hd);
	stat= ((float)(iapply))*((sz-sd-rz)/vel);

//...list statics to outlist
	memcpy(&shtn,&hd.spname,4); shtn[4]='\0';
	memcpy(&rcvn,&hd.rstnam,4); rcvn[4]='\0';

	fprintf(h3,"      %8.4f   %8.2f  %8.2f    %s      %s \n",
	stat,rz,(sz-sd),rcvn,shtn);
	fprintf(h4," %6.1f   %8.5f \n",(float) (jrec+1),stat);

//...apply statics
	nstat=-stat;
        vshft(npts,s1,s2,nstat,fsamin);
	
//...adjust first break picks
	fbrk=((float) hd.mst)*1.0E-06;  //convert from microseconds
	fbrk=fbrk+stat;
	hd.mst=(long) (fbrk*1.0E+06);
	
//...output trace
        if(c_bsegout(jrec,s2,npts,&hd,h2)!=0)
       {
        fprintf(stderr,"ABORT--bsegout");
        goto quit_it;
       }

//...display progress bar
      exbar(ntraces,bar,ibar,invbar,jrec+1);

     } //  end trace loop-----------------------------

      printf("\n   Listing ====>%s",lstfile);
      printf("\n   Output  ====>%s",ofile);
      printf("\n   Statics  ====>%s",datfile);
      printf("\n   Processed %d traces",ntraces);
	return(0);
quit_it:
        puts(" ");

        free(s2);
exit_s1:
        free(s1);
exit_2:
        fclose(h2);
exit_1:
        fclose(h1);
exit_3:
        fclose(h3);
        fclose(h4);

quick_exit:
	return(1);

     } //  end of main

/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: getparm.c                                   |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: gets input parameters from command                |
   |          line or prompts user from consol.                 |
   |                                                            |
   |                                                            |
   |------------------------------------------------------------|
*/

      int  getparm(int argc,char *argv[],
         float *vel, int *iapply, char infile[],
         char ofile[], char lstfile[], char datfile[], char pid[])
     {
      char              sfx1[4]="seg",
                        sfx2[4]="lst",
			sfx3[4]="dat";
      char		help[3]="-h";
      char              msg1[46],msg2[46];

//...test for command line help ?
      if(argc==2 && !strcmp(argv[1],help))
     {
//...screen header and copyright notice
      cr_labl(pid);

//...help pannel
      sprintf(msg1,"Datuming program for refraction data. Adjust ");
      sprintf(msg2,"data so that geoph. and charge at surface, sz");
      hlp_labl(msg1,msg2);
      puts(" ");

//...help display
      puts(" ");
      printf("  %s  infile vel iapply\n",pid);
      puts(" ");
      puts("     infile   =  input file name");
      puts("     vel      =  velocity of overburden");
      puts("     iapply   =  switch");
      puts("             1=  apply static to datum ");
      puts("            -1=  remove static, restore to observed ");
      puts(" ");
      puts(" NOTES: 1 Statics are computed from geometry in headers.  ");
      puts("         The receiver is datumed to the shot elevation. ");
      puts(" ");
      puts("        2 Intended use is to make recognition of first  ");
      puts("         arrivals easier, and picking easier.");
      puts(" ");
      puts("        3 First break pick headers are adjusted with ");
      puts("         each datum change ");
      return(1);
     }
      else
     {
//...get parameters------------------------------
//...pars off character parameters
      if(argc>1)
       strncpy(infile,argv[1],13);
       else
      {
       puts(" Enter INPUT file name ");
       scanf(" %13s",infile);
      }
	if (namecheck(12,infile) != 0 ) return (1);

//...convert string to float
      if(argc>2)
       *vel=atof(argv[2]);
       else
      {
       puts(" Enter overburden velocity (float)");
       scanf(" %f",vel);
      }

//...convert string to integer
      if(argc>3)
       *iapply=atoi(argv[3]);
       else
      {
       puts(" Enter iapply (int)");
      puts("             1=  apply static to datum ");
      puts("            -1=  remove static, restore to observed ");
       scanf(" %d",iapply);
      }
	if(*iapply<0)
	 *iapply=-1;
	 else
	 *iapply=+1;

//...name output files
      names(infile,pid,ofile,sfx1);
      names(infile,pid,lstfile,sfx2);
      names(infile,pid,datfile,sfx3);
       return(0);
     }  //  endif bhelp test
      } // end getparm

/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: outlst.c                                    |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: prints copyright header, file names, and          |
   |          echo check of input parameters in newly           |
   |          opened output listing file.                       |
   |                                                            |
   |------------------------------------------------------------|
*/

      void outlst(float vel, int iapply, char infile[],
        char pid[], char ofile[], char lstfile[], FILE *h3)
     {

//...print copyright label to outlst and screen

      fcr_labl(pid,h3);

//...echo check
      fputs("\n",h3);                      //blank line
      fprintf(h3,"\n     PROGRAM: %s.c ",pid);
      fputs("\n",h3);                      //blank line
      fputs("\n     FILES:       ",h3);
      fprintf(h3,"\n     Input   File= %s",infile);
      fprintf(h3,"\n     Output  File= %s",ofile);
      fprintf(h3,"\n     Listing File= %s",lstfile);
      fputs("\n",h3);                      //blank line
      fputs("\n     INPUT PARAMETERS: ",h3);
      fprintf(h3,"\n    Overburden velocity= %7.2f",vel);
      fprintf(h3,"\n     iapply=%5d ",iapply);
      if(iapply==1) fprintf(h3,"Shift data to datum");
      if(iapply==-1) fprintf(h3,"Remove static, restore data");
      fputs("\n",h3);                      //blank line
     }  //  end outlst

/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: valxx.c                                     |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: interpolates a sample                             |
   |                                                            |
   |------------------------------------------------------------|
*/

      float valxx(float tpic,float s1[],float fsamin)
     {
	float		fi,val;
	int		i,j;

	fi=tpic/fsamin+1.;
	i=floor(fi);	
	j=i+1;
	val=((s1[j]-s1[i])/fsamin)*(tpic-(float)(i-1)*fsamin)+s1[i];
	return(val);
     }  //end valxx

/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: vshft.c                                     |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: apply static shift by interpolation               |
   |                                                            |
   |------------------------------------------------------------|
*/

      void vshft(int npts, float s1[], float s2[], float stat,
	float fsamin)
      {
	int		jt;
	float		tpic,tmax;

//...clear output trace
	for (jt=0;jt<npts;jt++) s2[jt]=0.0;

//...apply static, stat
	tmax=((float)(npts-1))*fsamin;
	for (jt=0;jt<npts;jt++)
	{
	 tpic=((float)(jt-1))*fsamin+stat;
	   if(tpic<tmax && tpic>=0.0)
	   { s2[jt]=valxx(tpic,s1,fsamin);}
	   else
	   { s2[jt]=0.0; }  //endif
	}  //next jt
      }  //end vshft
