// |$Id: bfit.c,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $              |
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              |
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS    http://cgiss.boisestate.edu/~pm     | 
   | FUNCTION NAME: bfit.c                                      |
   | DIRECTORY: bsu-2.0.2/src/C/bsegy                           |
   | MODIFIED: 20 June 2002  Changed the bracket method for     |
   |           selecting data to fit.  Bracket points now are   |
   |           points closest to specified points (permits a    |
   |           bracket point slightly outside interval if close)|
   |------------------------------------------------------------|
   | purpose: least squares fit to vertical time computed       |
   |          from first break picks in headers (for VSPs)      |
   |------------------------------------------------------------|
   |  NOTICE:                                                   |
   |  Copyright (C) 2017 Paul Michaels                          |
   |  <pm@cgiss.boisestate.edu>                                 |
   |  This program is free software; you can                    |
   |  redistribute it and/or modify it under the terms          |
   |  of the GNU General Public License as published            |
   |  by the Free Software Foundation; either version           |
   |  3 of the License, or (at your option) any later           |
   |  version.  This program is distributed in the              |
   |  hope that it will be useful, but                          |
   |  WITHOUT ANY WARRANTY; without even the implied            |
   |  warranty of MERCHANTABILITY or FITNESS FOR A              |
   |  PARTICULAR PURPOSE.  See the GNU General Public           |
   |  License for more details.                                 |
   |  You should have received a copy of the GNU                |
   |  General Public License along with this program;           |
   |  if not, write to the Free Software Foundation,            |
   |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
   |------------------------------------------------------------|
   |INLINE FUNCTIONS:                                           |
   |  getparm    get parameters from command line or interactive|
   |  outlst     echo check of input parameters to listing file |
   |  fitplot    plots solution to postscript file              |
   |                                                            |
   |------------------------------------------------------------|
   |sublibC4 functions:                                         |
   |  c_bsegin   read seismic trace                             |
   |  c_bsegout  write a seismic trace                          |
   |  in_chk     scan input file for samples, number traces etc.|
   |  bargrid    write progress bar grid to screen              |
   |  exbar      write progress bar to screen                   |
   |  cr_labl    write copyright label to screen                |
   |  fcr_labl   write copyright label to file                  |
   |  names      create an output file name from input file name|
   |  lsqufit    least squares fit y=mx+b                       |
   |                                                            |
   |------------------------------------------------------------|
*/
      #include <stdio.h>
      #include <stdlib.h>
      #include <float.h>
      #include <math.h>
      #include <string.h>
      #include <time.h>
      #include "../include/c_bsegy.h"
#ifdef PLPLOT
      #include "plConfig.h"
      #include "plplot.h"
#endif
//...parameter definitions
      #define  maxdim 400

//...function prototypes
      #include "../include/sub4.h"

      int  getparm(int argc,char *argv[],
         float *emin, float *emax, char labl[], char infile[],
         char ofile[], char lstfile[], char pid[]);

      void outlst(float emin, float emax,char labl[], char infile[],
        char pid[], char ofile[], char lstfile[], FILE *h3);
#ifdef PLPLOT
      int fitplot( PLINT npts, float x[], float y[], float yc[],
        float velocity,  float sigmav, float r0, char labl[],char line[]);
#else
      void fitplot( int npts, float x[], float y[], float yc[],
        float velocity,  float sigmav, float r0, char labl[],char line[]);
#endif


      int  main(int argc, char *argv[])
     {

//...declare variables
      FILE              *h1,  *h3;
      int               jrec,
                        npts,
                        ntraces,
                        ibar,
                        invbar;
      float             fsamin,
                        bar,
                        *s1;
      char              pid[5]="bfit",   //process id name (4char)
                        ofile[13],
                        lstfile[13],
                        infile[14];
      struct header     hd;

//...specific to this application
      float		emin,
      			emax;
      char		labl[3]="xx";
      float		pic,sx,sy,sz,sd,
      			rx,ry,rz,range,
			vtime[400],
			tcal[400],
			elv[400],
			sums,sumx,sumy,sumxy,sumx2,
			slope,intercpt,r2,dt,lsqe,
			sigmay,sigmav,velocity,summ;
	float		sumy2;
      int		j,k;
#ifdef PLPLOT
      PLINT		kpts;
#else
      int		kpts;
#endif
      char		line[5];
      char		date[40];
      int		month,day;
      int		isw1;
      int		tbj1,tbj2,tbjhold;
      float		tb1,tb2;

//...get parameters
      if(getparm(argc,argv,&emin,&emax,labl,
         infile,ofile,lstfile,pid)!=0)
      {
       goto quick_exit;
      }

//...open output listing file
        if( (h3=fopen(lstfile,"w"))==NULL )
        {
          fprintf(stderr,"\n ABORT !! Error opening file %s",lstfile);
          goto quick_exit;
        }

//...echo check to listing file
      outlst(emin,emax,labl,infile,pid,ofile,lstfile,h3);

//...open input file
        if( (h1=fopen(infile,"rb"))==NULL )
        {
          fprintf(stderr,"\n ABORT !! Error opening file %s",infile);
          goto exit_3;
        }

	/*
//...open output file
        if( (h2=fopen(ofile,"wb"))==NULL )
        {
          fprintf(stderr,"\n ABORT !! Error opening file %s",ofile);
          goto exit_1;
        }
 */

//...check input file size
          if(in_chk(&ntraces,&npts,&fsamin,hd,h1)!=0)
         {
          goto exit_2;
         }

	  
//...print input file data to lstfile
        fprintf(h3,"\n     Number of samples= %d",npts);
        fprintf(h3,"\n     Sample Interval = %5.4f sec.",fsamin);
        fprintf(h3,"\n     Number of traces = %d",ntraces);

//...dynamic allocation of seimic trace memory
        s1=calloc(npts,sizeof(float));
          if(s1==NULL)
         {
          fprintf(stderr,"ABORT:s1 can't allocate enough trace memory");
          fprintf(stderr," npts = %d",npts);
          goto exit_2;
         }
//-----------------main loop---------------------------
//...display progress bar grid

      bargrid(ntraces,&bar,&ibar,&invbar);

//read times and find boundaries
      tb1=99999999.;
      tb2=99999999.;
      for (jrec=0;jrec<ntraces;jrec++)
     {
//...read trace
        if(c_bsegin(jrec,s1,npts,&hd,h1)!=0)
       {
        fprintf(stderr,"ABORT--c_bsegin");
        goto quit_it;
       }

	findxyz(&sx,&sy,&sz,&sd,&rx,&ry,&rz,&range,hd);
	if (fabs(rz-emax)<tb1) 
	{ tb1=fabs(rz-emax);
	  tbj1=jrec;	
	} //endif
	if (fabs(rz-emin)<tb2)
	{ tb2=fabs(rz-emin);
	  tbj2=jrec;
	} //endif
      } //next jrec

//determine top and bottom
      if(tbj2<tbj1)
      { tbjhold=tbj1;
	tbj1=tbj2;
        tbj2=tbjhold;
      }  //endif	

//...trace loop
      k=0;

      for (jrec=0;jrec<ntraces;jrec++)
     {

//...read trace
        if(c_bsegin(jrec,s1,npts,&hd,h1)!=0)
       {
        fprintf(stderr,"ABORT--c_bsegin");
        goto quit_it;
       }

//convert first arrival time to vertical time and save
	pic=(hd.mst)/1.0E+06;  //convert to seconds
	findxyz(&sx,&sy,&sz,&sd,&rx,&ry,&rz,&range,hd);
//	if (rz<=emax && rz>=emin)
	if (jrec<=tbj2 && jrec>=tbj1)	
	{
	vtime[k]=(fabs(sz-sd-rz)/range)*pic;
	elv[k]=rz;
	k=k+1; } //endif
	
//...display progress bar
      exbar(ntraces,bar,ibar,invbar,jrec+1);

     } //  end trace loop-----------------------------

// perform least squares fit
      kpts=k;
      sums=0;
      sumx=0;
      sumy=0;
      sumxy=0;
      sumx2=0;
      isw1=0;
                sumy2=0;
      fprintf(h3,"\n Number of points=%d \n",kpts);

      lsqufit(elv,vtime,kpts,isw1,&slope,&intercpt,&lsqe,&r2);
      fprintf(h3,"\n\n Elev  Vert_Time  Calc_V-Time \n");
      for (j=0;j<kpts;j++)
      {
        tcal[j]=elv[j]*slope+intercpt;
	dt=vtime[j]-tcal[j];       
	sums=sums+dt*dt;
	fprintf(h3," %6.2f  %.7f     %.7f  \n",
	  elv[j],vtime[j],tcal[j]);
        sumx=sumx+elv[j];
        sumy=sumy+vtime[j];
        sumxy=sumxy+elv[j]*vtime[j]; 
        sumx2=sumx2+elv[j]*elv[j];
	         sumy2=sumy2+vtime[j]*vtime[j];
      } //next j
      sigmay=sqrt(sums/((float)kpts-1));
      summ=0;
      for (j=0;j<kpts;j++)
      {	      
	dt=sumx-(float)kpts*elv[j];
	summ=summ+dt*dt;
      } //next j

//stdv for velocity
      dt=(sumx*sumy-(float)kpts*sumxy);
      sigmav=-((sigmay)*sqrt(summ)*(sumx*sumx-(float)kpts*sumx2)/(dt*dt));
      velocity=1.0/fabs(slope);
      fprintf(h3,"\n Velocity= %.2f m/s  (+/-) %.2f R2=%f \n",velocity,sigmav,r2);
      fprintf(h3,"\n Least Square Error= %.5f \n",lsqe);

//write out well and date
	  strncpy(line,hd.lineid,4);
	  line[4]='\0';
	  month=(int)(hd.tday/100);
	  day=hd.tday-100*month;
	  sprintf(date,"Date: D/M/YR %2.2d/%2.2d/%4.4d"
	     ,month,day,hd.tyear);
	  fprintf(h3,"\n Well: %s \n %s\n",line,date);
	  fprintf(h3," UNIT=%s \n",labl);

//...plot results
      fflush(stdout);
#ifdef PLPLOT
      fitplot(kpts,elv,vtime,tcal,velocity,sigmav,r2,labl,line);
#else
      fitplot(kpts,elv,vtime,tcal,velocity,sigmav,r2,labl,line);
#endif

      printf("\n   Listing ====>%s",lstfile);
#ifdef PLPLOT
      printf("\n   Plot====>bfit.ps");
#else
      printf("\n   Plot====>bfit.pdf");
#endif
      printf("\n   Processed %d traces",ntraces);
        return(0);

quit_it:
        puts(" ");
        free(s1);
exit_2:
        fclose(h1);
exit_3:
        fclose(h3);
quick_exit:
        return(1);
     } //  end of main
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: getparm.c                                   |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: gets input parameters from command                |
   |          line or prompts user from consol.                 |
   |                                                            |
   |                                                            |
   |------------------------------------------------------------|
*/
      int  getparm(int argc,char *argv[],
         float *emin, float *emax, char labl[], char infile[],
         char ofile[], char lstfile[], char pid[])
     {
      char              sfx1[4]="seg",
                        sfx2[4]="lst";
      char		help[3]="-h";
      char              msg1[46],msg2[46];

//...test for command line help ?
      if(argc==2 && !strcmp(argv[1],help))
     {
//...screen header and copyright notice
      cr_labl(pid);

//...help pannel
      sprintf(msg1,"Interval Velocity from least squares         ");
      sprintf(msg2,"fit to vertical times (computed from headers)");
      hlp_labl(msg1,msg2);
      puts(" ");

//...help display
      puts(" ");
      printf("  %s  infile   emin   emax  labl   \n",pid);
      puts(" ");
      puts("     infile  = input file name (4char minimum)");
      puts(" ");
      puts("     emin    =minimum elevation for interval");
      puts("     emax    =maximum elevation for interval");
      puts("     labl    =2 character ID label for interval ");
      puts(" ");
      return(1);
     }
      else
     {
//...get parameters------------------------------
//...pars off character parameters
      if(argc>1)
       strncpy(infile,argv[1],13);
       else
      {
       puts(" Enter INPUT file name ");
       scanf(" %13s",infile);
      }
	if (namecheck(12,infile) != 0 ) return (1);

//...convert string 
      if(argc>2)
       *emin=atof(argv[2]);
       else
      {
       puts(" Enter emin (float)");
       scanf(" %f",emin);
      }

//...convert string to float
      if(argc>3)
       *emax=atof(argv[3]);
       else
      {
       puts(" Enter emax (float)");
       scanf(" %f",emax);
      }

//...convert string to float
      if(argc>4)
       //strncpy(labl,argv[3],2);
       sprintf(labl,"%s",argv[4]);
       else
      {
       puts(" Enter labl (2char)");
       scanf(" %s",labl);
      }
       labl[2]='\0';

//...name output files
      names(infile,pid,ofile,sfx1);
      names(infile,pid,lstfile,sfx2);
       return(0);
     }  //  endif bhelp test
      } // end getparm
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: outlst.c                                    |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: prints copyright header, file names, and          |
   |          echo check of input parameters in newly           |
   |          opened output listing file.                       |
   |                                                            |
   |------------------------------------------------------------|
*/
      void outlst(float emin, float emax,char labl[], char infile[],
        char pid[], char ofile[], char lstfile[], FILE *h3)
     {

//...print copyright label to outlst and screen

      fcr_labl(pid,h3);

//...echo check
      fputs("\n",h3);                      //blank line
      fprintf(h3,"\n     PROGRAM: %s.c ",pid);
      fputs("\n",h3);                      //blank line
      fputs("\n     FILES:       ",h3);
      fprintf(h3,"\n     Input   File= %s",infile);
      fprintf(h3,"\n     Output  File= %s",ofile);
      fprintf(h3,"\n     Listing File= %s",lstfile);
      fputs("\n",h3);                      //blank line
      fputs("\n     INPUT PARAMETERS: ",h3);
      fprintf(h3,"\n     emin  = %f ",emin);
      fprintf(h3,"\n     emax  = %f ",emax);
      fprintf(h3,"\n     labl  = %s ",labl);
      fputs("\n",h3);                      //blank line
     }  //  end outlst
#ifdef PLPLOT
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              |
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS    http://cgiss.boisestate.edu/~pm     | 
   | FUNCTION NAME: fitplot.c                                   |
   |------------------------------------------------------------|
   |  NOTICE:                                                   |
   |  Copyright (C) 2017 Paul Michaels                          |
   |  <pm@cgiss.boisestate.edu>                                 |
   |  This program is free software; you can                    |
   |  redistribute it and/or modify it under the terms          |
   |  of the GNU General Public License as published            |
   |  by the Free Software Foundation; either version           |
   |  3 of the License, or (at your option) any later           |
   |  version.  This program is distributed in the              |
   |  hope that it will be useful, but                          |
   |  WITHOUT ANY WARRANTY; without even the implied            |
   |  warranty of MERCHANTABILITY or FITNESS FOR A              |
   |  PARTICULAR PURPOSE.  See the GNU General Public           |
   |  License for more details.                                 |
   |  You should have received a copy of the GNU                |
   |  General Public License along with this program;           |
   |  if not, write to the Free Software Foundation,            |
   |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
   |------------------------------------------------------------|
   |REQUIRES PLPLOT:                                            |
   |PLPLOT is a GNU plot library which supports both C and      |
   |Fortran calls.  The distribution may be downloaded from     |
   |http://plplot.sourceforge.net                               |
   |------------------------------------------------------------|
   |                                                            |
   | x = elevation (meters)                                     |
   | y = observed vertical times from picks                     |
   | yc = calculated vertical time from LSQ solution            |
   |------------------------------------------------------------|
*/
/*	#include "plplot/plConfig.h"
	#include "plplot/plplot.h"
        #include <math.h>
	#include <string.h>
	#include <ctype.h>  */

      int fitplot( PLINT npts, float x[], float y[],  float yc[],
	 float velocity,  float sigmav, float r0, char labl[],char line[])
     {
	PLFLT		fmargin,
	     		aspect,
			yjust,
			xjust;
        PLFLT		xd[maxdim],   //double precision versions
			yd[maxdim],   //plplot 5.3.1
			ycd[maxdim];

	char		pltfil[]="bfit.ps";
	char		titl[80],
			xlbl[80],
			ylbl[80];
	float		xmin,xmax;
	float		ymin,ymax,
			ycmin,ycmax;
	int		kmn,kmx,i,k;

//convert times to ms
	for (i=0;i<npts;i++)
	{ y[i]=y[i]*1000.;
	  yc[i]=yc[i]*1000.;
	} //next i	  

//  Portrait
      plscolor(0);
      plsori(1);

//...aspect ratio needed for portrait, unless you don't mind
//   plots which are squished in the vertical dimension.
//   aspect=8.5/11.0
      fmargin=0. ;
      aspect=0.77;
      yjust=0.5;
      xjust=0.0;
      plsdev("ps");

      plsdidev(fmargin,aspect,yjust,xjust);
      plsfnam(pltfil);
 
//...initialize plot
      plinit();
      plssub(1,1);
      plfontld(0);
 
//...determine maximum and minimums
      fmax_min(npts,x,&xmax,&xmin,0,&kmx,&kmn);
      fmax_min(npts,y,&ymax,&ymin,0,&kmx,&kmn);
      fmax_min(npts,yc,&ycmax,&ycmin,0,&kmx,&kmn);
      if(ycmax>ymax) ymax=ycmax;
      if(ycmin<ymin) ymin=ycmin;

 
//...set environment, window and viewport
//      plenv(xmin,xmax,ymin,ymax,0,0);
      pladv(0);
      plschr(0,.60);
      plprec(0,0);
      plssym(0,.60);
#ifdef PLPNEW
      plwidth(1.0);
#else
      plwid(1);
#endif
      plvpor(0.1, .95, 0.30, .95);
      //transpose plot
      //plwind(xmin,xmax,ymin,ymax);
      plwind((PLFLT)ymin,(PLFLT)ymax,(PLFLT)xmin,(PLFLT)xmax);

      plbox("bcnst",0.,0.,"bcnstv",0.,0.);

//...write labels and title
      sprintf(titl,"DH=%s UNIT=%s Velocity=%6.2f m/s (+/-)%6.2f m/s R=%.3f  ",
       line,labl,velocity,sigmav,r0);

      //transpose plot
      //sprintf(xlbl,"Elevation (m) ");
      //sprintf(ylbl,"Vertical Time (s) ");
      sprintf(ylbl,"Elevation (m) ");
      sprintf(xlbl,"Vertical Time (ms) ");

      pllab(xlbl,ylbl,titl);
      //transpose plot
      //plpoin(npts,x,y,9);
      for (k=0;k<npts;k++) 
      {
      xd[k]=x[k];
      yd[k]=y[k];
      ycd[k]=yc[k];
      } //end for

      plpoin(npts,yd,xd,9);
 
//...plot calculated
      //transpose plot
      //plline(npts,x,yc);
      plline(npts,ycd,xd);
      plend();
      return(0);
     }  //end fitplot
#else
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: fitplot                                     |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: saves plot to a file GNUPLOT VERSION              |
   |      x=elevation  y=vertical time                          |
   |------------------------------------------------------------|
*/
	void  fitplot (int npts, float x[], float y[], float ycal[],
	float velocity, float sigmav, float r0, char labl[],char line[])
	{
	FILE *pipe = popen("gnuplot -p","w");


	char	XL[]="Vertical Time (ms)";
	char	YL[]="Elevation (m)";
	char	TITLE[80];

//...set title
      sprintf(TITLE,"DH=%s UNIT=%s Velocity=%6.2f m/s (+/-)%6.2f m/s R=%.3f  ",
       line,labl,velocity,sigmav,r0);
/*
	#ifdef WIN
	char Xwin[4]="wxt";
	#else
	char Xwin[4]="x11";
	#endif
*/  
	fprintf(pipe,"set grid \n");
  //	fprintf(pipe,"set terminal %s persist \n",Xwin);
  	fprintf(pipe,"set terminal pdf color enhanced size 6.0,4.0 in \n");
	fprintf(pipe,"set output \"bfit.pdf\" \n");
  	fprintf(pipe,"set key left \n");
  	fprintf(pipe,"set xlabel \" %s \" \n",XL);
  	fprintf(pipe,"set ylabel \" %s \" \n",YL);
  	fprintf(pipe,"set title \" %s \"  \n",TITLE);
//        fprintf(pipe,"set xrange [%f:%f] \n",tmin,tmax);
  	fprintf(pipe," plot '-' u ($1):($2) w points pt 6 ps 1 lc 'black' t '' ,"); 
 	fprintf(pipe," '' u ($1):($2) w l lc 'black' lt 1 lw 2  t '' \n " );
  	int j;
  		for(j=0;j<npts;j++) {
    		fprintf(pipe,"%f %e \n",y[j],x[j]);
  		} //next j 
  	fprintf(pipe,"e\n");  //this is required when std in - plot
		for(j=0;j<npts;j++) {
		fprintf(pipe,"%e %e \n",ycal[j],x[j]);
		} //next j
//...close pipe
	fflush(pipe);
//	getchar(); //wait for user input
	pclose(pipe);

	} //end amplot
#endif
