// |$Id: lamb.c,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $          |
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              |
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS    http://cgiss.boisestate.edu/~pm     | 
   | FUNCTION NAME: lamb.c                                      |
   | DIRECTORY: bsu-2.0.2/src/C/bsegy                           |
   |------------------------------------------------------------|
   | purpose: compute solution to lamb's problem                |
   |          based on the paper:                               |
   |Mooney, H., 1974, "Some numerical solutions for             |
   |   Lamb's problem", Bull.Seis.Soc.Am., Vol 64               |
   |   no. 2, pp 472-491.                                       |
   |                                                            |
   | See also, esp. for elliptical integral definitions:        |
   |Pekeris, C, 1955, "The seismic surface pulse", Proc. Nat.   |
   |   Acad. Sci., Vol 41, p469-480. [equations (61) and (62) ] |
   |                                                            |
   |LIMITATIONS:                                                |
   |special case of vp/vs=sqrt(3) ONLY (can be extended to other|
   |Vp/Vs ratios, but this version is limited to just this one) |
   |------------------------------------------------------------|
   |  NOTICE:                                                   |
   |  Copyright (C) 2017 Paul Michaels                          |
   |  <pm@cgiss.boisestate.edu>                                 |
   |  This program is free software; you can                    |
   |  redistribute it and/or modify it under the terms          |
   |  of the GNU General Public License as published            |
   |  by the Free Software Foundation; either version           |
   |  3 of the License, or (at your option) any later           |
   |  version.  This program is distributed in the              |
   |  hope that it will be useful, but                          |
   |  WITHOUT ANY WARRANTY; without even the implied            |
   |  warranty of MERCHANTABILITY or FITNESS FOR A              |
   |  PARTICULAR PURPOSE.  See the GNU General Public           |
   |  License for more details.                                 |
   |  You should have received a copy of the GNU                |
   |  General Public License along with this program;           |
   |  if not, write to the Free Software Foundation,            |
   |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
   |------------------------------------------------------------|
   |INLINE FUNCTIONS:                                           |
   |  getparm    get parameters from command line or interactive|
   |  outlst     echo check of input parameters to listing file |
   |  zero       zeros a vector                                 |
   |  lambsoln   performs solution to Lamb's Problem            |
   |  deriv      takes derivative of a signal wrt time (Bilinear|
   |             Transform Method)                              |
   |  arma       apply filter (ARMA, uses feedback, min.phase)  |
   |  gencoef    generate filter coefficients for arma          |
   |  lamb4      apply filters to simulate geophone, band-      |
   |             limited source, particle displacement or vel.  |
   |  genhead    creates a trace header, 240 byte               |
   |                                                            |
   |------------------------------------------------------------|
   |sublibC4 functions:                                         |
   |  c_bsegin   read seismic trace                             |
   |  c_bsegout  write a seismic trace                          |
   |  in_chk     scan input file for samples, number traces etc.|
   |  bargrid    write progress bar grid to screen              |
   |  exbar      write progress bar to screen                   |
   |  cr_labl    write copyright label to screen                |
   |  fcr_labl   write copyright label to file                  |
   |  names      create an output file name from input file name|
   |                                                            |
   |  REQUIRES: GSL (GNU Science Library, Elliptical Integrals) |
   |  (what is needed was included in sublibC4, BUT NO LONGER)  |
   |     ellint.c  infnan.c  error.c  fdiv.c  prec.c  stream.c  |
   |  These had  been added to sublib4.a library                |
   |  Now you must install the GSL Library                      |
   |  Also: BSUROOT/C/gsl had contained the needed include files|
   |        config.h  error.h  eval.h                           |
   |        BSUROOT/C/gsl/gsl had contained other  include files|
   |        gsl_errno.h gsl_machine.h gsl_math.h gsl_message.h  |
   |        gsl_mode.h gsl_nan.h gsl_pow_int.h gsl_precision.h  |
   |        gsl_sf_ellint.h gsl_sf_result.h gsl_sys.h           |
   |                                                            |
   |------------------------------------------------------------|
*/

#include <stdio.h>
#include <stdlib.h>
#include <float.h>
#include <math.h>
#include <string.h>
#include <time.h>
#include "../include/c_bsegy.h"

#include "gsl/gsl_mode.h"
//...parameter definitions

//...function prototypes
#include "../include/sub4.h"

int getparm (int argc, char *argv[],
	     float *xmin1, float *dx, int *np, float *tmax1, float *fsamin,
	     float *vs, float *den, int *itype, float *sfrq, float *sdamp,
	     float *gfrq, float *gdamp, float *pol, float *stab,
	     char ofilev[], char ofileh[], char lstfile[], char pid[]);

void outlst (float xmin1, float dx, int np, float tmax1,
	     float fsamin, float vs, float den, int itype, float sfrq,
	     float sdamp, float gfrq, float gdamp, float pol, float stab,
	     char pid[], char ofilev[], char ofileh[], char lstfile[],
	     FILE * h3);

void zero (int npts, float x[]);

void lambsoln (float rr, float den, float fsamin, float z,
	       float tmax1, int npts, float g[], float vs, float r[]);

void deriv (int npts, float dt, float x[], float y[], float stab);

int arma (int npts, float x[], float y[],
	  float c0, float c1, float c2, float d0, float d1, float d2);

void gencoef (float dt, float b, float wo,
	      float *c0, float *c1, float *c2,
	      float *d0, float *d1, float *d2);

int lamb4 (float rr, float den, float fsamin,
	   float tmax1, int npts, float g[], float vs, float r[],
	   float dg[], float dr[], float fdg[], float w[], float w1[],
	   float fdr[], float vfdg[], float vfdr[], float vw[],
	   float sfrq, float sdamp, float gfrq, float gdamp, int itype, 
	   float stab);

void  genhead(int jrec, float fsamin, int npts, float gfrq, 
           float dx, float xmin1, float sfrq, struct header *hd);	

double  gsl_sf_ellint_P(double phi, double k, double n, gsl_mode_t mode);

double  gsl_sf_ellint_F(double phi, double k, gsl_mode_t mode);


int
main (int argc, char *argv[])
{
//...declare variables
  FILE *h1, *h2, *h3;
//  int jt, jrec, npts, ntraces, ibar, invbar;
  int jrec, npts, ntraces, ibar, invbar;
  float fsamin, bar, *s1, *s2, *work;
  char pid[5] = "lamb",		//process id name
    ofilev[13], ofileh[13], lstfile[13];
  struct header hd;
 // char          hd1[240];
  union head {
	  struct header hd;
	  char    hd1[240];
          };  //end union

//...specific to this application
  float rr, xmin1, dx, tmax1, vs, den, sfrq, sdamp, gfrq, gdamp, pol, stab;
  int np, j, itype;

//...get parameters
  if (getparm
      (argc, argv, &xmin1, &dx, &np, &tmax1, &fsamin, &vs, &den, &itype,
       &sfrq, &sdamp, &gfrq, &gdamp, &pol, &stab, ofilev, ofileh, lstfile, pid) != 0)
    {
      goto quick_exit;
    }

//...open output listing file
  if ((h3 = fopen (lstfile, "w")) == NULL)
    {
      fprintf (stderr, "\n abort !! error opening file %s", lstfile);
      goto quick_exit;
    }

//...echo check to listing file
  outlst (xmin1, dx, np, tmax1, fsamin, vs, den, itype, sfrq, sdamp, gfrq,
	  gdamp, pol, stab, pid, ofilev, ofileh, lstfile, h3);

//...open output file
  if ((h1 = fopen (ofilev, "wb")) == NULL)
    {
      fprintf (stderr, "\n abort !! error opening file %s", ofilev);
      goto exit_1;
    }

//...open output file
  if ((h2 = fopen (ofileh, "wb")) == NULL)
    {
      fprintf (stderr, "\n abort !! error opening file %s", ofileh);
      goto exit_1;
    }

//...compute trace parameters
  npts = tmax1 / fsamin;
  ntraces = np;
//...oversample NPTS
	double OVSdt=.000010;
	if (fsamin<OVSdt) {
		OVSdt=fsamin;}
  	long   NPTS = tmax1 / OVSdt; 

//...print input file data to lstfile
  fprintf (h3, "\n     number of samples= %d", npts);
  fprintf (h3, "\n     sample interval = %5.4f sec.", fsamin);
  fprintf (h3, "\n     number of traces = %d", ntraces);

//...dynamic allocation of seimic trace memory
  work = calloc (11 * NPTS, sizeof (float));
  if (work == NULL)
    {
      fprintf (stderr, "abort:work can't allocate enough trace memory");
      fprintf (stderr, " npts = %ld", NPTS);
      goto exit_s2;
    }

//-----------------main loop---------------------------
//...display progress bar grid

  bargrid (ntraces, &bar, &ibar, &invbar);

//...clear header
//  for (jt=0;jt<240;jt++) hd1[jt]='\0';

//...trace loop

  for (jrec = 0; jrec < ntraces; jrec++)
    {

//generate trace header
      genhead(jrec,fsamin,npts,gfrq,dx,xmin1,sfrq,&hd); 

//...compute traces
      rr = xmin1 + (float) (jrec) * dx;	//offset
// g=work[0] r=work[npts] dg=work[2*npts] dr=work[3*npts]
// fdg=work[4*npts] w=work[5*npts] w1=work[6*npts] fdr=work[7*npts]
// vfdg=work[8*npts] vfdr=work[9*npts] vw=work[10*npts]

      lambsoln (rr, den, OVSdt, pol, tmax1, NPTS, &work[0], vs,
		&work[NPTS]);

      lamb4 (rr, den, OVSdt, tmax1, NPTS, &work[0], vs, &work[NPTS],
	     &work[2 * NPTS], &work[3 * NPTS], &work[4 * NPTS],
	     &work[5 * NPTS], &work[6 * NPTS], &work[7 * NPTS],
	     &work[8 * NPTS], &work[9 * NPTS], &work[10 * NPTS], sfrq, sdamp,
	     gfrq, gdamp, itype, stab);

//choose output
      //ground displacement, step function source
      if (itype == 1)
	{
	  s1 = &work[0];	//vertical
	  s2 = &work[NPTS];	//radial
	}			//endif
      //ground particle velocity, step function source
      // (or ground displacement, impulse source)
      if (itype == 2)
	{
	  s1 = &work[NPTS * 2];	//vertical
	  s2 = &work[NPTS * 3];	//radial
	}			//endif

      //ground displacement, wavelet source
      // itype causes early bailout from lamb4
      if (itype == 3)
	{
	  s1 = &work[NPTS * 4];	//vertical
	  s2 = &work[NPTS * 7];	//radial
	}			//endif

      //ground displacement, wavelet source
      // itype causes early bailout from lamb4
      if (itype == 4)
	{
	  s1 = &work[NPTS * 8];	//vertical
	  s2 = &work[NPTS * 9];	//radial
	}			//endif

      //geophone displacement, wavelet source
      if (itype == 5)
	{
	  s1 = &work[NPTS * 4];	//vertical
	  s2 = &work[NPTS * 7];	//radial
	}			//endif

      //geophone velocity (volt), wavelet source
      if (itype == 6)
	{
	  s1 = &work[NPTS * 8];	//vertical
	  s2 = &work[NPTS * 9];	//radial
	}			//endif

      //source wavelet displacement
      // itype causes early bailout from lamb4
      if (itype == 7)
	{
	  s1 = &work[NPTS * 6];	//vertical
	  s2 = &work[NPTS * 6];	//radial
	}			//endif

      //source wavelet velocity
      // itype causes early bailout from lamb4
      if (itype == 8)
	{
	  s1 = &work[NPTS * 10];	//vertical
	  s2 = &work[NPTS * 10];	//radial
	}			//endif

      //source wavelet geophone displacement 
      if (itype == 9)
	{
	  s1 = &work[NPTS * 6];	//vertical
	  s2 = &work[NPTS * 6];	//radial
	}			//endif

      //source wavelet geophone velocity
      if (itype == 10)
	{
	  s1 = &work[NPTS * 10];	//vertical
	  s2 = &work[NPTS * 10];	//radial
	}			//endif

//resample
//	double Amx=-1000000.;
	int	OVS=ceil(fsamin/OVSdt); //oversample
//	fprintf(stderr,"\n OVS=%d",OVS);
	for (j=0; j<npts; j++) {
	s1[j]=s1[j*OVS];
	s2[j]=s2[j*OVS]; 
//		if (fabs(s2[j]) >Amx) Amx=fabs(s2[j]);
	} //next k
//	fprintf(stderr,"\n Resample Amx=%.10lE ",Amx);



//...vertical trace
      if (c_bsegout (jrec, s1, npts, &hd, h1) != 0)
	{
	  fprintf (stderr, "abort--c_bsegout");
	  goto quit_it;
	}
      hd.geoazi = (short) 90;
      hd.geover = (short) 90;
//...radial trace
      if (c_bsegout (jrec, s2, npts, &hd, h2) != 0)
	{
	  fprintf (stderr, "abort--c_bsegout");
	  goto quit_it;
	}

//...display progress bar
      exbar (ntraces,bar, ibar, invbar, jrec + 1);

    }				//  end trace loop-----------------------------

  printf ("\n   listing ====>%s", lstfile);
  printf ("\n   output v  ====>%s", ofilev);
  printf ("\n   output h  ====>%s", ofileh);
  printf ("\n   processed %d traces", ntraces);
  return(0);

quit_it:
  puts (" ");
  free (work);
exit_s2:
//        free(s2);
  fclose (h2);
exit_1:
  fclose (h1);
  fclose (h3);

quick_exit:
  return(1);

}				//  end of main

/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: getparm.c                                   |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: gets input parameters from command                |
   |          line or prompts user from consol.                 |
   |                                                            |
   |                                                            |
   |------------------------------------------------------------|
*/
int
getparm (int argc, char *argv[],
	 float *xmin1, float *dx, int *np, float *tmax1, float *fsamin,
	 float *vs, float *den, int *itype, float *sfrq, float *sdamp,
	 float *gfrq, float *gdamp, float *pol, float *stab,
	 char ofilev[], char ofileh[], char lstfile[], char pid[])
{
  float r;
  int j;
  char help[3] = "-h";

//...test for command line help ?
  if (argc == 2 && !strcmp (argv[1], help))
    {
//...screen header and copyright notice
      cr_labl (pid);

//...help display
      puts ("bhelp h");
      puts (" ");
      printf
      ("  %s  xmin dx np tmax fsamin vs den itype sfrq sdamp gfrq gdamp pol stab ",
	 pid);
      puts (" ");
      puts ("  xmin = minimum geophone offset (m) ");
      puts ("  dx    = spacing of geophones (m)");
      puts ("  np    = number of geophones ");
      puts ("  tmax1 = maximum time for seismogram (s) ");
      puts ("  fsamin= sample interval (seconds) ");
      puts ("  vs    = shear wave velocity (m/s) ");
      puts ("  den   = mass density (kg/m3) ");
      puts ("  itype = type of traces output ");
      puts ("       1= ground displacement, step function source  ");
      puts ("       2= ground particle velocity, step function source ");
      puts ("          (or ground displacement, impulse source)");
      puts ("       3= ground displacement, source wavelet=damped resonator");
      puts ("       4= ground particle velocity, source wavelet=damped resonator");
      puts ("       5= geophone displacement with source wavelet ");
      puts ("       6= geophone particle velocity with source wavelet (geophone voltage)");
      puts ("       7= source wavelet displacement (at source)");
      puts ("       8= source wavelet velocity (at source)");
      puts ("       9= source wavelet geophone displacement (at source)");
      puts ("      10= source wavelet geophone velocity (at source)");
      puts ("  sfrq  = source wavelet high-cut frequency (hz)");
      puts ("  sdamp = source damping (fraction of critical, example .7)");
      puts ("  gfrq  = geophone resonant frequency (hz) ");
      puts ("  gdamp = geophone damping (fraction of critical, example .7) ");
      puts ("  pol   = polarity switch");
      puts ("      -1= SEG Sign Convention (up motion = negative = trough)  ");
      puts ("       0= TEST MODE, for display of normalized solution, NO 1/R etc. ");
      puts ("      +1= REVERSE SEG Sign Convention (up motion = positive = peak ");
      puts ("  stab  = stability factor, for derivative, moves pole off unit circle ");
      puts ("          (not generally needed except for itype=8 (try stab=.16)  ");
      puts ("           since most other outputs have enough low-pass filtering) ");
      puts ("           See function deriv for more ");
      puts ("  ");
      return (1);
    }
  else
    {
//...get parameters------------------------------
      if (argc > 1)
	*xmin1 = atof (argv[1]);
      else
	{
	  puts (" enter xmin: minimum geophone offset (float)");
	  scanf (" %f", xmin1);
	}

//...domain check
      if(*xmin1<=0.) 
      {
	puts("===================================================================");
 	fprintf(stderr,"ABORT-- xmin must be > zero \n");
	puts("===================================================================");
	      return(1);
      } //endif

      if (argc > 2)
	*dx = atof (argv[2]);
      else
	{
	  puts (" enter dx: geophone spacing (float)");
	  scanf (" %f", dx);
	}

      if (argc > 3)
	*np = atoi (argv[3]);
      else
	{
	  puts (" enter np: number of phones  (int)");
	  scanf (" %d", np);
	}

//...domain check
      for(j=0;j<*np;j++)
      {
	      r=*xmin1 + ((float) j) * (*dx);
	      if (r==0.0)
              {
		puts("===================================================================");
		 fprintf(stderr,"\nABORT-- range CAN NOT be zero \n");
		puts("===================================================================");
	      return(1);
	      }  //endif
      }  //next j

      if (argc > 4)
	*tmax1 = atof (argv[4]);
      else
	{
	  puts (" enter tmax: maximum time on trace (float)");
	  scanf (" %f", tmax1);
	}

//...domain check
      if(*tmax1<=0.) 
      {
	puts("===================================================================");
	 fprintf(stderr,"ABORT-- tmax must be > zero \n");
	puts("===================================================================");
	      return(1);
      } //endif


      if (argc > 5)
	*fsamin = atof (argv[5]);
      else
	{
	  puts (" enter fsamin: sample interval (s) (float)");
	  scanf (" %f", fsamin);
	}

//...domain check
      if(*fsamin<=0.) 
      {
	puts("===================================================================");
	 fprintf(stderr,"ABORT-- sample interval must be > zero \n");
	puts("===================================================================");
	      return(1);
      } //endif


//...check to make sure trace length fits int 16 header word
	long	npts=ceil(*tmax1/(*fsamin));
	if (npts > 32767 ) {
	puts("===================================================================");
	fprintf(stderr,"\nABORT-- tmax and fsamin result in npts=%ld >32767",npts);
	fprintf(stderr,"\n Maximum trace length is 32767 due to int16 header\n");
	puts("===================================================================");
	return(1);
	} //endif

      if (argc > 6)
	*vs = atof (argv[6]);
      else
	{
	  puts (" enter vs: shear wave velocity (m/s) (float)");
	  scanf (" %f", vs);
	}

//...domain check
      if(*vs<=0.) 
      { 
	puts("===================================================================");
	fprintf(stderr,"ABORT-- shear velocity must be > zero \n");
	puts("===================================================================");
	      return(1);
      } //endif


      if (argc > 7)
	*den = atof (argv[7]);
      else
	{
	  puts (" enter den: mass density (kg/m3) (float)");
	  scanf (" %f", den);
	}
	if (*den <= 0.0 ) {
	puts("===================================================================");
	fprintf(stderr,"Density = %f which must be positive\n", *den);
	puts("===================================================================");
	return(1); } //endif

      if (argc > 8)
	*itype = atoi (argv[8]);
      else
	{
	  puts (" enter itype: type of output traces on v and h (int)");
	  puts ("       1= ground displacement, step function source  ");
	  puts ("       2= ground particle velocity, step function source ");
	  puts ("          (or ground displacement, impulse source)");
          puts ("       3= ground displacement, source wavelet=damped resonator");
          puts ("       4= ground particle velocity, source wavelet=damped resonator");
	  puts ("       5= geophone displacement with source wavelet ");
          puts ("       6= geophone particle velocity with source wavelet (geophone voltage)");
	  puts ("       7= source wavelet displacement (at source)");
	  puts ("       8= source wavelet velocity (at source)");
	  puts ("       9= source wavelet geophone displacement (at source)");
	  puts ("      10= source wavelet geophone velocity (at source)");
	  scanf (" %d", itype);
	}
//...domain check
      if(*itype<1 || *itype > 10) 
      { 
	puts("===================================================================");
	fprintf(stderr,"ABORT!!! itype out of range %d  \n",*itype);
	puts("===================================================================");
	return(1);
      } //endif

      if (argc > 9)
	*sfrq = atof (argv[9]);
      else
	{
	  puts (" enter sfrq: resonant frequency of source (hz) (float)");
	  scanf (" %f", sfrq);
	}

//...domain check
	float	nyquist=1/(2.0*(*fsamin));
      if(*sfrq>nyquist/2.0) 
      {
	puts("===================================================================");
 	 fprintf(stderr,"WARNING!!! Source frequency %f > 1/2 Nyquist (%f Hz) \n",*sfrq,nyquist);
	puts("===================================================================");
      } //endif


      if (argc > 10)
	*sdamp = atof (argv[10]);
      else
	{
          puts (" enter sdamp: damping of source (fraction critical, zb 0.7)  (float)");
	  scanf (" %f", sdamp);
	}
//...domain check
      if(*sdamp<=0.) 
      { 
	puts("===================================================================");
	fprintf(stderr,"ABORT!!! Source Damping %f < zero \n",*sdamp);
	puts("===================================================================");
	return(1);
      } //endif


      if (argc > 11)
	*gfrq = atof (argv[11]);
      else
	{
	  puts (" enter gfrq: resonant frequency of geophone (hz) (float)");
	  scanf (" %f", gfrq);
	}
//...domain check
      if(*gfrq>nyquist/2.0) 
      { 
	puts("===================================================================");
	fprintf(stderr,"WARNING!!! Geophone frequency %f > 1/2 Nyquist (%f Hz) \n",*gfrq,nyquist);
	puts("===================================================================");
      } //endif


      if (argc > 12)
	*gdamp = atof (argv[12]);
      else
	{
          puts (" enter gdamp: damping of geophone (fraction critical, zb 0.7)  (float)");
	  scanf (" %f", gdamp);
	}
//...domain check
      if(*gdamp<=0.) 
      { 
	puts("===================================================================");
	fprintf(stderr,"ABORT!!! Geophone Damping %f < zero \n",*gdamp);
	puts("===================================================================");
	return(1);
      } //endif


      if (argc > 13)
	*pol = atof (argv[13]);
      else
	{
          puts (" enter pol: sign convention  (float)");
      puts ("  pol   = polarity switch");
      puts ("      -1= SEG Sign Convention (up motion = negative = trough)  ");
      puts ("       0= TEST MODE, for display of normalized solution, NO 1/R etc. ");
      puts ("      +1= REVERSE SEG Sign Convention (up motion = positive = peak ");
	  scanf (" %f", pol);
	}

      if (argc > 14)
	*stab  = atof (argv[14]);
      else
	{
          puts ("  STABILITY FACTOR: for derivative, moves pole off unit circle ");
          puts ("  (not generally needed except for itype=8 (try stab=.16)  ");
          puts ("  since most other outputs have enough low-pass filtering) ");
          puts ("  See function deriv for more ");
          puts (" enter stab:  (float)");
	  scanf (" %f", stab );
	}
//...domain check
      if(*stab<0.) 
      { 
	puts("===================================================================");
	fprintf(stderr,"ABORT!!! stability factor %f < zero \n",*stab);
	puts("===================================================================");
	return(1);
      } //endif

//...name output files
      sprintf (ofilev, "lambv.seg");
      sprintf (ofileh, "lambh.seg");
      sprintf (lstfile, "lamb.lst");
      return (0);
    }				//  endif bhelp test
}				// end getparm

/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: outlst.c                                    |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: prints copyright header, file names, and          |
   |          echo check of input parameters in newly           |
   |          opened output listing file.                       |
   |                                                            |
   |------------------------------------------------------------|
*/
void
outlst (float xmin1, float dx, int np, float tmax1,
	float fsamin, float vs, float den, int itype, float sfrq,
	float sdamp, float gfrq, float gdamp, float pol, float stab,
	char pid[], char ofilev[], char ofileh[], char lstfile[], FILE * h3)
{

//...print copyright label to outlst and screen

  fcr_labl (pid, h3);

//...echo check
  fputs ("\n", h3);		//blank line
  fprintf (h3, "\n     program: %s.c ", pid);
  fputs ("\n", h3);		//blank line
  fputs ("\n     files:       ", h3);
  fprintf (h3, "\n     output vertical file= %s", ofilev);
  fprintf (h3, "\n     output horizontal file= %s", ofileh);
  fprintf (h3, "\n     listing file= %s", lstfile);
  fputs ("\n", h3);		//blank line
  fputs ("\n     input parameters: ", h3);
  fprintf (h3, "\n     xmin1 = %7.3f ", xmin1);
  fprintf (h3, "\n     dx    = %7.4f ", dx);
  fprintf (h3, "\n     np    = %5.5d ", np);
  fprintf (h3, "\n     tmax1 = %10.5f", tmax1);
  fprintf (h3, "\n     fsamin= %10.7f", fsamin);
  fprintf (h3, "\n     vs    = %7.2f", vs);
  fprintf (h3, "\n     den   = %7.2f", den);
  fprintf (h3, "\n     itype = %2.2d", itype);
  fprintf (h3, "\n       1= ground displacement, step function source  ");
  fprintf (h3, "\n       2= ground particle velocity, step function source ");
  fprintf (h3, "\n          (or ground displacement, impulse source)");
  fprintf (h3, "\n       3= ground displacement, source wavelet=damped resonator");
  fprintf (h3, "\n       4= ground particle velocity, source wavelet=damped resonator");
  fprintf (h3, "\n       5= geophone displacement with source wavelet ");
  fprintf (h3, "\n       6= geophone particle velocity with source wavelet (geophone voltage)");
  fprintf (h3, "\n       7= source wavelet displacement (at source)");
  fprintf (h3, "\n       8= source wavelet velocity (at source)");
  fprintf (h3, "\n       9= source wavelet geophone displacement (at source)");
  fprintf (h3, "\n      10= source wavelet geophone velocity (at source)");
  fprintf (h3, "\n     source freq and damping");
  fprintf (h3, "\n     sfrq    = %10.4f", sfrq);
  fprintf (h3, "\n     sdamp    = %7.4f", sdamp);
  fprintf (h3, "\n     geophone freq and damping");
  fprintf (h3, "\n     gfrq    = %10.4f", gfrq);
  fprintf (h3, "\n     gdamp    = %7.4f", gdamp);
  fprintf (h3, "\n     pol      = %7.4f", pol  );
  fprintf (h3, "\n     stab     = %10.6f", stab );
  fputs ("\n", h3);		//blank line
}				//  end outlst
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: lamb4.c                                     |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: applies filters to the solution of Lamb's problem |
   |          Simulates geophone response, limited source band. |
   |                                                            |
   |          (requires results from lambsoln.c)                |
   |                                                            |
   |  rr=shot to receiver offset                                |
   |  den=density kg/m3                                         |
   |  tmax1=maximum time of trace in real time                  |
   |  fsamin=sample interval                                    |
   |  g[ ]= step function vertical displacement                 |
   |  r[ ]= step function horizontal displacement               |
   |  dg[]= derivative g[], impulse response displ.             |
   |  dr[]= derivative r[], impulse response displ.             |
   |  w[]=  impulse function                                    |
   |  fdg[]=vertical displacement, w/source moment              |
   |  fdr[]=vertical displacement, w/source moment              |
   |  w1[]= source wavelet = scalar source moment(t)            |
   | -----------------------------------------------------------|
   |  save memory: copy fdg->dg, fdr->dr w1->w                  |
   |  Bilinear Transform s=(2/dt)*(1-z)/(1+z) method            |
   |  fdg[]=geophone displacement, vertical                     |
   |  fdr[]=geophone displacement, horizontal                   |
   |  w1[]= source wavelet filtered with geophone               |
   | -----------------------------------------------------------|
   |  vfdg[]=deriv. particle velocity (geoph.volt) Ver          |
   |  vfdr[]=deriv. particle velocity (geoph.volt) Hor          |
   |  vw[]=deriv. particle vel. geoph.filt wavelet              |
   |                                                            |
   |------------------------------------------------------------|
*/
int
lamb4 (float rr, float den, float fsamin,
       float tmax1, int npts, float g[], float vs, float r[],
       float dg[], float dr[], float fdg[], float w[], float w1[],
       float fdr[], float vfdg[], float vfdr[], float vw[],
       float sfrq, float sdamp, float gfrq, float gdamp, int itype,
       float stab)
#define sq(value) ( value*value )
#define pi 3.14159265358979323846264338327950
{
  int j;
  float dt;
  float  b, frq, wd, beta, c0, c1, c2, d0, d1, d2;

//...differentiate step function solution-->impulse response
//...use bilinear transform
  dt = fsamin;
  deriv (npts, dt, g, dg, stab);
  deriv (npts, dt, r, dr, stab);

//...define wavelet
  zero (npts, w);
  w[0] = 1.0;			//impulse
  frq = sfrq;
  wd = pi * 2 * frq;		//digital frequency desired
  beta = (2. / dt) * tan ((wd * dt) / 2.);	//analog frequency
  b = sdamp * beta;		//sdamp is fraction critical damp  

//...ztransform realization for s=(2/dt)*(1-z)/(1+z)
  gencoef (dt, b, beta, &c0, &c1, &c2, &d0, &d1, &d2);

//...compute vertical displacement:  source wavelet 
  arma (npts, dg, fdg, c0, c1, c2, d0, d1, d2);
  arma (npts, w, w1, c0, c1, c2, d0, d1, d2);

//...compute horizontal displacement:  source wavelet
  arma (npts, dr, fdr, c0, c1, c2, d0, d1, d2);

//...bail out early if no geophone required
  if (itype == 3)
    goto out3;
  if (itype == 4)
    goto out4;
  if (itype == 7)
    goto out3;
  if (itype == 8)
    goto out4;

//...copy output to input
  for (j = 0; j < npts; j++)
    dg[j] = fdg[j];
  for (j = 0; j < npts; j++)
    dr[j] = fdr[j];
  for (j = 0; j < npts; j++)
    w[j] = w1[j];

//...add a geophone transfer function (displacement)
  frq = gfrq;
  wd = pi * 2 * frq;		//digital frequency desired
  beta = (2. / dt) * tan ((wd * dt) / 2.);	//analog frequency
  b = gdamp * beta;		//gdamp is fraction critical damp  

//...ztransform realization for s=(2/dt)*(1-z)/(1+z)
  gencoef (dt, b, beta, &c0, &c1, &c2, &d0, &d1, &d2);

//...compute vertical phone displacement 
  arma (npts, dg, fdg, c0, c1, c2, d0, d1, d2);
  arma (npts, w, w1, c0, c1, c2, d0, d1, d2);

//compute horizontal phone displacement
  arma (npts, dr, fdr, c0, c1, c2, d0, d1, d2);

out4:
//...differentiate to particle velocity of phone (geophone voltage)
//...use bilinear transform         
  zero (npts, vfdg);
  zero (npts, vfdr);
  zero (npts, vw);
  deriv (npts, dt, fdg, vfdg, stab);
  deriv (npts, dt, fdr, vfdr, stab);
  deriv (npts, dt, w1, vw, stab);
out3:
  return(0);
}				//end lamb4

/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: zero.c                                      |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   | zeros out a vector                                         |
   |------------------------------------------------------------|
*/
void
zero (int npts, float x[])
{
  int j;
  for (j = 0; j < npts; j++)
    x[j] = 0.;
}				//end zeros
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: lambsoln.c                                  |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: function which computes lamb's problem            |
   |          based on the paper:                               |
   |mooney, h., 1974, "some numerical solutions for             | 
   |   lamb's problem", bull.seis.soc.am., vol 64               |
   |   no. 2, pp 472-491.                                       |
   |                                                            |
   |special case of vp/vs=sqrt(3)                               |
   |                                                            |
   |  rr=shot to receiver offset                                |
   |  den=density kg/m3                                         |
   |  tmax1=maximum time of trace in real time                  |
   |  fsamin=sample interval                                    |
   |  g[ ]= step function vertical displacement                 |
   |  r[ ]= step function horizontal displacement               |
   |  z   = factor to set sign convention, amplitudes           |
   | -1= SEG Convention: up motion = negative, trough           |
   |  0= Test mode, no 1/R etc factors, for normalized          |
   | +1= Reverse SEG: up motion = positive, black peak          |
   |------------------------------------------------------------|
*/
void
lambsoln (float rr, float den, float fsamin, float z,
	  float tmax1, int npts, float g[], float vs, float r[])
#define sq(value) ( value*value )
#define pi 3.14159265358979323846264338327950
#define mode GSL_PREC_DOUBLE
{
  double mu,			//rigidity
    delta,			//vp to vs ratio
    gama, gama2, dt, tau, tm, cs, amp;
//    gama, gama2, dt, tau, tm, cs, c, amp;
  long j;
  double delta2, tau2, k2, kappa2, xvir6, nup, num, nua8, nuap,
    nuam, k1, fp, fm;
  double kappa1, pia8, piap, piam, ka;
  double kb, pib8, pibp, pibm;
  double pio2=pi/(double) 2.0;

//some constants
	double  D1=1.0000;
	double	D3=3.0000;
	double  D4=4.0000;
	double  D5=5.0000;
	double	D6=6.0000;
	double	D8=8.0000;
	double	D12=12.0000;
	double	D16=16.0000;
	double	D18=18.0000;
	double	D20=20.0000;
	double	D24=24.0000;
	double	D48=48.0000;
	double	D96=96.0000;
	double	D1Em8=.00000001;

//... compute velocities remaining
  delta = sqrt ((double) 3.);
//  c = delta * (double) vs;
  cs = (double) vs;

//...rigidity
  mu = den * sq (cs);

//...rayleigh pole root
  gama = sqrt ((double) 3. + sqrt ((double) 3.)) / (double) 2.;
  gama2 = gama * gama;

//...unitless time scale is tau
  dt = fsamin;

//...scale factor, force function
  if (  z  != 0.0)
    {
      amp = (sq (delta) * (double) z) / (pi * pi * mu * (double) rr);
    }
  else
    {
      amp = (double) 1.0; 
    } 

//...vertical displacement
  for (j = 0; j < npts; j++)
    {
      tm = ((double) (j - 1)) * dt;
      tau = (cs / (double) rr) * tm;

      // ---fixup for  tau=1 or tau=gama-----
      if (tau == D1)
	{
	  tau = tau + D1Em8;
	}
      if (tau == gama)
	{
	  tau = tau + D1Em8;
	}

      if (tau < D1 / delta)
	g[j] = 0.;

      if (tau > (double) 1.000000 / delta && tau < (double) 1.00000)
	{
	  g[j] =
	    -(pi / D96 ) * (D6 -
			   sqrt (D3 * sqrt (D3) + D5) / sqrt (sq (gama) -
							      sq (tau)) +
			   sqrt (D3 * sqrt (D3) - D5) / sqrt (sq (tau) +
							      sqrt (D3) / D4 -
							      D3 / D4) -
			   sqrt (D3) / (sqrt (sq (tau) - D1 / D4)));
	}			//endif

      if (tau > (double) 1.0 && tau < gama)
	{
	  g[j] =
	    -(pi / D48) * (D6 -
			   sqrt (D3 * sqrt (D3) + D5) / sqrt (sq (gama) -
							      sq (tau)));
	}			//endif

      if (tau > gama)
	g[j] = -pi / D8;

//...apply scale factor
      g[j] = g[j] * (amp);

    }				//next j

//...horizontal displacement

//	double	Amx=-100000.;

  for (j = 0; j < npts; j++)
    {
      tm = ((double) (j - 1)) * dt;
      tau = (cs / (double) rr) * tm;

      // ---fixup for  tau=1 or tau=gama-----
      if (tau == D1)
	{
	  tau = tau + D1Em8;
	}
      if (tau == gama)
	{
	  tau = tau + D1Em8;
	}

      delta2 = delta * delta;
      tau2 = tau * tau;
      k2 = (delta2 * tau2 - D1) / (delta2 - D1);
      kappa2 = D1 / k2;
      xvir6 = D16 * sqrt (D6);

//...compute elliptic integrals
      nup = D20 + D12 * sqrt (D3);
      num = D20 - D12 * sqrt (D3);
      nua8 = D8 * k2;
      nuap = nup * k2;
      nuam = num * k2;
      k1 = sqrt (k2);
      kappa1 = sqrt (kappa2);
      fp = D6 + D4 * sqrt (D3);
      fm = D6 - D4 * sqrt (D3);

      if (tau < (D1 / delta))
	r[j] = 0.;

      if ((tau > (D1 / delta)) && (tau < D1))
	{
          ka=gsl_sf_ellint_F(pio2,k1,mode);
          pia8=gsl_sf_ellint_P(pio2,k1,nua8,mode);
          piap=gsl_sf_ellint_P(pio2,k1,nuap,mode);
          piam=gsl_sf_ellint_P(pio2,k1,nuam,mode);

	  r[j] =
	    -(tau / xvir6) * (D6 * ka - D18 * pia8 + fm * piam + fp * piap);
	}			//endif

      if ((tau > D1) && (tau < gama))
	{
          kb=gsl_sf_ellint_F(pio2,kappa1,mode);
          pib8=gsl_sf_ellint_P(pio2,kappa1,D8,mode);
          pibp=gsl_sf_ellint_P(pio2,kappa1,nup,mode);
          pibm=gsl_sf_ellint_P(pio2,kappa1,num,mode);

          r[j] =
	    -((tau * kappa1) / xvir6) * (D6 * kb - D18 * pib8 + fm * pibm +
					 fp * pibp);
	}			//endif

      if (tau > gama)
	{
          kb=gsl_sf_ellint_F(pio2,kappa1,mode);
          pib8=gsl_sf_ellint_P(pio2,kappa1,D8,mode);
          pibp=gsl_sf_ellint_P(pio2,kappa1,nup,mode);
          pibm=gsl_sf_ellint_P(pio2,kappa1,num,mode);

      	  r[j] =
	    -(((tau * kappa1) / xvir6) *
	      (D6 * kb - D18 * pib8 + fm * pibm + fp * pibp) +
	      (pi * tau) / (D24 * sqrt (tau2 - gama2)));
	}			//endif

//...apply scale factor
//		if (fabs(r[j]) >Amx) Amx=fabs(r[j]);
      r[j] = (double) r[j] * (amp);

    }				//next j
//	fprintf(stderr,"\n Amx=%.10lE ",Amx);


}				//end lambsoln
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: deriv.c                                     |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: computes derivative using bilinear                |
   |          transform approximation                           |
   |x[]=input  y[]=output=derivative of x                       |
   |dt = sample interval                                        |
   |                                                            |
   |         2     (1-Z)                                        |
   |  Y(Z)= ---- * ------------ * X(Z)                          |
   |         dt    ((1+stab)+Z)                                 |
   |                                                            |
   | stab=stability factor, moves pole outside unit             |
   |      circle so nyquist does not blow up                    |
   | can be zero unless output=8=derivative of source           |
   | wavelet ground motion (particle velocity at                |
   | source)                                                    |
   |                                                            |
   |all other output options are low-pass which filter          |
   | out the nyquist with sufficient attenuation                |
   |------------------------------------------------------------|
*/
void
deriv (int npts, float dt, float x[], float y[], float stab)
{
  int j;
/*...stab shifts the pole slightly off unit circle (stab=.16 good
     for a sample rate of .00001s and freq=100 hz
     (stabalizes runs which include frequencies out
      near the nyquist, ie. broad band wavelets)  */

  for (j = 0; j < npts; j++)
    {
      y[j] = 0.0;
    }
  y[0] = x[0];
  for (j = 1; j < npts; j++)
    {
      y[j] = ((2.0 / dt) * (x[j] - x[j - 1]) - y[j - 1] )/(1.0+stab);
    }				//next j
}				//end deriv

/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: arma.c                                      |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: applies an ARMA operator                          |
   |                                                            |
   |x[]=input  y[]=output                                       |
   |                                                            |
   | Z-transform is:                                            |
   |                                                            |
   | Y(Z)   c0+c1*Z+c2*z^2   output                             |
   | ---- = -------------- = -----                              |
   | X(Z)   d0+d1*Z+d2*z^2   input                              |
   |                                                            |
   |                                                            |
   |------------------------------------------------------------|
*/
int
arma (int npts, float x[], float y[],
      float c0, float c1, float c2, float d0, float d1, float d2)
{
  int j;

//...check npts
  if (npts < 2)
    {
      return (0);
    }				//abort too few points

//...initialize first values
//   (assumes causal system, x[j]=y[j]=0 for j<0)
  y[0] = (c0 * x[0]) / d0;
  y[1] = (c0 * x[1] + c1 * x[0] - d1 * y[0]) / d0;
  for (j = 2; j < npts; j++)
    {
      y[j] =
	(c0 * x[j] + c1 * x[j - 1] + c2 * x[j - 2] - d1 * y[j - 1] -
	 d2 * y[j - 2]) / d0;
    }

  return (1);
}				//end arma
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: gencoef.c                                   |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: generates coefficients for ARMA op                |
   |                                                            |
   | dt=sample interval for digital realization                 |
   | wo=radial frequency                                        |
   | b=damping                                                  |
   |        2     (1-Z)                                         |
   | s=jw= ---- * -----  Bilinear Transform                     |
   |        dt    (1+Z)                                         |
   |                                                            |
   |             1                                              |
   | H(s)= ---------------                                      |
   |       s^2 + 2b*s + wo^2                                    |
   |                                                            |
   | b<wo underdamped                                           |
   | b=wo critically damped                                     |
   | b>wo overdamped                                            |
   |                                                            |
   |------------------------------------------------------------|
*/
void
gencoef (float dt, float b, float wo,
	 float *c0, float *c1, float *c2, float *d0, float *d1, float *d2)
{
  *c0 = dt * dt;
  *c1 = 2. * dt * dt;
  *c2 = dt * dt;
  *d0 = wo * wo * dt * dt + 4. * b * dt + 4.;
  *d1 = 2. * wo * wo * dt * dt - 8.0;
  *d2 = wo * wo * dt * dt - 4. * b * dt + 4;

}				//end gencoef

/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: genhead.c                                   |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose:  generate a 240 byte trace header                 |
   |                                                            |
   |------------------------------------------------------------|
*/
void  genhead(int jrec, float fsamin, int npts, float gfrq, 
        float dx, float xmin1, float sfrq, struct header *hd)	
{

  int  jt;
  char buf1[10];
//generate trace header
      (*hd).tsnl = (long) jrec + 1;
      (*hd).tsnt = (long) jrec + 1;
      (*hd).ofrn = (long) 1;
      (*hd).tnofr = (long) jrec + 1;
      (*hd).cdp = (long) 1;
      (*hd).duse = (short) 1;
      (*hd).nvs = (short) 1;
      (*hd).nhs = (short) 1;
      (*hd).tic = (short) 1;
      (*hd).ictime = (short) 0;
      (*hd).isi = (short) ( (fsamin * 1000000.)+0.5);
      (*hd).length = (short) npts;
      (*hd).gaint = (short) 3;
      (*hd).gidb = (short) 0;
      (*hd).utimes = (short) 0;
      (*hd).utimeg = (short) 0;
      (*hd).charge = (short) 0;
      (*hd).smul1 = (short) -1000;
      (*hd).smul2 = (short) -1000;
      for (jt = 0; jt < 4; jt++)
	(*hd).spname[jt] = ' ';
      (*hd).spname[3] = '1';
      sprintf (buf1, "%4.4d", jrec + 1);
      for (jt = 0; jt < 4; jt++)
	(*hd).rstnam[jt] = buf1[jt];
      sprintf (buf1, "0001");
      for (jt = 0; jt < 4; jt++)
	(*hd).lineid[jt] = buf1[jt];
      (*hd).geoazi = (short) 0;
      (*hd).geover = (short) 0;
      (*hd).shtver = (short) 180;
      (*hd).shtazi = (short) 0;
      (*hd).mst = (long) 0;
      (*hd).sday = (short) 0;
      (*hd).shour = (short) 0;
      (*hd).smin = (short) 0;
      (*hd).sseco = (short) 0;
      (*hd).tyear = (short) 2001;
      (*hd).tday = (short) 01;
      (*hd).thour = (short) 12;
      (*hd).tmin = (short) 00;
      (*hd).tsec = (short) 00;
      (*hd).ishlo = (long) 0;
      (*hd).ishla = (long) 0;
      (*hd).ishd = (long) 0;
      (*hd).ishe = (long) 0;
      (*hd).irlo =
	(long) ((xmin1 + (float) (jrec) * dx) * (float) fabs ((*hd).smul2));
      (*hd).irla = (long) 0;
      (*hd).irel = (long) 0;
      (*hd).flc = (short) gfrq;
      (*hd).fhc = (short) sfrq;
      }  // end genhead

