// |$Id: setgeom.c,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $       |
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              |
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS    http://cgiss.boisestate.edu/~pm     | 
   | FUNCTION NAME: setgeom.c                                   |
   | DIRECTORY: bsu-2.0.2/src/C/bsegy                           |
   |------------------------------------------------------------|
   | purpose: set geometry for data which have been converted   |
   |  from a seismograph format to BSEGY (ie.  *.seg files).    |
   |  For example, a SEG-2 file (like 1001.dat) should be       |
   |  converted to an *.seg file by egg2seg (or if you have     |
   |  seismic unix and can produce a SEG-Y file from the        |
   |  seismograph, you could use segyread, SU program).         |
   |                                                            |
   |  This program then uses shots.txt and phones.txt           |
   |  files to control extraction of coordinates from           |
   |  an *.nez file (topcon).  The *.txt files can be           |
   |  generated with program "gensetg.c".  This program can be  |
   |  helpful in setting geometry for irregularly shot          |
   |  refraction work (typical of engineering problems).        |
   |                                                            |
   |  The other way you can go is to use the BSU topcon2 program|
   |  That program was designed to work with down-hole surveys  |
   |  but is also an option for surface data.                   |
   |------------------------------------------------------------|
   |  NOTICE:                                                   |
   |  Copyright (C) 2017 Paul Michaels                          |
   |  <pm@cgiss.boisestate.edu>                                 |
   |  This program is free software; you can                    |
   |  redistribute it and/or modify it under the terms          |
   |  of the GNU General Public License as published            |
   |  by the Free Software Foundation; either version           |
   |  3 of the License, or (at your option) any later           |
   |  version.  This program is distributed in the              |
   |  hope that it will be useful, but                          |
   |  WITHOUT ANY WARRANTY; without even the implied            |
   |  warranty of MERCHANTABILITY or FITNESS FOR A              |
   |  PARTICULAR PURPOSE.  See the GNU General Public           |
   |  License for more details.                                 |
   |  You should have received a copy of the GNU                |
   |  General Public License along with this program;           |
   |  if not, write to the Free Software Foundation,            |
   |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
   |------------------------------------------------------------|
   |INLINE FUNCTIONS:                                           |
   |  getparm    get parameters from command line or interactive|
   |  outlst     echo check of input parameters to listing file |
   |                                                            |
   |------------------------------------------------------------|
   |sublibC4 functions:                                         |
   |  c_bsegin   read seismic trace                             |
   |  c_bsegout  write a seismic trace                          |
   |  in_chk     scan input file for samples, number traces etc.|
   |  bargrid    write progress bar grid to screen              |
   |  exbar      write progress bar to screen                   |
   |  cr_labl    write copyright label to screen                |
   |  fcr_labl   write copyright label to file                  |
   |  names      create an output file name from input file name|
   |                                                            |
   |------------------------------------------------------------|
*/
      #include <stdio.h>
      #include <stdlib.h>
      #include <float.h>
      #include <math.h>
      #include <string.h>
      #include <time.h>
      #include "../include/c_bsegy.h"

//...parameter definitions

//...function prototypes
      #include "../include/sub4.h"

      int  getparm(int argc,char *argv[],
        char sfile[], char pfile[],char nezfile[],
        char lstfile[], char pid[]);

      void outlst(char sfile[], char pfile[], char nezfile[],
        char pid[], char lstfile[], FILE *h3);

      int  main(int argc, char *argv[])
     {

//...declare variables
      FILE              *h1, *h2, *h3;
      FILE		*fp1;
	int             jrec,
                        npts,
                        ntraces,
                        ibar,
                        invbar;
      float             fsamin,
                        bar,
                        *s1;
      char              pid[5]="setg",   //process id name
                        sfile[14],
                        pfile[14],
                        ofile[13],
                        lstfile[13],
			nezfile[14],
			infile[14];
	int		count;
	char		fnames[100][11];
	char		splabel[100][6];
	char		vplabel[100][6];
      struct header     hd;

//...specific to this application
	int		j,k,jj,oops,nshots;
        char            sfx1[4]="seg";
	int		chnl[100],cnt,
			nctl;
//			nchanls;
	float		yc[200],xc[200],zc[200];
	float		gx,gy,gz,
			sx,sy,sz;
	char		lbl[200][6];
	size_t		lng;
	char		tlabl[5];

//...get parameters
      if(getparm(argc,argv,sfile,pfile,nezfile,
         lstfile,pid)!=0)
      {
       goto quick_exit;
      }

//...open output listing file
        if( (h3=fopen(lstfile,"w"))==NULL )
        {
          fprintf(stderr,"\n ABORT !! Error opening file %s",lstfile);
          goto quick_exit;
        }
//...echo check to listing file
      outlst(sfile,pfile,nezfile,pid,lstfile,h3);

//open sfile
        if( (fp1=fopen(sfile,"r")) == NULL )
        {
          fprintf(stderr, "Error opening sfile"); 
          exit(1);
        }       //endif

//read in shot control
	fprintf(h3,"DATA FILE    SP LABEL\n");
	count=0;
        while ( !feof(fp1) )
        {
          fscanf(fp1,"%8s%s",
                 fnames[count],splabel[count]);
        if(!feof(fp1))
	fprintf(h3,"%8s <---  %s\n",fnames[count],splabel[count]);
	count=count+1;
	if(count>100)
       {
          fprintf(stderr, "too many shots"); 
          exit(1);
       }  //endif
	} // endwhile
	fclose(fp1);
	nshots=count-1;

//open pfile
        if( (fp1=fopen(pfile,"r")) == NULL )
        {
          fprintf(stderr, "Error opening pfile"); 
          exit(1);
        }       //endif

//read in phone control
	fprintf(h3,"\n\nChannel #  VP LABEL\n");
	count=0;
        while ( !feof(fp1) )
        {
          fscanf(fp1,"%d%s",
                 &chnl[count],vplabel[count]);
        if(!feof(fp1))
	fprintf(h3,"%5d <--- %s\n",chnl[count],vplabel[count]);
	count=count+1;
	if(count>100)
       {
          fprintf(stderr, "too many channels"); 
          exit(1);
       }  //endif
	} // endwhile
	fclose(fp1);
//	nchanls=count-1;

//open nezfile
        if( (fp1=fopen(nezfile,"r")) == NULL )
        {
          fprintf(stderr, "Error opening pfile"); 
          exit(1);
        }       //endif
//read in nez control
	fprintf(h3,"\n\nCount  Y          X        Z     LABEL\n");
	count=0;
        while ( !feof(fp1) )
        {
          fscanf(fp1,"%d%f%f%f%5s",
            &cnt,&yc[count],&xc[count],&zc[count],lbl[count]);
        if(!feof(fp1))
	fprintf(h3," %-5d %-8.3f %-8.3f %-8.3f %-5s \n",
	  cnt,yc[count],xc[count],zc[count],lbl[count] );
	count=count+1;
	if(count>200)
       {
          fprintf(stderr, "too many control"); 
          exit(1);
       }  //endif
	} // endwhile
	fclose(fp1);
	nctl=count-1;

//...display progress bar grid

      bargrid(ntraces,&bar,&ibar,&invbar);

//shot file loop
	for (j=0; j<nshots; j++)
       {
	strcpy(infile,fnames[j]);
//...name output files
        names(infile,pid,ofile,sfx1);

//...open input file
        if( (h1=fopen(infile,"rb"))==NULL )
        {
          fprintf(stderr,"\n ABORT !! Error opening file %s",infile);
          goto exit_3;
        }

//...open output file
        if( (h2=fopen(ofile,"wb"))==NULL )
        {
          fprintf(stderr,"\n ABORT !! Error opening file %s",ofile);
          goto exit_1;
        }

//...check input file size
          if(in_chk(&ntraces,&npts,&fsamin,hd,h1)!=0)
         {
          goto exit_2;
         }

	fprintf(h3,"\n %s",infile);

//...dynamic allocation of seimic trace memory
        s1=calloc(npts,sizeof(float));
          if(s1==NULL)
         {
          fprintf(stderr,"ABORT:s1 can't allocate enough trace memory");
          fprintf(stderr," npts = %d",npts);
          goto exit_2;
         }
//-----------------main loop---------------------------

//...trace loop

      for (jrec=0;jrec<ntraces;jrec++)
     {

//...read trace
        if(c_bsegin(jrec,s1,npts,&hd,h1)!=0)
       {
        fprintf(stderr,"ABORT--c_bsegin");
        goto quit_it;
       }

//------------------match header with control
	oops=0;
	for (k=0; k<nctl; k++)
       {
	  if (strcmp(vplabel[jrec],lbl[k])==0)
         {
	  gx=xc[k];
	  gy=yc[k];
	  gz=zc[k];
	  oops=oops+1;
         }  //endif

	  if (strcmp(splabel[j],lbl[k])==0)
         {
	  sx=xc[k];
	  sy=yc[k];
	  sz=zc[k];
	  oops=oops+1;
         }  //endif
       }  // next k

	if (oops<2)
	{
	fprintf(stderr,"OOPS! couldn't find a match\n");
	fprintf(stderr,"vp=%s  sp=%s\n",vplabel[jrec],splabel[j]);
	}  // endif

//transfer values to headers
	hd.smul1=-1000;
	hd.smul2=-1000;
	hd.ishlo=(int ) 1000.*sx;
	hd.ishla=(int ) 1000.*sy;
	hd.ishe=(int )  1000.*sz;
	hd.irlo=(int ) 1000.*gx;
	hd.irla=(int ) 1000.*gy;
	hd.irel=(int )  1000.*gz;

//copy sp and vp names
	lng=strlen(splabel[j]);
	for (jj=0; jj<4; jj++) tlabl[jj]=' ';
  	for (jj=(6-lng); jj<4; jj++) tlabl[jj]=splabel[j][2+jj-(6-lng)];
  	memcpy(&hd.spname[0],tlabl,4);
	lng=strlen(vplabel[jrec]);
	for (jj=0; jj<4; jj++) tlabl[jj]=' ';
  	for (jj=(6-lng); jj<4; jj++) tlabl[jj]=vplabel[jrec][2+jj-(6-lng)]; 
  	memcpy(&hd.rstnam[0],tlabl,4);

//...original field record number and other headers
	hd.ofrn=atoi(hd.spname);
	strcpy(hd.lineid,"001");
	strcpy(hd.geoor,"vrt");
	hd.geover=180;
	hd.geoazi=0;
	hd.shtver=180;
	hd.shtazi=0;

//...output trace
        if(c_bsegout(jrec,s1,npts,&hd,h2)!=0)
       {
        fprintf(stderr,"ABORT--c_bsegout");
        goto quit_it;
       }

//...display progress bar
      exbar(ntraces*nshots,bar,ibar,invbar,jrec+1);

     } //  end trace loop-----------------------------


      } // next j (shots loop)

      printf("\n   Listing ====>%s",lstfile);
      printf("\n   Output  ====>%s",ofile);
      printf("\n   Processed %d traces",ntraces);
      return(0);

quit_it:
        puts(" ");

        free(s1);
exit_2:
        fclose(h2);
exit_1:
        fclose(h1);
exit_3:
        fclose(h3);

quick_exit:
      return(1);

     } //  end of main

/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: getparm.c                                   |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: gets input parameters from command                |
   |          line or prompts user from consol.                 |
   |                                                            |
   |                                                            |
   |------------------------------------------------------------|
*/
      int  getparm(int argc,char *argv[],
         char sfile[], char pfile[], char nezfile[], char lstfile[],
         char pid[])
     {
      char		help[3]="-h";
      char              msg1[46],msg2[46];

//...test for command line help ?
      if(argc==2 && !strcmp(argv[1],help))
     {
//...screen header and copyright notice
      cr_labl(pid);

//...help pannel
      sprintf(msg1,"setgeom:  Applies shots.txt and phones.txt   ");
      sprintf(msg2,"files generated by gensetg                   ");
      hlp_labl(msg1,msg2);
      puts(" ");

//...help display
      puts(" ");
      printf("  setgeom    sfile   pfile   nezfile \n");
      puts(" ");
      puts("     Control Files Direct Processing of Data Files:");
      puts("     sfile    =  shots.txt file relating records to shots ");
      puts("     pfile    =  phones.txt file relating traces to phones ");
      puts("   nezfile    =  survey information for shots and phones   ");
      puts(" ");
      puts(" ");
      return(1);
     }
      else
     {
//...get parameters------------------------------
//...pars off character parameters
      if(argc>1)
       strncpy(sfile,argv[1],13);
       else
      {
       puts(" Enter sfile name  (shots.txt)");
       scanf(" %12s",sfile);
      }
	if (namecheck(12,sfile) != 0 ) return (1);

      if(argc>2)
       strncpy(pfile,argv[2],13);
       else
      {
       puts(" Enter pfile name  (phones.txt)");
       scanf(" %13s",pfile);
      }
	if (namecheck(12,pfile) != 0 ) return (1);

      if(argc>3)
       strncpy(nezfile,argv[3],13);
       else
      {
       puts(" Enter nezfile name  (survey.nez)");
       scanf(" %13s",nezfile);
      }
	if (namecheck(12,nezfile) != 0 ) return (1);

	sprintf(lstfile,"setgeom.lst");
       return(0);
     }  //  endif bhelp test
      } // end getparm

/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: outlst.c                                    |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: prints copyright header, file names, and          |
   |          echo check of input parameters in newly           |
   |          opened output listing file.                       |
   |                                                            |
   |------------------------------------------------------------|
*/
      void outlst(char sfile[],char pfile[],char nezfile[],
        char pid[], char lstfile[], FILE *h3)
     {

//...print copyright label to outlst and screen

      fcr_labl(pid,h3);

//...echo check
      fputs("\n",h3);                      //blank line
      fprintf(h3,"\n     PROGRAM: %s.c ",pid);
      fputs("\n",h3);                      //blank line
      fputs("\n     FILES:       ",h3);
      fprintf(h3,"\n      shots  File= %s",sfile);
      fprintf(h3,"\n      phones File= %s",pfile);
      fprintf(h3,"\n nez: survey File= %s",nezfile);
      fprintf(h3,"\n     Listing File= %s",lstfile);
      fputs("\n",h3);                      //blank line
      fputs("\n     INPUT PARAMETERS: ",h3);
      fputs("\n",h3);                      //blank line
     }  //  end outlst
