// |$Id: topcon2.c,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $       |
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              |
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS    http://cgiss.boisestate.edu/~pm     | 
   | MODIFIED: Added case 3 20bit packed SEG                    |
   | (inspired by Brett Bennett's seg2segy.c code, but fixed for|
   |  large negative values bug that produced a sawtooth error  |
   |  on converting the 20 bit packed case).                    |
   |                                                            |
   | References:                                                |
   |    Bennet, B. (1990),"A computer program to convert SEG-2  |
   |      data to SEG-Y", Geophysics, vol. 55, no.9, p.1272-1284|
   |                                                            |
   |    Pullan, S.E. et al.(1990),"Recommended standard for     |
   |      seismic (/radar) dtat files in the personal computer  |
   |      environment", Geophysics, vol. 55, no.9, p.1260-1271  |
   |                                                            |
   |    Bary, K.M, D.A. Cavers, and C. Kneale (1975),"Report on |
   |      recommended standards for digital tape formats",      |
   |      Geophysics, vol. 40, no. 02, p. 344-352               |
   |                                                            |
   | FUNCTION NAME: topcon2.c                                   |
   | DIRECTORY: bsu-2.0.2/src/C/bsegy                           |
   |------------------------------------------------------------|
   | purpose: topcon2 convert SEG-2  to *.seg                   |
   |          and merge topcon *.nez header info                |
   |------------------------------------------------------------|
   |  NOTICE:                                                   |
   |  Copyright (C) 2017 Paul Michaels                          |
   |  <pm@cgiss.boisestate.edu>                                 |
   |  This program is free software; you can                    |
   |  redistribute it and/or modify it under the terms          |
   |  of the GNU General Public License as published            |
   |  by the Free Software Foundation; either version           |
   |  3 of the License, or (at your option) any later           |
   |  version.  This program is distributed in the              |
   |  hope that it will be useful, but                          |
   |  WITHOUT ANY WARRANTY; without even the implied            |
   |  warranty of MERCHANTABILITY or FITNESS FOR A              |
   |  PARTICULAR PURPOSE.  See the GNU General Public           |
   |  License for more details.                                 |
   |  You should have received a copy of the GNU                |
   |  General Public License along with this program;           |
   |  if not, write to the Free Software Foundation,            |
   |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
   |------------------------------------------------------------|
   |INLINE FUNCTIONS:                                           |
   |  getparm    get parameters from command line or interactive|
   |  outlst     echo check of input parameters to listing file |
   |  readfdb    read SEG-2 File Descriptor Block  FDB          |
   |  dmy2dy     convert day,month,year to day of the year      |
   |  month2num  convert ascii month to integer                 |
   |  readtdb    reads Trace Descriptor Block  TDB              |
   |  readata    reads a data trace                             |
   |  readtop    reads topcon survey file                       |
   |                                                            |
   |------------------------------------------------------------|
   |sublibC4 functions:                                         |
   |  c_bsegin   read seismic trace                             |
   |  c_bsegout  write a seismic trace                          |
   |  in_chk     scan input file for samples, number traces etc.|
   |  bargrid    write progress bar grid to screen              |
   |  exbar      write progress bar to screen                   |
   |  cr_labl    write copyright label to screen                |
   |  fcr_labl   write copyright label to file                  |
   |  names      create an output file name from input file name|
   |                                                            |
   |------------------------------------------------------------|
*/
      #include <stdio.h>
      #include <stdlib.h>
      #include <float.h>
      #include <math.h>
      #include <string.h>
      #include <time.h>
      #include "../include/c_bsegy.h"

//...define constants
	#define MAXCH 66

//...function prototypes
      #include "../include/sub4.h"

      int  getparm(int argc,char *argv[],
         char topf[], char infile[],
	 char linenm[], float *fshdp, int *ispn, int *nchanl, int *ivp1, 
	 int *ivpn, int *irec, float *eshift, int *isaz, int *isvr,
	 int *irazi, int *itazi, int *islop,
         char ofile[], char lstfile[],char pid[]);

      void outlst(char infile[],
        char pid[], char ofile[], char lstfile[], FILE *h3);

      int   readfdb(int *ntraces, unsigned int trptr[],
	short *tyear, short *tday, short *thour, short *tmin, short *tsec,
        FILE *h1, FILE *h3);

      int   readtdb(int jtrace, unsigned int trptr[],unsigned int *nsamp,
        unsigned short *sztdblk, unsigned char *dfc,
	float *fsamin, float *sfactr, int *chnl, int *gain,
	float *delay, int *hcf, int *lcf, int *rawrec, int *stack,
      	float *skew, float *recx, float *sourcex, char *linid, 
        float *notchf, FILE *h1, FILE *h3);

      int   readata(int jtrace, unsigned int trptr[], int sztdblk,
        int nsamp, int idfc, void *data, float sfactr, float *s1, FILE *h1);

      int  month2num(char amonth[]);

     void  dmy2dy(short *day, short *month, short *year, 
        short *doy);

     int readtop(float *sx, float *sy, float *sz,
	float rx[], float ry[], float rz[], int chan[], int vp[],
	int islop, int ispn, int ivp1, float eshift, int nchanl, 
	FILE *h4);

      int  main(int argc, char *argv[])
     {

//...declare variables
      FILE              *h1, *h2, *h3, *h4;
      int               npts,
                        ibar,
                        invbar;
      float             fsamin,
			sfactr,
                        bar,
                        *s1;
      int		*ldata;
      int		*idata;
      float		*fdata;
      double		*ddata;
      unsigned char     *cdata;
      unsigned int     nbytes;
      char              pid[5]="topc",   //process id name
			topf[14],
                        ofile[13],
                        lstfile[13],
                        infile[14];
      struct header     hd;

//...specific to this application
	unsigned int	trptr[66],	//array trace descr. pointers
			nsamp;
	unsigned short  sztdblk;	//size trace descr. block
	int		ntraces,
			chnl,
			gain,
			hcf,
			lcf,
			rawrec,
			stack,
			jtrace;
	unsigned char	dfc;		//data format code
	char		buff[10],
			zro[241],
			linid[5],
			linenm[5];
	int		j,
			idfc,
			ispn,
			nchanl,
			islop,
			ivp1,
			ivpn,
			irec,
			isaz,
			isvr,
			irazi,
			itazi;
	float		skew,
			recx,
                        sourcex,
                        notchf,
			coordfac,
			delay,
			fshdp,
			eshift;
	short		tday,
			tyear,
			thour,
			tmin,
			tsec;
	float		sx,		//shot coordinates
			sy,
			sz;
	float		*rx,		//receiver coordinate arrays
			*ry,
			*rz;
	int		*chan,
			*vp;

//...zero header
//NOTE: PC Linux is little endian  lsb msb
	for (j=0; j<240; j++) zro[j]='\0';
	memcpy(&hd,&zro,240);	

//...get parameters
      if(getparm(argc,argv,
         topf,infile,linenm,&fshdp,&ispn,&nchanl,&ivp1,
	 &ivpn,&irec,&eshift,&isaz,&isvr,&irazi,&itazi,&islop,
         ofile,lstfile,pid)!=0)
      {
        goto quick_exit;
      }

//...open output listing file
        if( (h3=fopen(lstfile,"w"))==NULL )
        {
          printf("\n ABORT !! Error opening file %s",lstfile);
          goto quick_exit;
        }

//...open topcon file
        if( (h4=fopen(topf,"r"))==NULL )
        {
          printf("\n ABORT !! Error opening file %s",topf);
          goto exit_3;
        }

//...echo check to listing file
        outlst(infile,pid,ofile,lstfile,h3);

//...open input file
        if( (h1=fopen(infile,"rb"))==NULL )
        {
          printf("\n ABORT !! Error opening file %s",infile);
          goto exit_4;
        }

//...open output file
        if( (h2=fopen(ofile,"wb"))==NULL )
        {
          printf("\n ABORT !! Error opening file %s",ofile);
          goto exit_1;
        }

//...read SEG-2 File Descriptor Block FDB
        if( readfdb(&ntraces,&trptr[0],
	  &tyear,&tday,&thour,&tmin,&tsec,
 	  h1,h3)==0 ) puts("error reading FDB");

        bargrid(ntraces,&bar,&ibar,&invbar);

//...read Trace Descriptor Block TDB
	jtrace=0;
        if( readtdb(jtrace,&trptr[0],&nsamp,&sztdblk,&dfc,&fsamin,
         &sfactr,&chnl,&gain,&delay,&hcf,&lcf,&rawrec,&stack,&skew,
         &recx,&sourcex,linid,&notchf,h1,h3)==0 )
		 puts("error reading TDB");
	fprintf(h3,"FSAMIN=%f \n",fsamin);
	fprintf(h3,"SFACTR=%.8f \n",sfactr);
	fprintf(h3,"CHNL=%d \n",chnl);
	fprintf(h3,"GAIN=%d \n",gain);
	fprintf(h3,"DELAY=%f \n",delay);
	fprintf(h3,"HCF=%d \n",hcf);
	fprintf(h3,"LCF=%d \n",lcf);
	fprintf(h3,"RAWREC=%d \n",rawrec);
	fprintf(h3,"STACK=%d \n",stack);
	fprintf(h3,"SKEW=%f \n",skew);
        fprintf(h3,"RECEIVER LOCATION=%f \n",recx);
        fprintf(h3,"SOURCE LOCATION=%f \n",sourcex);
        fprintf(h3,"LINE ID=%s \n",linid);
        fprintf(h3,"NOTCH FREQUENCY=%f \n",notchf);


//...cast numbers to integer
	npts=(int) nsamp;
	idfc=(int) dfc;

//...dynamic allocation of seimic trace memory *.seg Trace
        s1=calloc(npts,sizeof(float));
          if(s1==NULL)
         {
          fprintf(stderr,"ABORT:s1 can't allocate enough trace memory");
          fprintf(stderr," npts = %d",npts);
          goto exit_2; 
         }

//...dynamic allocation of seimic trace memory *.seg Trace
        rx=calloc(ntraces,sizeof(float));
          if(rx==NULL)
         {
          fprintf(stderr,"ABORT:rx can't allocate enough  memory");
          fprintf(stderr," ntraces = %d",ntraces);
          goto exit_s1; 
         }

//...dynamic allocation of seimic trace memory *.seg Trace
        ry=calloc(ntraces,sizeof(float));
          if(ry==NULL)
         {
          fprintf(stderr,"ABORT:ry can't allocate enough memory");
          fprintf(stderr," ntraces = %d",ntraces);
          goto exit_rx; 
         }

//...dynamic allocation of seimic trace memory *.seg Trace
        rz=calloc(ntraces,sizeof(float));
          if(rz==NULL)
         {
          fprintf(stderr,"ABORT:rz can't allocate enough memory");
          fprintf(stderr," ntraces = %d",ntraces);
          goto exit_ry; 
         }

//...dynamic allocation of seimic trace memory *.seg Trace
        chan=calloc(ntraces,sizeof(int));
          if(chan==NULL)
         {
          fprintf(stderr,"ABORT:chan can't allocate enough memory");
          fprintf(stderr," ntraces = %d",ntraces);
          goto exit_rz; 
         }

//...dynamic allocation of seimic trace memory *.seg Trace
        vp=calloc(ntraces,sizeof(int));
          if(vp==NULL)
         {
          fprintf(stderr,"ABORT:vp can't allocate enough memory");
          fprintf(stderr," ntraces = %d",ntraces);
          goto exit_chan; 
         }

/*  Data Format Codes (idfc)
	1=16 bit fixed INT
	2=32 bit fixed LONG  (Default type EGG Strataview Seismograph)
	3=20 bit packed floating SEG format
	4=32 bit IEEE floating point FLOAT
	5=64 bit IEEE floating point DOUBLE
*/

	if (idfc>5 || idfc<1)
	{fprintf(stderr,"ABORT!! Invalid dfc=%d\n",dfc);
	goto exit_s1; }

//...dynamic allocation of seimic trace memory SEG-2 Trace
	switch(idfc)
       {
	case 1:
        {idata=calloc(npts,sizeof(int)); 
	 if(idata==NULL) {fprintf(stderr,"ABORT: calloc idata\n"); goto exit_s1;}
	 break;}  //1 esac
	case 2:
        {ldata=calloc(npts,sizeof(int));
	 if(ldata==NULL) {fprintf(stderr,"ABORT: calloc ldata\n"); goto exit_s1;}
	 break;}  //2 esac
	case 3:
//        {fprintf(stderr,"ABORT!! dfc not supported\n"); goto exit_s1;
         nbytes=(npts*5)/2;
         {cdata=calloc(nbytes,sizeof(unsigned char));
         if(cdata==NULL) {fprintf(stderr,"ABORT: calloc cdata\n"); goto exit_s1;}
	 break;}  //3 esac
	case 4:
        {fdata=calloc(npts,sizeof(float));
	 if(fdata==NULL) {fprintf(stderr,"ABORT: calloc fdata\n"); goto exit_s1;}
	 break;}  //4 esac
	case 5:
        {ddata=calloc(npts,sizeof(double));
	 if(ddata==NULL) {fprintf(stderr,"ABORT: calloc ddata\n"); goto exit_s1;}
	 break;}  //5 esac
       } //end switch

//read topcon data
        if(readtop(&sx,&sy,&sz,rx,ry,rz,chan,vp,
	islop,ispn,ivp1,eshift,nchanl,h4)!=0)
	goto quit_it;



//------------------------start trace loop------------------------------------
//...traces loop, read seg-2 trace header and data
	for (jtrace=0; jtrace<ntraces; jtrace++)
	{
//...read Trace Descriptor Block TDB
        if( readtdb(jtrace,&trptr[0],&nsamp,&sztdblk,&dfc,&fsamin,
        &sfactr,&chnl,&gain,&delay,&hcf,&lcf,&rawrec,&stack,&skew,
        &recx,&sourcex,linid,&notchf,h1,h3)==0 )
		 puts("error reading TDB");

//...convert from mvolts to microvolts
	sfactr=(sfactr*1000.)/stack;

	switch(idfc)
       {
	case 1:
        {if (readata(jtrace,&trptr[0],sztdblk,npts,idfc,idata,sfactr,s1,h1)==0 )
	 puts("error reading idata");
	break;}  //1 esac
	case 2:
        {if (readata(jtrace,&trptr[0],sztdblk,npts,idfc,ldata,sfactr,s1,h1)==0 )
	 puts("error reading ldata");
	break;}  //2 esac
	case 3:
        {if (readata(jtrace,&trptr[0],sztdblk,npts,idfc,cdata,sfactr,s1,h1)==0 )
         puts("error reading cdata");
        break;}  //3 esac
	case 4:
        {if (readata(jtrace,&trptr[0],sztdblk,npts,idfc,fdata,sfactr,s1,h1)==0 )
	 puts("error reading fdata");
	break;}  //4 esac
	case 5:
        {if (readata(jtrace,&trptr[0],sztdblk,npts,idfc,ddata,sfactr,s1,h1)==0 )
	 puts("error reading ddata");
	break;}  //5 esac
       }  //end switch

//...copy seg-2 header info to current header
  	hd.isi=(short) (fsamin*1000000.);  //sample interval in microseconds
  	hd.length=(short) npts;		//number of samples per trace
	hd.nvs=(short) stack;
	hd.gidb=(short) gain;
	int	IK;
	IK=floor( (float) rawrec /1000.0 );
	hd.ofrn=(int) (rawrec-IK*1000);
//	hd.ofrn=(int) rawrec;
	hd.flc=(short) lcf;
	hd.fhc=(short) hcf;
	hd.tnofr=(int) chnl;
	hd.tyear=tyear;
	hd.tday=tday;
	hd.thour=thour;
	hd.tmin=tmin;
	hd.tsec=tsec;
        hd.smul1=1;
        hd.smul2=1;
        hd.ishlo=(int) sourcex;
        hd.irlo=(int) recx;
        if (delay>= 0.0)
        {hd.ictime=(int)(delay*1000.0+0.5);}
        else
        {hd.ictime=(int)(delay*1000.0-0.5);} //endif 
	sprintf(buff,"xxxx");
	memcpy(&hd.lineid[0],&linid,4);  
	sprintf(buff,"ssss");
   	memcpy(&hd.spname[0],&buff,4);
	sprintf(buff,"rrrr");
	memcpy(&hd.rstnam[0],&buff,4);

//...topcon file header contributions
	coordfac=10000.;
	if (fabs(sx)<coordfac && fabs(sy)<coordfac && fabs(sz)<coordfac)
	{
	//...shot coordinates scaled
	 hd.smul2=(short) -coordfac;
	 hd.ishlo=(int) (sx*coordfac);
	 hd.ishla=(int) (sy*coordfac);
	 hd.smul1=(short) -coordfac;
	 hd.ishe=(int) (sz*coordfac);
	//...receiver coordinates scaled
	 hd.irlo=(int) (rx[jtrace]*coordfac);
	 hd.irla=(int) (ry[jtrace]*coordfac);
	 hd.irel=(int) (rz[jtrace]*coordfac);
	}
	else
	{
	//...shot coordinates scaled
	coordfac=100.;
	 hd.smul2=(short) -coordfac;
	 hd.ishlo=(int) (sx*coordfac);
	 hd.ishla=(int) (sy*coordfac);
	 hd.smul1=(short) -coordfac;
	 hd.ishe=(int) (sz*coordfac);
	//...receiver coordinates scaled
	 hd.irlo=(int) (rx[jtrace]*coordfac);
	 hd.irla=(int) (ry[jtrace]*coordfac);
	 hd.irel=(int) (rz[jtrace]*coordfac);
	} //fi

//...range (often used by SU, Seismic Unix)
	 hd.idist=(int) sqrt( (rx[jtrace]-sx)*(rx[jtrace]-sx) +
			       (ry[jtrace]-sy)*(ry[jtrace]-sy) +
			       (rz[jtrace]-sz)*(rz[jtrace]-sz));

	 memcpy(&hd.lineid[0],&linenm,4);  
	 hd.shtazi=(short) isaz;
	 hd.shtver=(short) isvr;
	 hd.mst=(int) 0;
	 sprintf(buff,"%4.4d",ispn);
   	 memcpy(&hd.spname[0],&buff,4);
	 sprintf(buff,"%4.4d",vp[jtrace]);
	 memcpy(&hd.rstnam[0],&buff,4);
 
//...geophone polarizations  VSP Rules 
	 hd.geoazi=0;
	 hd.geover=0;	 /*Oyo SWC Phone GeoStuff Tool*/
//	 hd.geover=180;  /*Mark Products SWC Phone*/
	if(nchanl==6)
	{
	   if(ivpn>ivp1)
	  {
	    if ( jtrace==5-1 ) hd.geoazi=(short) irazi;
	    if ( jtrace==6-1 ) hd.geoazi=(short) itazi;
	    if ( jtrace==2-1 || jtrace==3-1) hd.geover=90;
	    if ( jtrace==5-1 || jtrace==6-1) hd.geover=90;
	  }
	  else
	  {
	    if ( jtrace==2-1 ) hd.geoazi=(short) irazi;
	    if ( jtrace==1-1 ) hd.geoazi=(short) itazi;
	    if ( jtrace==5-1 || jtrace==4-1) hd.geover=90;
	    if ( jtrace==2-1 || jtrace==1-1) hd.geover=90;
	  } //fi
	} //fi  nchanl 6

	if(nchanl==7)
	{
	   if(ivpn>ivp1)
	  {
	    if ( jtrace==5-1 ) hd.geoazi=(short) irazi;
	    if ( jtrace==6-1 ) hd.geoazi=(short) itazi;
	    if ( jtrace==2-1 || jtrace==3-1) hd.geover=90;
	    if ( jtrace==5-1 || jtrace==6-1) hd.geover=90;
	  }
	  else
	  {
	    if ( jtrace==3-1 ) hd.geoazi=(short) irazi;
	    if ( jtrace==2-1 ) hd.geoazi=(short) itazi;
	    if ( jtrace==5-1 || jtrace==6-1) hd.geover=90;
	    if ( jtrace==2-1 || jtrace==3-1) hd.geover=90;
	  } //fi
	} //fi  nchanl 7

//...output trace
            if(c_bsegout(jtrace,s1,npts,&hd,h2)!=0)
       {
        fprintf(stderr,"ABORT--c_bsegout");
        goto quit_it;
       }

//...display progress bar
      exbar(ntraces,bar,ibar,invbar,jtrace+1);
    
	} //next jtrace
  
//--------------------------------end trace loop---------------------------

      printf("\n   Listing ====>%s",lstfile);
      printf("\n   Output  ====>%s",ofile);
      printf("\n   Processed %d traces",ntraces);

quit_it:
        puts(" ");

	switch(idfc)
	{ case 1: {free(idata); break;}
	  case 2: {free(ldata); break;}
	  case 3: {free(cdata); break;}
	  case 4: {free(fdata); break;}
	  case 5: {free(ddata); break;}
	} //end switch
	return(0);

exit_chan:
	free(chan);
exit_rz:
	free(rz);
exit_ry:
	free(ry);
exit_rx:
	free(rx);
exit_s1:
	free(s1);
exit_2:
        fclose(h2);
exit_1:
        fclose(h1);
exit_4:
	fclose(h4);
exit_3:
        fclose(h3);

quick_exit:
	return(1);

     } //  end of main

/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: getparm.c                                   |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: gets input parameters from command                |
   |          line or prompts user from consol.                 |
   |                                                            |
   |                                                            |
   |------------------------------------------------------------|
*/
      int  getparm(int argc,char *argv[],
         char topf[], char infile[],
	 char linenm[], float *fshdp, int *ispn, int *nchanl, int *ivp1, 
	 int *ivpn, int *irec, float *eshift, int *isaz, int *isvr,
	 int *irazi, int *itazi, int *islop,
         char ofile[], char lstfile[],char pid[])
     {
      int               i,
			ii,
			nchar;
      char              sfx1[4]="seg",
			stmp[12],
                        sfx2[4]="lst";
      char		help[3]="-h";
      char              msg1[46],msg2[46];

//...test for command line help ?
      if(argc==2 && !strcmp(argv[1],help))
     {
//...screen header and copyright notice
      cr_labl(pid);

//...help pannel
      sprintf(msg1,"Convert EGG Geometrics Seismograph data files");
      sprintf(msg2,"from SEG-2 to BSEGY with Geometry Setting    ");
      hlp_labl(msg1,msg2);
      puts(" ");

//...help display
      puts(" ");
      printf("  topcon2  topf seg2f lid shdp is nch vpl vpn ir esh isa isv ira ita \n");
      puts(" ");
      puts("     topf   =  topcon file name");
      puts("     seg2f  =  seg-2 file name ");
      puts("     lid    =  line ID ");
      puts("     shdp   =  shot depth");
      puts("     is     =  shot location number");
      puts("     nch    =  number of channels (nch<66)");
      puts("     vp1    =  geophone station channel 1");
      puts("     vpn    =  geophone station channel n");
      puts("     ir     =  shot record number");
      puts("     esh    =  elevation adjustment to be added ");
      puts("     isa    =  source polarization azimuth (deg.)");
      puts("     isv    =  source polarization vertical (deg.)");
      puts("     ira    =  reference phone polarization R-axis (deg.)");
      puts("     ita    =  reference phone polarization T-axis (deg.)");
      puts(" ");
      return(1);
     }
      else
     {
//...get parameters------------------------------
//...pars off character parameters
      if(argc>1)
       strncpy(topf,argv[1],13);
       else
      {
       puts(" Enter topcon file name ");
       scanf(" %13s",topf);
      }
	if (namecheck(12,topf) != 0 ) return (1);


      if(argc>2)
       strncpy(infile,argv[2],13);
       else
      {
       puts(" Enter seg-2 data file name ");
       scanf(" %13s",infile);
      }
	if (namecheck(12,infile) != 0 ) return (1);

      if(argc>3)
       strcpy(linenm,argv[3]);
       else
      {
       puts(" Enter line or well name (4char)");
       scanf(" %4s",linenm);
      }

      if(argc>4)
       *fshdp=atof(argv[4]);
       else
      {
       puts(" Enter shot depth (float)");
       scanf(" %f",fshdp);
      }

      if(argc>5)
       *ispn=atoi(argv[5]);
       else
      {
       puts(" Enter shot station SP Number(int)");
       scanf(" %d",ispn);
      }

      if(argc>6)
       *nchanl=atoi(argv[6]);
       else
      {
       puts(" Enter number of channels (int)");
       scanf(" %d",nchanl);
      }

      if(argc>7)
       *ivp1=atoi(argv[7]);
       else
      {
       puts(" Enter VP Number first channel (int)");
       scanf(" %d",ivp1);
      }

      if(argc>8)
       *ivpn=atoi(argv[8]);
       else
      {
       puts(" Enter VP Number last channel (int)");
       scanf(" %d",ivpn);
      }

//check for consistency
	ii=*ivpn-*ivp1;
	ii=ii/abs(ii);
	*islop=(*ivpn-*ivp1+ii)/(*nchanl);
	if (abs(*islop)!=1)
	{ fprintf(stderr,"ABORT!! Inconsistent nchanl,ivpn,ivp1");
	  return(1);
	}

      if(argc>9)
       *irec=atoi(argv[9]);
       else
      {
       puts(" Enter sequential record number (int)");
       scanf(" %d",irec);
      }

      if(argc>10)
       *eshift=atof(argv[10]);
       else
      {
       puts(" Enter elevation datum shift (float)");
       scanf(" %f",eshift);
      }

      if(argc>11)
       *isaz=atoi(argv[11]);
       else
      {
       puts(" Enter source polarization azimuth degrees (int)");
       scanf(" %d",isaz);
      }

      if(argc>12)
       *isvr=atoi(argv[12]);
       else
      {
       puts(" Enter source polarization from vertical degrees (int)");
       scanf(" %d",isvr);
      }

      if(argc>13)
       *irazi=atoi(argv[13]);
       else
      {
       puts(" Enter Reference Phone Radial Azimuth degrees (int)");
       scanf(" %d",irazi);
      }

      if(argc>14)
       *itazi=atoi(argv[14]);
       else
      {
       puts(" Enter Reference Phone Transverse Azimuth degrees (int)");
       scanf(" %d",itazi);
      }

//...name output files
	nchar=0;
     for (i=0;i<8;i++)
	{
 	  if(infile[i]!='.') 
	  { stmp[i]=infile[i];
	    nchar=nchar+1; }
	else
	  goto label1;
	}	//next i
label1:
     stmp[nchar]=0;
     sprintf(ofile,"%s.%s",stmp,sfx1);    
     sprintf(lstfile,"%s.%s",stmp,sfx2); 
       return(0);
     }  //  endif bhelp test
      } // end getparm

/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: outlst.c                                    |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: prints copyright header, file names, and          |
   |          echo check of input parameters in newly           |
   |          opened output listing file.                       |
   |                                                            |
   |------------------------------------------------------------|
*/
      void outlst(char infile[],
        char pid[], char ofile[], char lstfile[], FILE *h3)
     {

//...print copyright label to outlst and screen

        fcr_labl(pid,h3);

//...echo check
      fputs("\n",h3);                      //blank line
      fprintf(h3,"\n     PROGRAM: %s.c ",pid);
      fputs("\n",h3);                      //blank line
      fputs("\n     FILES:       ",h3);
      fprintf(h3,"\n     Input   File= %s",infile);
      fprintf(h3,"\n     Output  File= %s",ofile);
      fprintf(h3,"\n     Listing File= %s",lstfile);
      fputs("\n",h3);                      //blank line
      fputs("\n     INPUT PARAMETERS: ",h3);
      fprintf(h3,"\n     none ");
      fputs("\n",h3);                      //blank line
     }  //  end outlst
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: readfdb.c                                   |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: reads File Descriptor Block   FDB                 |
   |          ntraces= number of traces in file                 |
   |          trptr  = array of pointers, start of each         |
   |                   trace descriptor block                   |
   |                                                            |
   |------------------------------------------------------------|
*/
      int   readfdb(int *ntraces, unsigned int trptr[],
	short *tyear, short *tday, short *thour, short *tmin, short *tsec,
        FILE *h1, FILE *h3)
     {
	int			exstat,		//exit status 0=error
				j;
	unsigned short		fdbid,		//id code for SEG-2
				szptrblk,	//size trace ptr block
				ntrc,		//number of traces
				strl;
	unsigned char		fststerm,	//first string term. char
				scndsterm,	//second string term. char
				fstlterm,	//first line term. char
				scndlterm,	//second line term. char
				jchar;
	char			strg[80];
	char			key1[]="ACQUISITION_DATE",
				key2[]="ACQUISITION_TIME",
				amonth[4]="   \0";
	char			*loc1,
				*loc2;
	unsigned int		j1,
				j2,
				jj;
	short			tmonth;
	short			doy;

//...read ID code, Test if SEG-2?
	exstat=0;
	if(fread(&fdbid, sizeof(short), 1, h1) != 1)
	{
	fprintf(stderr,"Error reading SEG-2 Header");
	return(0);
	}	//endif
	if(fdbid==0X3A55)
	fprintf(h3,"\n Confirmed SEG-2 Data: %#X \n",fdbid);
	else
	{
	fprintf(h3,"\n ABORT!! Not SEG-2 Data: %#X \n",fdbid);
	return(0);
	}

//...echo check of FDB
	fprintf(h3,"\n");
	for (jj=0; jj<72; jj++) fprintf(h3,"-");
	fprintf(h3,"\n \nFILE DESCRIPTOR BLOCK\n");	
	for (jj=0; jj<72; jj++) fprintf(h3,"-");
	fprintf(h3,"\n");

	fseek(h1,2,SEEK_CUR);
	exstat=fread(&szptrblk, sizeof(short), 1, h1);
	fprintf(h3," Size of trace pointer block = %d \n",szptrblk);

	exstat=fread(&ntrc, sizeof(short), 1, h1);
	fprintf(h3," Number of traces = %d \n",ntrc);
	*ntraces=ntrc;

	fseek(h1,1,SEEK_CUR);
	exstat=fread(&fststerm, sizeof(char), 1, h1);
	fprintf(h3," first string terminator character = %#X \n",fststerm);

	exstat=fread(&scndsterm, sizeof(char), 1, h1);
	fprintf(h3," second string terminator character = %#X \n",scndsterm);

	fseek(h1,1,SEEK_CUR);
	exstat=fread(&fstlterm, sizeof(char), 1, h1);
	fprintf(h3," first line terminator character = %#X \n",fstlterm);

	exstat=fread(&scndlterm, sizeof(char), 1, h1);
	fprintf(h3," second line terminator character = %#X \n",scndlterm);

	fseek(h1,0X20,SEEK_SET);
	for (j=0;j<ntrc;j++)
	{
	exstat=fread(&trptr[j], sizeof(int), 1, h1);
//	fprintf(h3," trace pointer (%d) = %#lX \n",j,trptr[j]);
	fprintf(h3," trace pointer (%d) = %#X \n",j,trptr[j]);
	}  //next j

//...get time and date strings
	j1=ftell(h1);
	j2=trptr[0];
        for (jj=j1;jj<j2;jj++)
	{
	exstat=fread(&jchar, sizeof(char), 1, h1);
	if(jchar!='\0') goto found1;
	}  //next jj
	return(0);
found1:  
	fseek(h1,-1,SEEK_CUR);
   	exstat=fread(&strl, sizeof(short), 1, h1);
	exstat=fread(&strg, sizeof(char),strl, h1);
	fprintf(h3," number of characters=%d \n",strl);
	fprintf(h3," %s \n",strg);	
         if (strl>16 && strncmp(key1,strg,16)==0)
	  {	*tday=atoi(&strg[16]);
		loc1=strchr(&strg[0],'/');
		loc2=strchr(loc1+1,'/');
		*tyear=atoi(loc2+1);
		strncpy(amonth,loc1+1,3);
                tmonth=month2num(amonth);
                dmy2dy(tday,&tmonth,tyear,&doy); 
/* doy = day of year  tmonth=numeric month tday=numeric day
		Tue Aug 16 14:03:28 MDT 2016 CHANGE by PM
header: hd.tday
                Problem: bis2seg puts out hd.tday header as monthday integer
                and this conflicts with egg2seg.c and topcon2.c which has put
                out hd.tday header as dayofyear integer  
		This change brings egg2seg.c match bis2seg.c in function.
                bis2seg acts on bison files of which there are many
                topcon2 acts on seg2  files (so will make bison and seg2 produce
		the same header hd.tday)
*/
//		*tday=doy;
		*tday=tmonth*100+*tday;
		fprintf(h3," Date (dayofyear.year): %d.%d\n",doy,*tyear);
	  }  //fi
	
	fseek(h1,-2,SEEK_CUR);
   	exstat=fread(&strl, sizeof(short), 1, h1);
	fprintf(h3," number of characters=%d \n",strl);
	exstat=fread(&strg, sizeof(char),strl, h1);
	fprintf(h3," %s \n",strg);	

         if (strl>16 && strncmp(key2,strg,16)==0)
	  {	*thour=atoi(&strg[16]);
		loc1=strchr(&strg[0],':');
		loc2=strchr(loc1+1,':');
		*tmin=atoi(loc1+1);
		*tsec=atoi(loc2+1);
		fprintf(h3," TIME: %d %d %d \n",*thour,*tmin,*tsec);
	  }  //fi
	
	for (jj=0; jj<72; jj++) fprintf(h3,"-");
	fprintf(h3,"\n \nTRACE DESCRIPTOR BLOCKS\n");	
	for (jj=0; jj<72; jj++) fprintf(h3,"-");
	fprintf(h3,"\n");

	return(exstat);
     }  //end readfdb

/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: dmy2dy.c                                    |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: convert day,month,year to day of the year         |
   |                                                            |
   |------------------------------------------------------------|
*/
     void  dmy2dy(short *day, short *month, short *year, 
        short *doy)
    {
	int		days[12]={31,28,31,30,31,30,31,31,
				  30,31,30,31} ;
	int		j;
	short		sum=0;

//...sum up days, non leap year
	for (j=1; j<*month; j++) sum=sum+days[j-1]; 
	sum=sum+*day;

//...figure leap year
	if ((*year % 4)==0 && *month >=2 ) sum=sum+1;
	*doy= sum;

    }  //end dmy2dy

/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: month2num.c                                 |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: convert ascii month to integer                    |
   |                                                            |
   |------------------------------------------------------------|
*/
      int  month2num(char amonth[])
     {
	 if (strcmp(amonth,"JAN")==0) return(1); 
	 if (strcmp(amonth,"FEB")==0) return(2); 
	 if (strcmp(amonth,"MAR")==0) return(3); 
	 if (strcmp(amonth,"APR")==0) return(4); 
	 if (strcmp(amonth,"MAY")==0) return(5); 
	 if (strcmp(amonth,"JUN")==0) return(6); 
	 if (strcmp(amonth,"JUL")==0) return(7); 
	 if (strcmp(amonth,"AUG")==0) return(8); 
	 if (strcmp(amonth,"SEP")==0) return(9); 
	 if (strcmp(amonth,"OCT")==0) return(10); 
	 if (strcmp(amonth,"NOV")==0) return(11); 
	 if (strcmp(amonth,"DEC")==0) return(12); 
	 if (strcmp(amonth,"Jan")==0) return(1); 
	 if (strcmp(amonth,"Feb")==0) return(2); 
	 if (strcmp(amonth,"Mar")==0) return(3); 
	 if (strcmp(amonth,"Apr")==0) return(4); 
	 if (strcmp(amonth,"May")==0) return(5); 
	 if (strcmp(amonth,"Jun")==0) return(6); 
	 if (strcmp(amonth,"Jul")==0) return(7); 
	 if (strcmp(amonth,"Aug")==0) return(8); 
	 if (strcmp(amonth,"Sep")==0) return(9); 
	 if (strcmp(amonth,"Oct")==0) return(10); 
	 if (strcmp(amonth,"Nov")==0) return(11); 
	 if (strcmp(amonth,"Dec")==0) return(12); 
	 return(0);
     }  //end month2num
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: readtdb.c                                   |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: reads Trace Descriptor Block  TDB                 |
   |                                                            |
   |------------------------------------------------------------|
*/
      int   readtdb(int jtrace, unsigned int trptr[],unsigned int *nsamp,
        unsigned short *sztdblk, unsigned char *dfc,
	float *fsamin, float *sfactr, int *chnl, int *gain,
	float *delay, int *hcf, int *lcf, int *rawrec, int *stack,
        float *skew, float *recx, float *sourcex, char *linid,
        float *notchf, FILE *h1, FILE *h3)
     {
	int			exstat,
				j;
	unsigned short		tdbid,		//tdb id code
				strl;

	unsigned int		szdblk;		//size of data block bytes
	char			strg[80];
	char			key1[]="SAMPLE_INTERVAL",
				key2[]="DESCALING_FACTOR",
				key3[]="CHANNEL_NUMBER",
				key4[]="FIXED_GAIN",
				key5[]="DELAY",
				key6[]="HIGH_CUT_FILTER",
				key7[]="LOW_CUT_FILTER",
				key8[]="SHOT_SEQUENCE_NUMBER",
				key9[]="STACK",
				key10[]="SKEW",
                                key11[]="RECEIVER_LOCATION",
                                key12[]="SOURCE_LOCATION",
                                key13[]="LINE_ID",
                                key14[]="NOTCH_FREQUENCY";
			//Geode now uses different key 8
			//	key8[]="RAW_RECORD",

	fseek(h1,trptr[jtrace],SEEK_SET);
//...initialize values in case keywords missing
        *sfactr=0.0;
        *chnl=0;
        *gain=0;
        *delay=0.0;
        *hcf=0;
        *lcf=0;
        *rawrec=0;
        *stack=0;
        *skew=0.0;
        *recx=0.0;
        *sourcex=0.0;
        sprintf(linid,"xxxx");
        *notchf=0.0;

//...read ID code, Test if TDB?
	exstat=0;
	if(fread(&tdbid, sizeof(short), 1, h1) != 1)
	{
	fprintf(stderr,"Error reading SEG-2 Header");
	return(0);
	}	//endif
	if(tdbid!=0X4422)
	{
	fprintf(stdout,"\n ABORT!! Not SEG-2 Data: %#X \n",tdbid);
	return(0);
	}
//	else
//	{
//	fprintf(stdout,"\n Confirmed SEG-2 Data: %#X \n",tdbid);
//	}

//...echo check TDB
	
	exstat=fread(sztdblk, sizeof(short), 1, h1);
//	fprintf(stdout,"Size of TDB=%#X \n",*sztdblk);

	exstat=fread(&szdblk, sizeof(int), 1, h1);
//	fprintf(stdout,"Size of Data Block=%#lX \n",szdblk);

	exstat=fread(nsamp, sizeof(int), 1, h1);
//	fprintf(stdout,"Number of samples per trace=%ld \n",*nsamp);

	exstat=fread(dfc, sizeof(char), 1, h1);
//	fprintf(stdout,"Data Format Code=%#X \n",*dfc);

	fseek(h1,21,SEEK_CUR);
//--------------loop on keywords-------------------------
	for (j=0; j<20; j++)
	{
	fseek(h1,-2,SEEK_CUR);
   	exstat=fread(&strl, sizeof(short), 1, h1);
//	fprintf(stdout," number of characters=%d \n",strl);
	if(strl==0) break;
	exstat=fread(&strg, sizeof(char),strl, h1);
         if (strl>15 && strncmp(key1,strg,15)==0)  *fsamin=atof(&strg[15]);
         if (strl>16 && strncmp(key2,strg,16)==0)  *sfactr=atof(&strg[16]);
         if (strl>14 && strncmp(key3,strg,14)==0)  *chnl=atoi(&strg[14]);
         if (strl>10 && strncmp(key4,strg,10)==0)  *gain=atoi(&strg[10]);
         if (strl>5 && strncmp(key5,strg,5)==0)    *delay=atof(&strg[5]);
         if (strl>15 && strncmp(key6,strg,15)==0)  *hcf=atoi(&strg[15]);
         if (strl>14 && strncmp(key7,strg,14)==0)  *lcf=atoi(&strg[14]);
         //if (strl>10 && strncmp(key8,strg,10)==0)  *rawrec=atoi(&strg[10]);
         if (strl>20 && strncmp(key8,strg,20)==0)  *rawrec=atoi(&strg[20]);
         if (strl>5 && strncmp(key9,strg,5)==0)    *stack=atoi(&strg[5]);
         if (strl>4 && strncmp(key10,strg,4)==0)    *skew=atof(&strg[4]);

         if (strl>4 && strncmp(key11,strg,17)==0)    *recx=atof(&strg[17]);
         if (strl>4 && strncmp(key12,strg,15)==0)    *sourcex=atof(&strg[15]);
         if (strl>4 && strncmp(key13,strg,7)==0)     strncpy(&linid[0],&strg[7],4);
         if (strl>4 && strncmp(key14,strg,15)==0)    *notchf=atof(&strg[15]);
	  
	fprintf(h3," %s \n",strg);	
	if (ftell(h1)>(trptr[jtrace]+*sztdblk) ) break;
	}  //next j
//---------------------------------------------------------
	return(exstat);
     }  //end readtdb

/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: readata.c                                   |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: reads a data trace                                |
   |                                                            |
   |------------------------------------------------------------|
*/
      int   readata(int jtrace, unsigned int trptr[], int sztdblk,
      int nsamp, int idfc, void *data, float sfactr, float *s1, FILE *h1)
      {
	int		exstat,
			j,k,i;
        unsigned int   nbytes, spointr;
        unsigned int    xponent;
        int            ldata;  
        short int *ibuf = (short int *) data;
        short int       sdata; 
//...begining of data
	exstat=0;
	switch (idfc)
       {
	case 1:
	{fseek(h1,(trptr[jtrace]+sztdblk),SEEK_SET);
   	exstat=fread((int *)data, sizeof(int), nsamp, h1);
	for (j=0; j<nsamp; j++) s1[j]=((float) ( ((int *)data)[j]))*sfactr;
	break;}  // 1 esac
	case 2:
	{fseek(h1,(trptr[jtrace]+sztdblk),SEEK_SET);
   	exstat=fread((int *)data, sizeof(int), nsamp, h1);
	for (j=0; j<nsamp; j++) s1[j]=((float) ( ((int *)data)[j]))*sfactr;
	break;}  // 2 esac
        case 3:
        {fseek(h1,(trptr[jtrace]+sztdblk),SEEK_SET);
        nbytes=(nsamp*5)/2;
        exstat=fread((unsigned char *)data, 1, nbytes, h1);
/*  seg2segy.c inspired code is buggy for large negative values
        for (k=0; k<nsamp; ) {
                spointr=(k/4)*5;
                xponent=(unsigned) ibuf[spointr++];
                for (i=0; i<4; i++) {
                        if(0x8000 & ibuf[spointr])
                                ldata=0xffff8000;
                        else
                                ldata=01;
        ldata=ldata | ((int) ibuf[spointr++] << (0x000f & xponent));
                xponent >>= 4;
                s1[k++]=((float) ldata)*sfactr;
                } //next i
        } //next k
*/
//PM's version works on large negative values (no sawtooth error)
        for (k=0; k<nsamp; ) {
                spointr=(k/4)*5;
                xponent=(unsigned) ibuf[spointr++];
                for (i=0; i<4; i++) {
                        sdata=ibuf[spointr++];
                        if (sdata>0)
                         {
                                ldata=(int) sdata;
                                ldata=ldata << (0x000f & xponent);
                         }
                        else
                         {
                                ldata=(int) -sdata;
                                ldata=ldata <<  (0x000f & xponent);
                                ldata = -ldata;
                          } //endif
                xponent >>= 4;
                s1[k++]=((float) ldata)*sfactr;
                // s1[k++]=((float) ldata); //to match seg2segy scaling
                } //next i
        } //next k
        break;}  // 3 esac
	case 4:
	{fseek(h1,(trptr[jtrace]+sztdblk),SEEK_SET);
   	exstat=fread((float *)data, sizeof(float), nsamp, h1);
	for (j=0; j<nsamp; j++) s1[j]=( ((float *)data)[j] )*sfactr;
	break;}  // 4 esac
	case 5:
	{fseek(h1,(trptr[jtrace]+sztdblk),SEEK_SET);
   	exstat=fread((double *)data, sizeof(double), nsamp, h1);
	for (j=0; j<nsamp; j++) s1[j]=( (float) ( ((double *)data)[j]))*sfactr;
	break;}  // 5 esac

       }  //end switch

      return(exstat);

      } //end readata

/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: readtop.c                                   |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: reads a topcon file                               |
   |                                                            |
   |------------------------------------------------------------|
*/

     int readtop(float *sx, float *sy, float *sz,
	float rx[], float ry[], float rz[], int chan[], int vp[],
	int islop, int ispn, int ivp1, float eshift, int nchanl, 
	FILE *h4) 
     {
	#define BUFSIZE 80
	char			buf[BUFSIZE];
	int			jrec,
				ichan2,
				jcount,
				j,
				k,
				k1,k2,
				ibomb,
				shfound,
				ichan;
	float			ycoord,
				xcoord,
				fi,
				elev;
	char			label[13];

	jrec=-1;
	ibomb=0;
//clear chan array
	for(j=0; j<nchanl; j++) chan[j]=0;
//clear shot coord
	*sx=0.;
	*sy=0.;
	*sz=0.;
	shfound=0;

//----------------------------------loop
	while (!feof(h4))
	{
	jrec=jrec+1;
	fgets(buf, BUFSIZE, h4);
	if(feof(h4)) { jrec=jrec-1; break;}
	sscanf(buf,"%d%f%f%f%s",&jcount,&ycoord,&xcoord,&elev,label);
	if (strncmp(label,"SP",2)==0 || strncmp(label,"sp",2)==0)
	{
	 ichan=atoi(&label[2]);
 	  if(ispn==ichan)
	  {
	   *sx=xcoord;
	   *sy=ycoord;
	   *sz=elev;
	   shfound=1;
	  }  //fi
//  	fprintf(stdout," ichan=%d sx=%f sy=%f sz=%f\n",ichan,*sx,*sy,*sz);
	}  //fi

	if (strncmp(label,"VP",2)==0 || strncmp(label,"vp",2)==0)
	{
	 ichan=atoi(&label[2]);
	 ichan2=(ichan-ivp1)/islop+1;
	 if(ichan2<=nchanl && ichan2>=1) 
	  {
	   chan[ichan2-1]=ichan;
	   rx[ichan2-1]=xcoord;
	   ry[ichan2-1]=ycoord;
	   rz[ichan2-1]=elev;
	   vp[ichan2-1]=ichan;
	  }  //fi
	}  //fi
	}  //end while
//---------------------------------end loop

//...test for missing points, interpolate if can and needed
	for (j=1; j<nchanl+1; j++)
       {
	if (chan[j-1]==0)
	 {	 
	  fprintf(stdout,"WARNING: Channel %d Missing\n",j);
	  k1=j-1;
	  k2=j+1;
	  if (k1<=nchanl && k1>=1 && k2<=MAXCH && k2>=1)
	   {
		for (k=1; k<(nchanl+1); k++)
		{
		 if(chan[k-1]==0) 
		   k1=k1-1;
		 else
		  break; 
		} // next k
		
		for (k=1; k<(nchanl+1); k++)
		{
		  if(chan[k-1]==0)
		   k2=k2+1;
		 else
		   break;
		}  //next k
		fprintf(stdout," k1,j,k2= %d %d %d\n",k1,j,k2);

//...interpolate, but only if bracketed
		if (k1<=nchanl && k1>=1 && k2<=nchanl && k2>=1)
		{
 	 	 chan[j-1]=j;
		 rx[j-1]=rx[k1-1]+((float)(j-k1)/(float)(k2-k1))*(rx[k2-1]-rx[k1-1]);
		 ry[j-1]=ry[k1-1]+((float)(j-k1)/(float)(k2-k1))*(ry[k2-1]-ry[k1-1]);
		 rz[j-1]=rz[k1-1]+((float)(j-k1)/(float)(k2-k1))*(rz[k2-1]-rz[k1-1]);
		 fi=0.5*(float) (vp[k2-1]-vp[k1-1])/abs(vp[k2-1]-vp[k1-1]); 
		 vp[j-1]=vp[k1-1]+floor( 
		   ((float)(j-k1)/(float)(k2-k1))*(float)(vp[k2-1]-vp[k1-1])+fi );
		}
		else
		{
		 fprintf(stdout,"ABORT: Unable to Interpolate, must edit *.nez file\n");
		 fprintf(stdout," Edit channel number %d \n",j);
		 ibomb=1;
		 chan[j-1]=j;
	         vp[j-1]=j;
		} //fi
	   }
	   else
   	   {
	    fprintf(stdout,"ABORT: Unable to Interpolate, must edit *.nez file\n");
	    fprintf(stdout," Edit channel number %d \n",j);
	    ibomb=1;
	    chan[j-1]=j;
	    vp[j-1]=j;
	   }  //fi k1,k2

	 }  //fi chan 
       }  //next j

//...apply elevation datum shift
	*sz=*sz+eshift;

/*
	for (j=0; j<nchanl; j++) rz[j]=rz[j]+eshift;
        for(jcount=0;jcount<nchanl;jcount++)
        {
        fprintf(stdout,"vp=%d chan=%d rx=%f ry=%f rz=%f\n",vp[jcount],chan[jcount],
        rx[jcount],ry[jcount],rz[jcount]);
        }
        fprintf(stdout," jrec=%d\n",jrec);
*/
	if(shfound==0)
	{fprintf(stdout,"ABORT: Shot Coord. Not Found");
	 ibomb=1;
	}  //fi

	if (ibomb==0)
	return(0);
	else
	return(1);
     }  //end readtop
 
