c |$Id: bcrd.f,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $          |
c --------------------------------------------------------------
c | Basic Seismic Utilities (BSU)                              |
c | LANGUAGE: Fortran77                                        |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: bcrd.f                                       |
c | DIRECTORY: bsu-2.0.2/src/Fort/bsegy                        |
c |------------------------------------------------------------|
c | bcrd Applies a coordinate transformation to header xyz data|
c | Rotation in X-Y plane only                                 |
c |------------------------------------------------------------|
c |  NOTICE:                                                   |
c |  Copyright (C) 2017 Paul Michaels                          |
c |  <pm@cgiss.boisestate.edu>                                 |
c |  This program is free software; you can                    |
c |  redistribute it and/or modify it under the terms          |
c |  of the GNU General Public License as published            |
c |  by the Free Software Foundation; either version           |
c |  3 of the License, or (at your option) any later           |
c |  version.  This program is distributed in the              |
c |  hope that it will be useful, but                          |
c |  WITHOUT ANY WARRANTY; without even the implied            |
c |  warranty of MERCHANTABILITY or FITNESS FOR A              |
c |  PARTICULAR PURPOSE.  See the GNU General Public           |
c |  License for more details.                                 |
c |  You should have received a copy of the GNU                |
c |  General Public License along with this program;           |
c |  if not, write to the Free Software Foundation,            |
c |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
c |------------------------------------------------------------|
c |  PARAMETERS:                                               |
c |  ndim = maximum number of samples in a seismic trace       |
c |------------------------------------------------------------|
c |  INCLUDE FILE REQUIRED:                                    |
c |  ../include/bsegy.inc    (modified segy header definitions)|
c |                (no reel header, floating point data)       |
c |------------------------------------------------------------|
c |  SUBROUTINES CALLED:                                       |
c | In-line===============                                     |
c |     GETPRM  get input parameters from command line or      |
c |             interactive query.  Should be inline with      |
c |             main program, specific to application.         |
c |     LSTPRM  echo check of input parameters from GETPRM.    |
c |             Values written to a history file (*.lst)       |
c | Located in bsu-2.0.2/src/sublibF4===========               |
c |     CHKTRC  get number of samples, sample interval, and    |
c |             number of traces in the input data file.       |
c |             Also returns values for progress bar, stdout   |
c |     BSEGIN  Reads a seismic trace.                         |
c |     BSEGOUT Write a seismic trace.                         |
c |     PLTBAR  Writes progress bar on stdout                  |
c |                                                            |
c |                                                            |
c --------------------------------------------------------------
c
      include '../include/bsegy.inc'
      parameter (ndim=32767)
c
      real*4            s1(ndim)
      real*8            theta,sfact,x0,y0,z0,cse,sne
      real*8            rx,ry,rz,rxnew,rynew,rznew
      real*8            sx,sy,sz,sd,sxnew,synew,sznew,sdnew
      character*4       aplc
      character*40      infil, outfil, outlst
      common /header/ hd
c
c...assign I/O units, initialize....
      jrec=1
      iunit1=1
      iunit2=2
      icall1=0
      icall2=0
c
c...define application prefix
      write(aplc,'(''bcrd'')')
c
c...get parameters-------------------------------------------
      call GETPRM(nargsx,infil,
     +theta,sfact,x0,y0,z0)
c
c...count traces and check dimensions------------------------
      call CHKTRC(iunit1,icall1,npts,s1,fsamin,infil,
     +bar,invbar,ibar,ntrace,ndim)
c
c...name outputfile
      write(outfil,'(a,a,''.seg'')') aplc(1:4),infil(1:4)
c
c...output listing-------------------------------------------
      call LSTPRM(io11,ntrace,aplc,outlst,outfil,
     +infil,theta,sfact,x0,y0,z0)
c
c...clear output file...
      open(10,file=outfil,access='direct',recl=240)
      close(10,status='delete')
c
      icall1=0
      icall2=0
      ntr=0
c
c...set up rotation matrix
      cse=cos(theta*(3.1415926/180.))*sfact
      sne=sin(theta*(3.1415926/180.))*sfact
c
c...data loop=======================================....
c
      do 100 jrec=1,ntrace
c
c...input a trace
      CALL BSEGIN(iunit1,ndim,
     +icall1,npts,s1,fsamin,infil,jrec,iexit)
c
      ntr = ntr + 1
c
      call findxyz(sx4,sy4,sz4,sd4,rx4,ry4,rz4,range4,rangex4)
      sx=dble(sx4)
      sy=dble(sy4)
      sz=dble(sz4)
      rx=dble(rx4)
      ry=dble(ry4)
      rz=dble(rz4)
      sd=dble(sd4)
c
c...convert coordinates
      rxnew=cse*rx+sne*ry+x0
      rynew=-sne*rx+cse*ry+y0
      rznew=rz*sfact+z0
c
      sxnew=cse*sx+sne*sy+x0
      synew=-sne*sx+cse*sy+y0
      sznew=sz*sfact+z0
      sdnew=sd*sfact
c
c...update headers with new coordinates
      smul1=-100.
      smul2=-10.
      if(smul2.gt.0) then
      ishlo=int(sxnew/smul2)
      ishla=int(synew/smul2)
      irlo=int(rxnew/smul2)
      irla=int(rynew/smul2)
      else
      ishlo=int(-sxnew*smul2)
      ishla=int(-synew*smul2)
      irlo=int(-rxnew*smul2)
      irla=int(-rynew*smul2)
      endif
      if(smul1.gt.0) then
      ishe=sznew/smul1
      ishd=sdnew/smul1
      irel=rznew/smul1
      else
      ishe=-sznew*smul1
      ishd=-sdnew*smul1
      irel=-rznew*smul1
      endif
      range=sqrt( (sxnew-rxnew)**2+(synew-rynew)**2+
     + (rznew-sznew+sdnew)**2 )
      idist=range
c
c
c...output trace
      iunit2=2
      CALL BSEGOUT(iunit2,
     +icall2,npts,s1,fsamin,outfil,jrec)
c
c...display progress....
      call pltbar(bar,invbar,ibar,jrec,ntrace)
c
c...END LOOP========================================....
  100 continue
c
      write(*,'('' '')')
      write(6,240) ntrace,outfil,outlst
240   format(/' Traces Processed = ',i5/
     +5x,'====> output in ',a/'====> listing in ',a)
c
c
      close(io11)
      close(iunit1)
      close(iunit2)
      stop
      end
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: getprm.f                                     |
c |------------------------------------------------------------|
c | get input parameters or help                               |
c --------------------------------------------------------------
c
      subroutine GETPRM(nargsx,infil,
     +theta,sfact,x0,y0,z0)
c
      character*40      infil
      character*40      arg1,arg2,arg3,arg4,arg5,arg6
      real*8            theta,sfact,x0,y0,z0
c
c...get input/output file names...
c   |--------------------------------------------|
c   | file naming convention                     |
c   | application prefix 4 characters "bxxx"     |
c   | data set suffix 4 characters "dddd"        |
c   | file name = bxxxdddd.seg                   |
c   |--------------------------------------------|
      write(*,*) '|---------------------------------|'
      write(*,*) '| Copyright (C) 2017  P. Michaels |'
      write(*,*) '|       All rights reserved       |'
      write(*,*) '|see GNU General Public License   |'
      write(*,*) '|---------------------------------|'
c
ccc---LINUX VERSION
      nargsx=iargc()
      karg=-1
c...test for help
      if(nargsx.ge.(2+karg)) then
      call getarg(1,arg1)
      if(arg1(1:2).eq.'-h') then
      write(*,*) ' '
      write(*,*) '|------------------------------------------|'
      write(*,*) '| Basic Seismic Utilities     FORTRAN      |'
      write(*,*) '| ONLINE HELP:                             |'
      write(*,*) '|------------------------------------------|'
      write(*,*) '| bcrd: Transform geometry with coordinate |'
      write(*,*) '| rotation and translation                 |'
      write(*,*) '|------------------------------------------|'
      write(*,*) ' '
      write(*,*)
     +'bcrd infile theta sfact x0 y0 z0'
      write(*,*) ' '
      write(*,*) 'infile  =input file name '
      write(*,*) 'theta   =angle (deg) from current to new x-axis '
      write(*,*) '   (+)theta counterclockwise (-)theta clockwise'
      write(*,*) 'sfact   =units scale factor: (old)*(sfact)=(new) '
      write(*,*) 'x0      =X-offset in new coord. units '
      write(*,*) 'y0      =Y-offset in new coord. units '
      write(*,*) 'zo      =Z-offset in new coord. units '
      write(*,*) ' '
      stop
      endif
      endif
      if(nargsx.lt.(2+karg)) then
      write(*,*) '  enter: name of input file'
      read(*,'(a13)') infil
      else
      call getarg(1,infil)
      endif
      call namechk(12,infil)
c
      if(nargsx.lt.(3+karg)) then
      write(*,*) '  enter: Rotation Angle, theta '
      write(*,*) 'theta   =angle (deg) from current to new x-axis '
      write(*,*) '   (+)theta counterclockwise (-)theta clockwise'
      read(*,*) theta
      else
      call getarg(2,arg2)
      read(arg2,'(f16.0)') theta
      endif
c
      if(nargsx.lt.(4+karg)) then
      write(*,*) '  enter: Units scale factor, sfact'
      write(*,*) '  (old)*(sfact)=(new) '
      read(*,*) sfact
      else
      call getarg(3,arg3)
      read(arg3,'(f16.0)') sfact
      endif
c
      if(nargsx.lt.(5+karg)) then
      write(*,*) '  enter: Offset X0'
      read(*,*) x0
      else
      call getarg(4,arg4)
      read(arg4,'(f16.0)') x0
      endif
c
      if(nargsx.lt.(6+karg)) then
      write(*,*) '  enter: Offset Y0'
      read(*,*) y0
      else
      call getarg(5,arg5)
      read(arg5,'(f16.0)') y0
      endif
c
      if(nargsx.lt.(7+karg)) then
      write(*,*) '  enter: Offset Z0'
      read(*,*) z0
      else
      call getarg(6,arg6)
      read(arg6,'(f16.0)') z0
      endif
c
      return
      end
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: lstprm.f                                     |
c |------------------------------------------------------------|
c | echo check of input parameters, writes to a history file   |
c --------------------------------------------------------------
c
      subroutine LSTPRM(io11,ntrace,aplc,outlst,outfil,
     +infil,theta,sfact,x0,y0,z0)
      character*4       aplc
      character*40      outlst,outfil,infil
      real*8            theta,sfact,x0,y0,z0
c
c...open output listing file...
      io11=11
      write(outlst,'(a,a,''.lst'')') aplc,infil(1:4)
      open(io11,file=outlst,access='sequential')
      write(io11,*) '|---------------------------------|'
      write(io11,*) '| Copyright (C) 2017  P. Michaels |'
      write(io11,*) '|       All rights reserved       |'
      write(io11,*) '|see GNU General Public License   |'
      write(io11,*) '|---------------------------------|'
c
      write(io11,'('' Program bcrd  '')')
      write(io11,'('' Input File: '',a)') infil
      write(io11,'('' Output File: '',a)') outfil
c
      write(io11,'(''  Number of traces='',i5)') ntrace
      write(io11,'(''  Parameters:  '')')
      write(io11,'(''   theta= '',f16.6)') theta
      write(io11,'(''   sfact= '',f16.6)') sfact
      write(io11,'(''   x0= '',f16.6)') x0
      write(io11,'(''   y0= '',f16.6)') y0
      write(io11,'(''   z0= '',f16.6)')    z0
      write(io11,'(50(1h-))')
      write(io11,'(''    '')')
c
      return
      end
