c |$Id: bdcn.f,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $          |
c --------------------------------------------------------------
c | Basic Seismic Utilities (BSU)                              |
c | LANGUAGE: Fortran77                                        |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: bdcn.f                                       |
c | DIRECTORY: bsu-2.0.2/src/Fort/bsegy                        |
c |------------------------------------------------------------|
c |      Wiener Least Squares Recursive Spiking Decon          |
c |------------------------------------------------------------|
c |  NOTICE:                                                   |
c |  Copyright (C) 2017 Paul Michaels                          |
c |  <pm@cgiss.boisestate.edu>                                 |
c |  This program is free software; you can                    |
c |  redistribute it and/or modify it under the terms          |
c |  of the GNU General Public License as published            |
c |  by the Free Software Foundation; either version           |
c |  3 of the License, or (at your option) any later           |
c |  version.  This program is distributed in the              |
c |  hope that it will be useful, but                          |
c |  WITHOUT ANY WARRANTY; without even the implied            |
c |  warranty of MERCHANTABILITY or FITNESS FOR A              |
c |  PARTICULAR PURPOSE.  See the GNU General Public           |
c |  License for more details.                                 |
c |  You should have received a copy of the GNU                |
c |  General Public License along with this program;           |
c |  if not, write to the Free Software Foundation,            |
c |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
c |------------------------------------------------------------|
c |  PARAMETERS:                                               |
c |  ndim = maximum number of samples in a seismic trace       |
c |------------------------------------------------------------|
c |  INCLUDE FILE REQUIRED:                                    |
c |  ../include/bsegy.inc    (modified segy header definitions)|
c |                (no reel header, floating point data)       |
c |------------------------------------------------------------|
c |  SUBROUTINES CALLED:                                       |
c | In-line===============                                     |
c |     GETPRM  get input parameters from command line or      |
c |             interactive query.  Should be inline with      |
c |             main program, specific to application.         |
c |     LSTPRM  echo check of input parameters from GETPRM.    |
c |             Values written to a history file (*.lst)       |
c | Located in bsu-2.0.2/src/sublibF4===========               |
c |     CHKTRC  get number of samples, sample interval, and    |
c |             number of traces in the input data file.       |
c |             Also returns values for progress bar, stdout   |
c |     BSEGIN  Reads a seismic trace.                         |
c |     BSEGOUT Write a seismic trace.                         |
c |     PLTBAR  Writes progress bar on stdout                  |
c |     XCOR1   One sided auto-correlation(mine, gate built in)|
c |     CROSS   One sided auto-correlation (E.A.Robinson)      |
c |     EUREKA  Levinson recursion solution to LSQ filter(Rob.)|
c |     CONV2   Convolves a trace with a filter (mine,zero pt) |
c |     FOLD    Convolves a trace with a filter (E.A.Robinson) |
c |                                                            |
c | Programer's Note:                                          |
c | This program illustrates equivalent ways of doing correl.  |
c | and convolution with different subroutines in sublib4.     |
c | My subroutines are commented out here, Robinson's are used.|
c --------------------------------------------------------------
c
      include '../include/bsegy.inc'
      parameter (ndim=32767)
c
      real*4            s1(ndim),s2(ndim)
      real*4            p(ndim),f(ndim),a(ndim)
      character*4       aplc
      character*40      infil, outfil, outlst, outfl3
      character*70 title
      common /header/ hd
c
c...assign I/O units, initialize....
      jrec=1
      iunit1=1
      iunit2=2
      iunit3=3
      icall1=0
      icall2=0
      icall3=0
c
c...define application prefix
      write(aplc,'(''bdcn'')')
c
c...get parameters-------------------------------------------
      call GETPRM(nargsx,infil,s1,ndim,
     +tmingt,tmaxgt,mpts,stabf,iprof,imode)
c
c...count traces and check dimensions------------------------
      call CHKTRC(iunit1,icall1,npts,s1,fsamin,infil,
     +bar,invbar,ibar,ntrace,ndim)
c
c...name outputfile
      write(outfil,'(a,a,''.seg'')') aplc(1:4),infil(1:4)
      write(outfl3,'(a,''fltr.seg'')') aplc(1:4)
c
c...output listing-------------------------------------------
      call LSTPRM(io11,ntrace,aplc,outlst,outfil,
     +infil,outfl3,tmingt,tmaxgt,mpts,stabf,iprof,imode)
c
c...clear output files...
      write(outfil,'(a,a,''.seg'')') aplc(1:4),infil(1:4)
      open(12,file=outfil,access='direct',recl=240)
      close(12,status='delete')
c
      open(12,file=outfil,access='direct',recl=240)
      close(12,status='delete')
c
c...data loop=======================================....
c
c.../////////////////////////TRACE MODE/////////////////
      if(iprof.eq.0) then
c
c...Compute start and end samples for autocorrelation gate
      jmingt=tmingt/fsamin+1
      jmaxgt=tmaxgt/fsamin+1
      jpts=jmaxgt-jmingt+1
c
      do 100 jrec=1,ntrace
c
c...input a trace
      CALL BSEGIN(iunit1,ndim,
     +icall1,npts,s1,fsamin,infil,jrec,iexit)
c
      ntr = ntr + 1
c
c...take autocorrelation
c...using my subroutine (time gate determined inside)
c     call xcor1(npts,s1,p,fsamin,mpts,tmingt,tmaxgt)
c...using Robinson's subroutine
      call cross(jpts,s1(jmingt),jpts,s1(jmingt),mpts+1,p)  
c
c...add stability factor to zero lag term
      p(1)=p(1)+stabf*p(1)
c
c...compute prediction filter, "f", and prediction error, "a".
c (prediction error, "a", is decon operator since cross cor.
c of desired (impulse) with input is autocorrelation
c starting on second lag, "p(2)". 
c In essence rhs is for unit prediction distance.)
      call eureka(mpts,p(1),p(2),f,a)
c
c...apply decon operator
c...using my subroutine (permits designation of time zero)
c     npz=1
c     call conv2(npts,s1,mpts,a,s2,npz)
c...using Robinson's subroutine
      if (imode.eq.0) then
      call fold((npts-mpts+1),s1,mpts,a,lc,s2)
c
c...output decon operators, "a"
      do 50 jt=1,mpts
  50  s1(jt)=a(jt)
      else
      call fold((npts-mpts+1),s1,mpts,f,lc,s2)
c
c...output prediction operators, "f"
      do 51 jt=1,mpts
  51  s1(jt)=f(jt)
      endif
c
      CALL BSEGOUT(iunit3,
     +icall3,mpts,s1,fsamin,outfl3,jrec)
c
c...output trace
      CALL BSEGOUT(iunit2,
     +icall2,npts,s2,fsamin,outfil,jrec)
c
c...display progress....
      call pltbar(bar,invbar,ibar,jrec,ntrace)
c
c
c...END LOOP========================================....
  100 continue
      else
c...///////////////////////PROFILE MODE/////////////////////
c
      do 34 jt=1,mpts
   34 s2(jt)=0.
c------------------------------profile autocorrelation
c
c...Compute start and end samples for autocorrelation gate
      jmingt=tmingt/fsamin+1
      jmaxgt=tmaxgt/fsamin+1
      jpts=jmaxgt-jmingt+1
c
      do 101 jrec=1,ntrace
c
c...input a trace
      CALL BSEGIN(iunit1,ndim,
     +icall1,npts,s1,fsamin,infil,jrec,iexit)
c
      ntr = ntr + 1
c
c...take autocorrelation
c...using my subroutine (time gate determined inside)
c     call xcor1(npts,s1,p,fsamin,mpts+1,tmingt,tmaxgt)
c...using Robinson's subroutine
      call cross(jpts,s1(jmingt),jpts,s1(jmingt),mpts+1,p)  
c
c...sum autocorrelations
      do 35 jt=1,mpts+1
   35 s2(jt)=p(jt)+s2(jt)
  101 continue
c------------------------------compute mean autocorrelation
      do 36 jt=1,mpts+1
      p(jt)=s2(jt)/float(ntrace)
   36 s2(jt)=0.
c
c...add stability factor to zero lag term
      p(1)=p(1)+stabf*p(1)
c
c...compute decon operator
      call eureka(mpts,p(1),p(2),f,a)
c
c...list profile operator
      if (imode.eq.0) then
      write(title,'(''Profile Decon Operator: stabf= '',f10.7)') stabf
      call tlist(mpts,a,title)
      else
      write(title,'(''Prediction Operator:    stabf= '',f10.7)') stabf
      call tlist(mpts,f,title)
      endif
c
c-------------------------------apply decon loop
      do 102 jrec=1,ntrace
c
c...input a trace
      CALL BSEGIN(iunit1,ndim,
     +icall1,npts,s1,fsamin,infil,jrec,iexit)
c
      ntr = ntr + 1
c...apply decon operator
c...using my subroutine (permits designation of time zero)
c     npz=1
c     call conv2(npts,s1,mpts,a,s2,npz)
c...using Robinson's subroutine
      if (imode.eq.0) then
      call fold((npts-mpts+1),s1,mpts,a,lc,s2)
c
c...output decon operators
      do 52 jt=1,mpts
  52  s1(jt)=a(jt)
      else
      call fold((npts-mpts+1),s1,mpts,f,lc,s2)
c
c...output prediction operators
      do 53 jt=1,mpts
  53  s1(jt)=f(jt)
      endif
c
      CALL BSEGOUT(iunit3,
     +icall3,mpts,s1,fsamin,outfl3,jrec)
c
c...output trace
      CALL BSEGOUT(iunit2,
     +icall2,npts,s2,fsamin,outfil,jrec)
c
c...display progress....
      call pltbar(bar,invbar,ibar,jrec,ntrace)
c
c
c...END LOOP========================================....
  102 continue
      endif
c/////////////////////end of PROFILE/TRACE MODE BLOCKS///////
c
      write(*,'('' '')')
      write(6,240) ntrace,outfil,outlst,outfl3
240   format(/' Traces Processed = ',i5/
     +5x,'====> output in ',a/'====> listing in ',a/
     +5x,'===> decon operators in ',a)
c
c
      close(io11)
      close(iunit1)
      close(iunit2)
      close(iunit3)
      stop
      end
c
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: getprm.f                                     |
c |------------------------------------------------------------|
c | get input parameters or help                               |
c --------------------------------------------------------------
c
c
      subroutine GETPRM(nargsx,infil,s1,ndim,
     +tmingt,tmaxgt,mpts,stabf,iprof,imode)
c
      character*40      infil
      character*40      arg1,arg2,arg3,arg4,arg5,arg6,arg7
      real*4            s1(*)
c
c...get input/output file names...
c   |--------------------------------------------|
c   | file naming convention                     |
c   | application prefix 4 characters "bxxx"     |
c   | data set suffix 4 characters "dddd"        |
c   | file name = bxxxdddd.seg                   |
c   |--------------------------------------------|
      write(*,*) '|---------------------------------|'
      write(*,*) '| Copyright (C) 2017  P. Michaels |'
      write(*,*) '|       All rights reserved       |'
      write(*,*) '|see GNU General Public License   |'
      write(*,*) '|---------------------------------|'
c
ccc---LINUX VERSION
      nargsx=iargc()
      karg=-1
c...test for help
      if(nargsx.ge.(2+karg)) then
      call getarg(1,arg1)
      if(arg1(1:2).eq.'-h') then
      write(*,*) ' '
      write(*,*) '|------------------------------------------|'
      write(*,*) '| Basic Seismic Utilities     FORTRAN      |'
      write(*,*) '| ONLINE HELP:                             |'
      write(*,*) '|------------------------------------------|'
      write(*,*) '| bdcn: Deconvolution of seismic traces    |'
      write(*,*) '|       (least squares spiking, or predict)|'
      write(*,*) '|------------------------------------------|'
      write(*,*) ' '
      write(*,*)
     +'bdcn infile tmin tmax mpts stabf iprof imode '
      write(*,*) ' '
      write(*,*) 'infile  =input file name '
      write(*,*) 'tmin    =Autocorrelation Gate: START  '
      write(*,*) 'tmax    =Autocorrelation Gate: END '
      write(*,*) 'mpts    =Length of Decon Operator '
      write(*,*) 'stabf   =Stability Factor (ex: 0.01) '
      write(*,*) 'iprof   1=profile mode  0=trace mode '
      write(*,*) 'imode   1=Prediction 0=Prediction error'
      write(*,*) '        Choose 0 for spiking decon'        
      write(*,*) ' '
      stop
      endif
      endif
      if(nargsx.lt.(2+karg)) then
      write(*,*)'  enter: name of input file'
      read(*,'(a13)') infil
      else
      call getarg(1,infil)
      endif
      call namechk(12,infil)
c
c...input a trace
      iunit1=1
      iopen=0
      icall1=0
      jrec=1
      CALL BSEGIN(iunit1,ndim,
     +icall1,npts,s1,fsamin,infil,jrec,iexit)
      close(iunit1)
      tmx=float(npts-1)*fsamin
      write(*,'(''  Maximum Time= '',f10.4)') tmx
c
      if(nargsx.lt.(3+karg)) then
      write(*,*)' Design Gate enter: tmin'
      read(*,*) tmingt
      else
      call getarg(2,arg2)
      read(arg2,'(f3.0)') tmingt
      endif
      if(tmingt.lt.0.) tmingt=0.
c
      if(nargsx.lt.(4+karg)) then
      write(*,*)' Design Gate enter: tmax'
      read(*,*) tmaxgt
      else
      call getarg(3,arg3)
      read(arg3,'(f3.0)') tmaxgt
      endif
      if(tmaxgt.gt.tmx) tmaxgt=tmx
c
      if(nargsx.lt.(5+karg)) then
      write(*,*)' Operator Length enter: mpts'
      read(*,*) mpts
      else
      call getarg(4,arg4)
      read(arg4,'(i5)') mpts
      endif
c
      if(nargsx.lt.(6+karg)) then
      write(*,*)' Stability Factor enter: stabf'
      read(*,*) stabf
      else
      call getarg(5,arg5)
      read(arg5,'(f7.3)') stabf
      endif
c
      if(nargsx.lt.(7+karg)) then
      write(*,*)' 1=profile mode  0=trace mode enter: iprof'
      read(*,*) iprof
      else
      call getarg(6,arg6)
      read(arg6,'(i5)') iprof
      endif
c
c
      if(nargsx.lt.(8+karg)) then
      write(*,*) 'Enter imode:  Choose 0 for Spiking Decon'
      write(*,*) ' 1=Prediction mode 0=Prediction Error mode'
      read(*,*) imode
      else
      call getarg(7,arg7)
      read(arg7,'(i5)') imode
      endif
c
      return
      end
c
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: lstprm.f                                     |
c |------------------------------------------------------------|
c | echo check of input parameters, writes to a history file   |
c --------------------------------------------------------------
c
      subroutine LSTPRM(io11,ntrace,aplc,outlst,outfil,
     +infil,outfl3,tmingt,tmaxgt,mpts,stabf,iprof,imode)
      character*4       aplc
      character*40      outlst,outfil,infil,outfl3
c
c...open output listing file...
      io11=10
      write(outlst,'(a,a,''.lst'')') aplc,infil(1:4)
      open(io11,file=outlst,access='sequential')
      write(io11,*)'|---------------------------------|'
      write(io11,*)'| Copyright (C) 2017  P. Michaels |'
      write(io11,*)'|       All rights reserved       |'
      write(io11,*)'|see GNU General Public License   |'
      write(io11,*)'|---------------------------------|'
c
      write(io11,'('' Program bdcn  '')')
      write(io11,'('' Input File: '',a)') infil
      write(io11,'('' Output File: '',a)') outfil
      write(io11,'('' Decon Operator File: '',a)') outfl3
      write(io11,'(''  Number of traces='',i5)') ntrace
      write(io11,'(''  Parameters:  '')')
      write(io11,'(''   tmin= '',f10.2)') tmingt
      write(io11,'(''   tmax= '',f10.2)') tmaxgt
      write(io11,'(''   mpts= '',i5)') mpts
      write(io11,'(''   stabf= '',f10.2)') stabf
      write(io11,'(''  imode= '',i5)') imode
      if(imode.eq.1) then
      write(io11,*) '     PREDICTIION MODE'
      else
      write(io11,*) '     SPIKING DECON MODE'
      endif
      if(iprof.eq.1) then
      write(io11,*) '     PROFILE MODE'
      else
      write(io11,*) '     TRACE MODE'
      endif
      write(io11,'(50(1h-))')
      write(io11,'(''    '')')
c
      return
      end
