c |$Id: bfil.f,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $          |
c --------------------------------------------------------------
c | Basic Seismic Utilities (BSU)                              |
c | LANGUAGE: Fortran77                                        |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: bfil.f                                       |
c | DIRECTORY: bsu-2.0.2/src/Fort/bsegy                        |
c |------------------------------------------------------------|
c | BSU Filter program.  Uses bilinear transform to realize an |
c | ARMA filter from the Laplace Transform                     |
c |------------------------------------------------------------|
c |  NOTICE:                                                   |
c |  Copyright (C) 2017 Paul Michaels                          |
c |  <pm@cgiss.boisestate.edu>                                 |
c |  This program is free software; you can                    |
c |  redistribute it and/or modify it under the terms          |
c |  of the GNU General Public License as published            |
c |  by the Free Software Foundation; either version           |
c |  3 of the License, or (at your option) any later           |
c |  version.  This program is distributed in the              |
c |  hope that it will be useful, but                          |
c |  WITHOUT ANY WARRANTY; without even the implied            |
c |  warranty of MERCHANTABILITY or FITNESS FOR A              |
c |  PARTICULAR PURPOSE.  See the GNU General Public           |
c |  License for more details.                                 |
c |  You should have received a copy of the GNU                |
c |  General Public License along with this program;           |
c |  if not, write to the Free Software Foundation,            |
c |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
c |------------------------------------------------------------|
c |  PARAMETERS:                                               |
c |  ndim = maximum number of samples in a seismic trace       |
c |------------------------------------------------------------|
c |  INCLUDE FILE REQUIRED:                                    |
c |  ../include/bsegy.inc    (modified segy header definitions)|
c |                (no reel header, floating point data)       |
c |------------------------------------------------------------|
c |  SUBROUTINES CALLED:                                       |
c | In-line===============                                     |
c |     GETPRM  get input parameters from command line or      |
c |             interactive query.  Should be inline with      |
c |             main program, specific to application.         |
c |     LSTPRM  echo check of input parameters from GETPRM.    |
c |             Values written to a history file (*.lst)       |
c |     DISPLY  Displays frequency response and impulse resp.  |
c |             on output listing file                         |
c |     LPASS   Computes low-pass filter coef.                 |
c |     BPASS   Computes band-pass filter coef.                |
c |     HPASS   Computes high-pass filter coef.                |
c |     FILTER  Applies filter                                 |
c |  Note: The ?PASS and FILTER subroutines are equivalent     |
c |        to the SUBLIB4 subroutine ARMA, but are included    |
c |        inline here because they have some I/O to the list  |
c |        file.  This is intended to be a more informative    |
c |        version than the workhorse versions.                |
c |                                                            |
c | Located in bsu-2.0.2/src/sublibF4===========               |
c |     CHKTRC  get number of samples, sample interval, and    |
c |             number of traces in the input data file.       |
c |             Also returns values for progress bar, stdout   |
c |     BSEGIN  Reads a seismic trace.                         |
c |     BSEGOUT Write a seismic trace.                         |
c |     PLTBAR  Writes progress bar on stdout                  |
c |                                                            |
c --------------------------------------------------------------
c
c
      PROGRAM MAIN
      include '../include/bsegy.inc'
      parameter (ndim=32767,mdim=512)
      real*4 xi(mdim),xr(mdim),amp(mdim),phz(mdim)
      complex x(mdim),zz
      real*4            s1(ndim),s2(ndim)
      character*4       aplc
      character*40      infil, outfil, outlst
      common /header/ hd
c
c...assign I/O units, initialize....
      jrec=1
      iunit1=1
      iunit2=2
      icall1=0
      icall2=0
c
c...define application prefix
      write(aplc,'(''bfil'')')
c
c...get parameters-------------------------------------------
      call GETPRM(nargsx,infil,
     +itype,npoles,fcenter,bwidth,ifaz)       
c
c...count traces and check dimensions------------------------
      call CHKTRC(iunit1,icall1,npts,s1,fsamin,infil,
     +bar,invbar,ibar,ntrace,ndim)
c
c...name outputfile
      write(outfil,'(a,a,''.seg'')') aplc(1:4),infil(1:4)
c
c...Derived Filter Parameters
      fnyq=1./(2.*fsamin)
      if(itype.ne.1) then
      bwidth=1.*fcenter
      endif
c   Must be even number of poles
      npow=npoles/2
      npoles=2*npow
c
c...output listing-------------------------------------------
      call LSTPRM(io11,ntrace,aplc,outlst,outfil,
     +infil,itype,npoles,fcenter,bwidth,ifaz,fsamin,fnyq)       
c
c...calculate approximate filter duration
      tfmx=4./bwidth
      ntfmx=tfmx/fsamin
c------------------------------Branch to Filter Type             
c
      if(ifaz.ne.1) then
      npow=npow*2
      endif
c
      if(itype.eq.1) then
      call BPASS(fcenter,bwidth,npow,fsamin,C0,C1,C2,D0,D1,D2)
      endif
      if(itype.eq.0) then
      call LPASS(fcenter,npow,fsamin,C0,C1,C2,D0,D1,D2)
      endif
      if(itype.eq.2) then
      call HPASS(fcenter,npow,fsamin,C0,C1,C2,D0,D1,D2)
      endif
c
c-----------------------------------------------------             
c
      if(ifaz.ne.1) then
      npow=npow/2
      endif
c
c...Display Filter On Output Listing File
      call DISPLY(C0,C1,C2,D0,D1,D2,itype,x,zz,mdim,s2,s1,
     +bwidth,npow,fcenter,npoles,xi,xr,amp,phz,fsamin,ifaz)
c
c...clear output file...
      open(10,file=outfil,access='direct',recl=240)
      close(10,status='delete')
c
      icall1=0
      icall2=0
      ntr=0
c
c...data loop=======================================....

c
      do 100 jrec=1,ntrace
c
c
c...input a trace
      call BSEGIN(iunit1,ndim,
     +icall1,npts,s1,fsamin,infil,jrec,iexit)
c
      ntr = ntr + 1
c
c...apply filter....TIME DOMAIN ARMA...........
      call FILTER(npts,npow,ifaz,s1,s2,C0,C1,C2,D0,D1,D2)
c
c...output trace
      iunit2=2
      call BSEGOUT(iunit2,
     +icall2,npts,s2,fsamin,outfil,jrec)
c
c...display progress....
      call pltbar(bar,invbar,ibar,jrec,ntrace)
c
c
c...END LOOP========================================....
  100 continue
c
c
      write(*,'('' '')')
      write(6,240) ntrace,outfil,outlst
240   format(' Traces Processed = ',i5/
     +5x,'====> output in ',a/'====> listing in ',a)
c
      close(io11)
      close(iunit1)
      close(iunit2)
      stop
      end
c
c --------------------------------------------------------------
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c |                                                            |
c | BPASS      B-SEGY format (no reel header, floating point)  |
c |------------------------------------------------------------|
c | Computes bandpass filter Z-transform                       |
c | Method: Bilinear Transform of Laplace Transform            |
c |                                                            |
c |                                                            |
c |                                                            |
c --------------------------------------------------------------
c

      subroutine BPASS(fcenter,bwidth,npow,fsamin,C0,C1,C2,D0,D1,D2)
c
      io11=10
c
c...convert to radians/sec
      pi=3.1415926
      wcd=2.*pi*fcenter
      bwd=bwidth*2.*pi
c ratio 2/dt
      r2dt=2.0/fsamin
c
c...map to Laplace S-Plane
      wc=tan((-wcd*fsamin)/2.0)*r2dt
      bw2=tan(((-wcd*fsamin)-(bwd*fsamin)/2.0)/2.0)*r2dt
      bw1=tan(((-wcd*fsamin)+(bwd*fsamin)/2.0)/2.0)*r2dt
      bw=bw2-bw1
      bw=bw/sqrt(2.0**(1./float(npow))-1)
c
      write(io11,'(5x,''S_Plane: wo='',f10.4,'' B='',f10.4)') wc,bw
      write(io11,'(//)')
c
c...Bilinear Transform s=(z-1)/(z+1)*(2/dt)
c
c...numerator (zeros)
      C0=-bw*r2dt
      C1=0.0
      C2=bw*r2dt
c
c...denominator (poles)
      D0=r2dt*r2dt - bw*r2dt + wc*wc
      D1=2.*(wc*wc - r2dt*r2dt)
      D2=r2dt*r2dt + bw*r2dt + wc*wc
c
      write(io11,'(5x,''Z-Plane: '')')
      write(io11,'(5x,''Numerator:C0,C1,C2='',3(2x,e14.7))') C0,C1,C2
      write(io11,'(//)')
      write(io11,'(5x,''Denominator:D0,D1,D2='',3(2x,e14.7))') D0,D1,D2
      write(io11,'(//)')
      write(io11,*)'  ARMA OPERATOR:'
      write(io11,*)'    y=output  x=input '
      write(io11,800)
  800 format(/5x,'y(j)=[C2*x(j-2)+C1*x(j-1)+C0*x(j)-D2*y(j-2)',
     +'-D1*y(j-1)]/D0')
      write(io11,'(//)')
      return
      end
c
c --------------------------------------------------------------
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c |                                                            |
c | DISPLY     B-SEGY format (no reel header, floating point)  |
c |------------------------------------------------------------|
c | Displays Spectral Response of Filter                       |
c | and Time Domain Impulse Response                           |
c |                                                            |
c |                                                            |
c |                                                            |
c --------------------------------------------------------------
c
      subroutine DISPLY(C0,C1,C2,D0,D1,D2,itype,x,zz,mdim,s2,s1,
     +bwidth,npow,fcenter,npoles,xi,xr,amp,phz,fsamin,ifaz)
      character*70      title
c
      real*4 xi(*),xr(*),amp(*),phz(*),s2(*),s1(*)
      complex x(*),zz
c
      io11=10
c
c...calculate approximate filter duration
      tfmx=4./bwidth
c
c...Note: To save space, compute an appropriate sample interval, dt,
c         consistent with the length, ntfmx.  This will put the poles
c         at the correct angle when computing the Z-transform for the
c         listing ONLY.(see do loop 85)
      ntfmx=512
      dt=tfmx/float(ntfmx)
      npr2mx=ntfmx
      write(io11,'(/''    Filter Duration Approx='',f10.4/)') tfmx
c
      pi=3.1415926
      fnyq=1./(2.*fsamin)
      if(itype.eq.1) then
      if((fcenter+bwidth/2.).gt.fnyq) then
      write(*,*) 'ABORT!! Sample interval too large for this filter'
      stop
      endif
      if((fcenter-bwidth/2.).lt.0.) then
      write(*,*) 'ABORT!! Bandwidth too large for center frequency'
      stop
      endif
      endif
c
c...compute short response for display
      if(npr2mx.le.mdim) then
      do 85 j=1,npr2mx
      w=-2.*pi*(float(j-1)/(float(npr2mx)*dt))
      if(j.eq.1) then
      zz=cmplx(1.0,0.0)
      else
      zz=cmplx(cos(w*dt),sin(w*dt))
      endif
      x(j)=(C2*zz*zz+C1*zz+C0)/(D2*zz*zz+D1*zz+D0)
c
c...Linux can't raise zero to a power
      if(x(j).ne.cmplx(0.,0.)) then
      x(j)=x(j)**float(npow)
      endif
c
      if(ifaz.eq.0) then
      x(j)=x(j)*conjg(x(j))
      endif
   85 continue
      tzero=0.
      ilin=0
      write(title,900) fcenter,bwidth,npoles
  900 format('Filter Response: fc=',f7.2,'  BW=',f7.2,' Npoles=',i5)
      call flist(npr2mx,x,title,fsamin,tzero,xr,xi,amp,phz,ilin)
c
c...ARMA Computation of Impulse Response
c===============================================
c
c...set up impuse in s1
      do 290 jt=1,ntfmx
  290 s1(jt)=0.
      ntfm2=ntfmx/2
      if(ifaz.eq.1) then
      s1(1)=1.0
      else
      s1(ntfm2)=1.0
      endif
c
c...apply filter....TIME DOMAIN ARMA...........
c
      if(ifaz.eq.1) then
      do 301 jpow=1,npow
      s2(1)=(s1(1)*C0)/D0
      s2(2)=(s1(2)*C0+s1(1)*C1-s2(1)*D1)/D0
c
c
      do 351 jt=3,ntfmx
      tt1=(s1(jt-2)*C2)/D0
      tt2=(s1(jt-1)*C1)/D0
      tt3=(s1(jt)*C0)/D0
c
      tt4=(s2(jt-2)*D2)/D0
      tt5=(s2(jt-1)*D1)/D0
      s2(jt)=(tt1+tt2+tt3-tt4-tt5)
  351 continue
c
      do 361 jt=1,ntfmx
  361 s1(jt)=s2(jt)
  301 continue
      else
c
c...zero phase (forward/reverse trick)
c
      do 401 jpow=1,npow
      s2(1)=(s1(1)*C0)/D0
      s2(2)=(s1(2)*C0+s1(1)*C1-s2(1)*D1)/D0
c
c
      do 451 jt=3,ntfmx
      tt1=(s1(jt-2)*C2)/D0
      tt2=(s1(jt-1)*C1)/D0
      tt3=(s1(jt)*C0)/D0
c
      tt4=(s2(jt-2)*D2)/D0
      tt5=(s2(jt-1)*D1)/D0
      s2(jt)=(tt1+tt2+tt3-tt4-tt5)
  451 continue
c
      do 461 jt=1,ntfmx
  461 s1(jt)=s2(jt)
  401 continue
c
      do 402 jpow=1,npow
      s2(1)=(s1(ntfmx)*C0)/D0
      s2(2)=(s1(ntfmx-1)*C0+s1(ntfmx)*C1-s2(ntfmx)*D1)/D0
c
c
      npts3=ntfmx-2
      do 452 jt=npts3,1,-1
c
      tt1=(s1(jt+2)*C2)/D0
      tt2=(s1(jt+1)*C1)/D0
      tt3=(s1(jt)*C0)/D0
c
      tt4=(s2(jt+2)*D2)/D0
      tt5=(s2(jt+1)*D1)/D0
      s2(jt)=(tt1+tt2+tt3-tt4-tt5)
  452 continue
c
      do 462 jt=1,ntfmx
  462 s1(jt)=s2(jt)
  402 continue
      endif
c
      write(title,901) fcenter,bwidth,npoles
  901 format('ARMA Impulse Response: fc=',f7.2,'  BW=',f7.2,
     +' Npoles=',i5)
c
      write(io11,'(a)') title
      call tlist(ntfmx,s2,title)
c===============================================
          endif
c
      return
      end
c
c --------------------------------------------------------------
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c |                                                            |
c | LPASS      B-SEGY format (no reel header, floating point)  |
c |------------------------------------------------------------|
c | Computes lowpass  filter Z-transform                       |
c | Method: Bilinear Transform of Laplace Transform            |
c |                                                            |
c |                                                            |
c |                                                            |
c --------------------------------------------------------------
c

      subroutine LPASS(fcenter,npow,fsamin,C0,C1,C2,D0,D1,D2)
c
      io11=10
c
c...convert to radians/sec
      pi=3.1415926
      wcd=2.*pi*fcenter
c ratio 2/dt
      r2dt=2.0/fsamin
c
c...map to Laplace S-Plane
      wc=tan((-wcd*fsamin)/2.0)*r2dt
      wo=(1./(2.0**(1./float(npow))-1.0)**(.25))*wc
      bw=wo*sqrt(2.0)
c
      write(io11,'(5x,''S_Plane: wo='',f10.4,'' B='',f10.4)') wc,bw
      write(io11,'(//)')
c
c...Bilinear Transform s=(z-1)/(z+1)*(2/dt)
c
c...numerator (zeros)
      C0=wo*wo
      C1=wo*wo*2.0
      C2=wo*wo
c
c...denominator (poles)
      D0=r2dt*r2dt - bw*r2dt + wo*wo
      D1=2.*(wo*wo - r2dt*r2dt)
      D2=r2dt*r2dt + bw*r2dt + wo*wo
c
      write(io11,'(5x,''Z-Plane: '')')
      write(io11,'(5x,''Numerator:C0,C1,C2='',3(2x,e14.7))') C0,C1,C2
      write(io11,'(5x,''Denominator:D0,D1,D2='',3(2x,e14.7))') D0,D1,D2
      write(io11,'(//)')
      write(io11,*)'  ARMA OPERATOR:'
      write(io11,*)'    y=output  x=input '
      write(io11,800)
  800 format(/5x,'y(j)=[C2*x(j-2)+C1*x(j-1)+C0*x(j)-D2*y(j-2)',
     +'-D1*y(j-1)]/D0')
      write(io11,'(//)')
      return
      end
c
c --------------------------------------------------------------
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c |                                                            |
c | HPASS      B-SEGY format (no reel header, floating point)  |
c |------------------------------------------------------------|
c | Computes highpass filter Z-transform                       |
c | Method: Bilinear Transform of Laplace Transform            |
c |                                                            |
c |                                                            |
c |                                                            |
c --------------------------------------------------------------
c

      subroutine HPASS(fcenter,npow,fsamin,C0,C1,C2,D0,D1,D2)
c
      io11=10
c
c...convert to radians/sec
      pi=3.1415926
      wcd=2.*pi*fcenter
c  ratio 2/dt
      r2dt=2.0/fsamin
c
c...map to Laplace S-Plane
      wc=tan((-wcd*fsamin)/2.0)*r2dt
      wo=((2.0**(1./float(npow))-1.0)**(.25))*wc
      bw=wo*sqrt(2.0)
c
      write(io11,'(5x,''S_Plane: wo='',f10.4,'' B='',f10.4)') wc,bw
      write(io11,'(//)')
c
c...Bilinear Transform s=(z-1)/(z+1)*(2/dt)
c
c...numerator (zeros)
      C0=r2dt*r2dt
      C1=-2.0*r2dt*r2dt
      C2=r2dt*r2dt
c
c...denominator (poles)
      D0=r2dt*r2dt - bw*r2dt + wo*wo
      D1=2.*(wo*wo - r2dt*r2dt)
      D2=r2dt*r2dt + bw*r2dt + wo*wo
c
      write(io11,'(5x,''Z-Plane: '')')
      write(io11,'(5x,''Numerator:C0,C1,C2='',3(2x,e14.7))') C0,C1,C2
      write(io11,'(5x,''Denominator:D0,D1,D2='',3(2x,e14.7))') D0,D1,D2
      write(io11,'(//)')
      write(io11,*)'  ARMA OPERATOR:'
      write(io11,*)'    y=output  x=input '
      write(io11,800)
  800 format(/5x,'y(j)=[C2*x(j-2)+C1*x(j-1)+C0*x(j)-D2*y(j-2)',
     +'-D1*y(j-1)]/D0')
      write(io11,'(//)')
      return
      end
c
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: getprm.f                                     |
c |------------------------------------------------------------|
c | get input parameters or help                               |
c --------------------------------------------------------------
c
      subroutine GETPRM(nargsx,infil,
     +itype,npoles,fcenter,bwidth,ifaz)       
c
      character*40      infil
      character*40      arg1,arg2,arg3,arg4,arg5,arg6
c
c
c...get input/output file names...
c   |--------------------------------------------|
c   | file naming convention                     |
c   | application prefix 4 characters "bxxx"     |
c   | data set suffix 4 characters "dddd"        |
c   | file name = bxxxdddd.seg                   |
c   |--------------------------------------------|
      write(*,*)'|---------------------------------|'
      write(*,*)'| Copyright (C) 2017  P. Michaels |'
      write(*,*)'|       All rights reserved       |'
      write(*,*)'|see GNU General Public License   |'
      write(*,*)'|---------------------------------|'
c
ccc---LINUX VERSION
      nargsx=iargc()
      karg=-1
c...test for help
      if(nargsx.ge.(2+karg)) then
      call getarg(1,arg1)
      if(arg1(1:2).eq.'-h') then
      write(*,*)' '
      write(*,*) '|------------------------------------------|'
      write(*,*) '| Basic Seismic Utilities     FORTRAN      |'
      write(*,*) '| ONLINE HELP:                             |'
      write(*,*) '|------------------------------------------|'
      write(*,*) '| bfil: Filters seismic traces using       |'
      write(*,*) '| bilinear transform (low, band, or high)  |'
      write(*,*) '|------------------------------------------|'
      write(*,*)' '
      write(*,*)
     +'bfil infile itype npoles fcenter bwidth ifaz '
      write(*,*)' '
      write(*,*)'infile  =input file name '
      write(*,*)'itype  0=low-pass filter, cut off freq= fc (-3dBv)'
      write(*,*)'       1=band-pass filter, center frequency= fc'
      write(*,*)'       2=high-pass filter, cut off freq= fc (-3dBv)'
      write(*,*)'npoles  =number of poles in filter'
      write(*,*)'         (6dB/octave)/(pair of poles)'
      write(*,*)'fcenter =center frequency Hz    '
      write(*,*)'bwidth  =band-pass filter bandwidth (-3dB) Hz '
      write(*,*)'ifaz    1=minimum phase  0=zero phase filter  '
      write(*,*)' '
      stop
      endif
      endif
      if(nargsx.lt.(2+karg)) then
      write(*,*)'  enter: name of input file'
      read(*,'(a13)') infil
      else
      call getarg(1,infil)
      endif
      call namechk(12,infil)
c
c...get parameters
      if(nargsx.lt.(3+karg)) then
      write(*,*)'  Type of Filter:'
      write(*,*)'    0=low-pass (0.707 dampening)'
      write(*,*)'    1=band-pass (bw=bandwidth)'
      write(*,*)'    2=high-pass (0.707 dampening)'
      read(*,*) itype
      else
      call getarg(2,arg2)
      read(arg2,'(i5)') itype
      endif
      write(*,'(I5,1x,I5 )') itype,nargsx
c
c...check for bandwidth entry error
      if ( (itype .ne. 1).and.(nargsx.ge.6)) then
      write(*,*) 'Number of arguments too many for high or low pass'
      write(*,*) 'Only bandpass filters permit entry of bandwidth  '
      write(*,*) '!!!!ABORT: Check command line arguments!!!!'
      stop
      endif
c
      if(nargsx.lt.(4+karg)) then
      write(*,*)'  Enter number of poles in filter'
      read(*,*) npoles
      else
      call getarg(3,arg3)
      read(arg3,'(i5)') npoles
      endif
      npoles=iabs(npoles)
      if (npoles .lt. 2) then
      npoles=2
      endif
c
      if(nargsx.lt.(5+karg)) then
      write(*,*)'  Enter Center or Cutoff Frequency'
      read(*,*) fcenter
      else
      call getarg(4,arg4)
      read(arg4,'(f6.0)') fcenter
      endif
      fcenter=abs(fcenter)
c
      if(itype.eq.1) then
      if(nargsx.lt.(6+karg)) then
      write(*,*)'  Enter Bandwidth Hz'
      write(*,*)'  ONLY FOR BANDPASS FILTER'
      read(*,*) bwidth
      else
      call getarg(5,arg5)
      read(arg5,'(f6.0)') bwidth
      bwidth=abs(bwidth)
      endif
      bwidth=abs(bwidth)
      endif
c
      ifaz=1
      if(itype.eq.1) then
      if(nargsx.lt.(7+karg)) then
      write(*,*)'  1=Minimum Phase Filter (ARMA)'
      write(*,*)'  0=Zero Phase Filter'
      write(*,*)'  Enter Option'
      read(*,*) ifaz
      else
      call getarg(6,arg6)
      read(arg6,'(i5)') ifaz
      endif
      else
      if(nargsx.lt.(6+karg)) then
      write(*,*)'  1=Minimum Phase Filter (ARMA)'
      write(*,*)'  0=Zero Phase Filter'
      write(*,*)'  Enter Option'
      read(*,*) ifaz
      else
      call getarg(5,arg5)
      read(arg5,'(i5)') ifaz
      endif
      endif
      return
      end
c
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: lstprm.f                                     |
c |------------------------------------------------------------|
c | echo check of input parameters, writes to a history file   |
c --------------------------------------------------------------
c
      subroutine LSTPRM(io11,ntrace,aplc,outlst,outfil,
     +infil,itype,npoles,fcenter,bwidth,ifaz,fsamin,fnyq)       
      character*4       aplc
      character*40      outlst,outfil,infil
c
c...open output listing file...
      io11=10
      write(outlst,'(a,a,''.lst'')') aplc,infil(1:4)
      open(io11,file=outlst,access='sequential')
      write(io11,*)'|---------------------------------|'
      write(io11,*)'| Copyright (C) 2017  P. Michaels |'
      write(io11,*)'|       All rights reserved       |'
      write(io11,*)'|see GNU General Public License   |'
      write(io11,*)'|---------------------------------|'
c
      write(io11,'('' Program bfil'')')
      write(io11,'('' Input File: '',a)') infil
      write(io11,'('' Output File: '',a)') outfil
      write(io11,'(''  Number of traces='',i5)') ntrace
      write(io11,'(''  Sample Interval='',f10.6,'' sec.'')') fsamin
      write(io11,'(''  Nyquist Frequency='',f10.2,''Hz'')') fnyq
      write(io11,'(//)')
      write(io11,'(''  Parameters: '')')
      write(io11,'(''  itype='',i5)') itype
      if(itype.eq.1) then
      write(io11,'(''     Center Frequency, fc='',f10.4)')  fcenter
      else
      if(itype.eq.0) then
      write(io11,'(''     High-Cut Frequency, fc='',f10.4)')  fcenter
      else
      write(io11,'(''     Low-Cut Frequency, fc='',f10.4)')  fcenter
      endif
      endif
      write(io11,'(''     Band Width,       BW='',f10.4)')  bwidth
c
      write(io11,'(''     Number of Poles   np='',i5)') npoles
      write(io11,'(''  ifaz ='',i5)') ifaz 
      if(ifaz.eq.1) then
      write(io11,*)'      (06dB/Octave) Slope for every pair of poles'
      else
      write(io11,*)'      (12dB/Octave) per pair of poles'
      endif
      if(ifaz.eq.0) then
      write(io11,*)'      Zero Phase Filter (ARMA)'
      else
      write(io11,*)'      Minimum Phase Filter (ARMA)'
      endif
      write(io11,'(50(1h-))')
      write(io11,'(''    '')')
c
      if(itype.eq.0) then
      write(io11,*)'  LOW-PASS FILTER DESIGN: Bilinear Transform'
      write(io11,*)'                          s=(z-1)/(z+1)*(2/dt)'
      write(io11,*)' '
      write(io11,*)' Laplace:  LP(s)=[wo^2/(s^2+B*s+wo^2)]^pow'
      write(io11,*)'    where pow=npoles/2 '
      write(io11,*)'           wo=wc/[2^(1/pow)-1]^.25 '
      write(io11,*)'            B=sqrt(2)*wo   '
      write(io11,*)'             wc=cut-off frequency (-3dBv) '
      write(io11,*)' '
      write(io11,*)' Laplace <=======> Z-Transform Critical Freq Map'
      write(io11,*)'      wc     =     tan((-wcd*delt)/2)*(2/dt)  '
      write(io11,*)'                 where wcd=desired cut-off freq. '
      write(io11,*)'                       delt=sample interval  '
      write(io11,*)' '
      endif
      if(itype.eq.1) then
      write(io11,*)'   BAND-PASS FILTER DESIGN: Bilinear Transform'
      write(io11,*)'                            s=(z-1)/(z+1)*(2/dt)'
      write(io11,*)' '
      write(io11,*)' Laplace:  BP(s)=[(B*s)/(s^2+B*s+wo^2)]^pow'
      write(io11,*)'    where pow=npoles/2 '
      write(io11,*)'           wo=center frequency '
      write(io11,*)'            B=B1/sqrt[2^(1/pow)-1]   '
      write(io11,*)'            B=Bandwidth for npoles, B1 for 2poles'
      write(io11,*)' '
      write(io11,*)' Laplace <=======> Z-Transform Critical Freq Map'
      write(io11,*)'      wo     =     tan(-(wod*delt)/2)*(2/dt)  '
      write(io11,*)'                 where wod=desired center freq. '
      write(io11,*)'                       delt=sample interval  '
      write(io11,*)'      bc1    =     tan((-wod*delt+BW/2)/2)*(2/dt)  '
      write(io11,*)'      bc2    =     tan((-wod*delt-BW/2)/2)*(2/dt)  '
      write(io11,*)' B1=(bc2-bc1)=   BW Bandwidth Desired  '
      write(io11,*)' '
      endif
      if(itype.eq.2) then
      write(io11,*)'   HIGH-PASS FILTER DESIGN: Bilinear Transform'
      write(io11,*)'                            s=(z-1)/(z+1)*(2/dt)'
      write(io11,*)' '
      write(io11,*)' Laplace: HP(s)=[s^2/(s^2+B*s+wo^2)]^pow'
      write(io11,*)'    where pow=npoles/2 '
      write(io11,*)'           wo=wc*[2^(1/pow)-1]^.25 '
      write(io11,*)'            B=sqrt(2)*wo   '
      write(io11,*)'             wc=cut-off frequency (-3dBv) '
      write(io11,*)' '
      write(io11,*)' Laplace <=======> Z-Transform Critical Freq Map'
      write(io11,*)'      wc     =     tan((-wcd*delt)/2)*(2/dt)  '
      write(io11,*)'                 where wcd=desired cut-off freq. '
      write(io11,*)'                       delt=sample interval  '
      write(io11,*)' '
      endif
      write(io11,*)' '
c
      write(io11,'(50(1h-))')
      write(io11,'(''    '')')
c
      return
      end
c
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: filter.f                                     |
c |------------------------------------------------------------|
c | apply ARMA filter                                          |
c --------------------------------------------------------------
c
      subroutine FILTER(npts,npow,ifaz,s1,s2,C0,C1,C2,D0,D1,D2)
      real*4  s1(*),s2(*)
c
      if(ifaz.eq.1) then
c
c...minimum phase
c
      do 301 jpow=1,npow
      s2(1)=(s1(1)*C0)/D0
      s2(2)=(s1(2)*C0+s1(1)*C1-s2(1)*D1)/D0
c
      do 351 jt=3,npts
      tt1=(s1(jt-2)*C2)/D0
      tt2=(s1(jt-1)*C1)/D0
      tt3=(s1(jt)*C0)/D0
c
      tt4=(s2(jt-2)*D2)/D0
      tt5=(s2(jt-1)*D1)/D0
      s2(jt)=(tt1+tt2+tt3-tt4-tt5)
  351 continue
c
      do 361 jt=1,npts
  361 s1(jt)=s2(jt)
  301 continue
      else
c
c...zero phase (forward/reverse trick)
c
      do 401 jpow=1,npow
      s2(1)=(s1(1)*C0)/D0
      s2(2)=(s1(2)*C0+s1(1)*C1-s2(1)*D1)/D0
c
      do 451 jt=3,npts
      tt1=(s1(jt-2)*C2)/D0
      tt2=(s1(jt-1)*C1)/D0
      tt3=(s1(jt)*C0)/D0
c
      tt4=(s2(jt-2)*D2)/D0
      tt5=(s2(jt-1)*D1)/D0
      s2(jt)=(tt1+tt2+tt3-tt4-tt5)
  451 continue
c
      do 461 jt=1,npts
  461 s1(jt)=s2(jt)
  401 continue
c
      do 402 jpow=1,npow
      s2(1)=(s1(npts)*C0)/D0
      s2(2)=(s1(npts-1)*C0+s1(npts)*C1-s2(npts)*D1)/D0
c
      npts3=npts-2
      do 452 jt=npts3,1,-1
c
      tt1=(s1(jt+2)*C2)/D0
      tt2=(s1(jt+1)*C1)/D0
      tt3=(s1(jt)*C0)/D0
c
      tt4=(s2(jt+2)*D2)/D0
      tt5=(s2(jt+1)*D1)/D0
      s2(jt)=(tt1+tt2+tt3-tt4-tt5)
  452 continue
c
      do 462 jt=1,npts
  462 s1(jt)=s2(jt)
  402 continue
      endif
      return
      end
