c |$Id: bref.f,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $          |
c --------------------------------------------------------------
c | Basic Seismic Utilities (BSU)                              |
c | LANGUAGE: Fortran77                                        |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: bref.f                                       |
c | DIRECTORY: bsu-2.0.2/src/Fort/bsegy                        |
c |                                                            |
c | REVISED: 1 Sept 2012   shot=recv location constraints      |
c |                        (save to scratch, write out last)   |
c |------------------------------------------------------------|
c | Refraction first break analysis setup program.   Creates   |
c | matrices and vectors for Scilab program, delaytm.sci,      |
c | which does the actual inversion.  This two step procedure  |
c | allows one to edit the matrices and vectors with a text    |
c | editor (perhaps to add some constraint equations) before   |
c | the Scilab procedure is run, and a solution obtained.      |
c |  NOTE: You need to put first arrival time picks in headers |
c |  (see bpic.f)                                              |
c |------------------------------------------------------------|
c | Reads SEGY trace headers to get geometry and pick times    |
c | of the first breaks. Handles multiple shot records         |
c | for a joint inversion of all records at once.              |
c |                                                            |
c | Outputs three files:1. System Matrix                       |
c |                     2. Data Vector                         |
c |                     3. Elevation File                      |
c |  NOTE: each shot record must have same number of traces    |
c |    Columns of system Matrix:[shots/ recvrs/ refr. slowness]|
c |    Rows are [travel time equations shot #1]                |
c |             [travel time equations shot #2]                |
c |                      .                                     |
c |                      .                                     |
c |                      .                                     |
c |             [travel time equations shot #N]                |
c |             [constraint equations when no pick available]  |
c |                                                            |
c |    Parameter vector will be [shot delay times]             |
c |                             [recvr. delay times]           |
c |                             [refractor slowness]           |
c |    Data column vector [observed pick times]                |
c |    Elevation file of 2 columns [channel/flag/elevation]    |
c |                  Here channel #=sequential trace #         |
c |------------------------------------------------------------|
c |  NOTICE:                                                   |
c |  Copyright (C) 2017 Paul Michaels                          |
c |  <pm@cgiss.boisestate.edu>                                 |
c |  This program is free software; you can                    |
c |  redistribute it and/or modify it under the terms          |
c |  of the GNU General Public License as published            |
c |  by the Free Software Foundation; either version           |
c |  3 of the License, or (at your option) any later           |
c |  version.  This program is distributed in the              |
c |  hope that it will be useful, but                          |
c |  WITHOUT ANY WARRANTY; without even the implied            |
c |  warranty of MERCHANTABILITY or FITNESS FOR A              |
c |  PARTICULAR PURPOSE.  See the GNU General Public           |
c |  License for more details.                                 |
c |  You should have received a copy of the GNU                |
c |  General Public License along with this program;           |
c |  if not, write to the Free Software Foundation,            |
c |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
c |------------------------------------------------------------|
c |  PARAMETERS:                                               |
c |  ndim = maximum number of samples in a seismic trace       |
c |------------------------------------------------------------|
c |  INCLUDE FILE REQUIRED:                                    |
c |  ../include/bsegy.inc    (modified segy header definitions)|
c |                (no reel header, floating point data)       |
c |------------------------------------------------------------|
c |  SUBROUTINES CALLED:                                       |
c | In-line===============                                     |
c |     GETPRM  get input parameters from command line or      |
c |             interactive query.  Should be inline with      |
c |             main program, specific to application.         |
c |     LSTPRM  echo check of input parameters from GETPRM.    |
c |             Values written to a history file (*.lst)       |
c | Located in bsu-2.0.2/src/sublibF4===========               |
c |     CHKTRC  get number of samples, sample interval, and    |
c |             number of traces in the input data file.       |
c |             Also returns values for progress bar, stdout   |
c |     BSEGIN  Reads a seismic trace.                         |
c |     BSEGOUT Write a seismic trace.                         |
c |     PLTBAR  Writes progress bar on stdout                  |
c |                                                            |
c --------------------------------------------------------------
c
      include '../include/bsegy.inc'
      parameter (ndim=32767,ncolm=300,nst=50)
      real*4            s1(ndim),r0(2),r1(2)
      character*40      infil , outfil, outlst
      character*40      infilx(nst), outfil2, outfil3
      character*4       aplc
      real*4            svx(nst),svy(nst)
      integer*2         irow(ncolm)
      integer*2         itest(ncolm)
      character*240     card1
      common /header/ hd
c
c...define application prefix
      write(aplc,'(''bref'')')
c
c...get parameters-------------------------------------------
      call GETPRM(nargsx,infil,outfil,
     +outfil2,outfil3,nshots,rmin,rmax,infilx,isw1,irecip,nst)     
c
c...initialize irow
      do 6 j=1,ncolm
      itest(j)=0
    6 irow(j)=0
      zero=0.0
c
c...clear output file...
      open(unit=3,file=outfil,access='sequential')
      open(unit=4,file=outfil2,access='sequential')
      open(unit=8,file=outfil3,access='sequential')
c revision 01Sep2012
      open(unit=9,file="scratch3",access='sequential')
      open(unit=10,file="scratch4",access='sequential')
      close(9,status='delete')
      close(10,status='delete')
      close(3,status='delete')
      close(4,status='delete')
      close(8,status='delete')
c
c...open output files
      open(unit=3,file=outfil,access='sequential')
      open(unit=4,file=outfil2,access='sequential')
      open(unit=8,file=outfil3,access='sequential')
      open(unit=9,file="scratch3",access='sequential')
      open(unit=10,file="scratch4",access='sequential')
c
c...START SHOT LOOP////////////////////////////////////
      do 25 jshot=1,nshots
      infil(1:12)=infilx(jshot)(1:12)
c
c...assign I/O units, initialize....
      iunit1=1
      iunit2=2
      icall1=0
      icall2=0
      if (jshot.eq.1) then
c...open list file
      io11=11
      write(outlst,'(a,a,''.lst'')') aplc,infil(1:4)
      open(unit=io11,file=outlst,access='sequential')
      endif
c
c...save shot coordinates for constraint equations
      jrec=1
      CALL BSEGIN(iunit1,ndim,
     +icall1,npts,s1,fsamin,infil,jrec,iexit)
c
c...shot and receiver coordinates
      call findxyz(sx,sy,sz,sd,rx,ry,rz,range,rangex)
c     
      if(irecip.eq.0) then
      svx(jshot)=sx
      svy(jshot)=sy
      write(io11,'(''       shot x,y,z='',3(f10.3,2x))') sx,sy,sz
      else
      svx(jshot)=rx
      svy(jshot)=ry
      write(io11,'(''   receiver x,y,z='',3(f10.3,2x))') rx,ry,rz
      endif
      close(iunit1)
   25 continue
      write(io11,'(''END SHOTS 0.0 0.0 0.0'')')
c
      do 800 jshot=1,nshots
      infil(1:12)=infilx(jshot)(1:12)
c
c...assign I/O units, initialize....
      iunit1=1
      iunit2=2
      icall1=0
      icall2=0
c
c...input a trace and check dimensions...
      jrec=1
      CALL BSEGIN(iunit1,ndim,
     +icall1,npts,s1,fsamin,infil,jrec,iexit)
c
c--------------------------------------------$$
      if(jshot.eq.1) then
c
c...count traces...
      jrec = 0
      ntr = 0
      r1(1)=0.
      r1(2)=0.
      arcvsp=0.
      jacnt=0
 10   jrec = jrec + 1
      CALL BSEGIN(iunit1,ndim,
     +icall1,npts,s1,fsamin,infil,jrec,iexit)
      if(iexit.eq.1) go to 20
c
c...receiver x-y coordinates
      call findxyz(sx,sy,sz,sd,rx,ry,rz,range,rangex)
c
      if(irecip.ne.0) then
c...use shot as if receiver
      rx=sx
      ry=sy
      endif
c
      r0(1)=r1(1)
      r0(2)=r1(2)
      r1(1)=rx
      r1(2)=ry
      rng1=sqrt( (r0(1)-r1(1))**2+(r0(2)-r1(2))**2 )
c
c...compute average receiver spacing
      if(jrec.ge.2) then
      arcvsp=arcvsp+abs(rng1)
      jacnt=jacnt+1
      endif
c
      ntr = ntr + 1
      go to 10
 20   ntrace=ntr
      arcvsp=arcvsp/float(jacnt)
c
c...output listing-------------------------------------------
      call LSTPRM(io11,ntrace,outfil,eps1,arcvsp,
     +infil,outfil2,outfil3,nshots,rmin,rmax,isw1,irecip)     
c
c--------------------------------------------$$
      endif
      write(io11,'(10x,a)') infil(1:12)
c
c...START Traces LOOP========
      ntr=0
      nstop=ntrace+nshots
      do 750 jrec=1,ntrace
c
c...read traces, compute--------------------
c
c...input a trace
      CALL BSEGIN(iunit1,ndim,
     +icall1,npts,s1,fsamin,infil,jrec,iexit)
c
      if(iexit.eq.1) go to 600
c
c...shot and receiver coordinates
      call findxyz(sx,sy,sz,sd,rx,ry,rz,range,rangex)
c
      if (isw1.eq.0) then
c...refraction analysis
      range=sqrt( (sx-rx)**2+(sy-ry)**2 )
      else
c...direct wave analysis
      range=sqrt( (sx-rx)**2+(sy-ry)**2+(sz-rz-sd)**2)
      endif
      pick=float(mst)/1.E+06
      if(irecip.eq.0) then
      write(8,'(2x,i5,3x,a4,2x,f10.1)') jrec,rstnam,rz
      else
      write(8,'(2x,i5,3x,a4,2x,f10.1)') jrec,spname,sz
      endif
      ntr = ntr + 1
c
c...compare to shot coordinates for constraint equation
      if(isw1.eq.0) then
      do 35 ks=1,nshots
      if(irecip.eq.0) then
      rdist=sqrt(  (rx-svx(ks))**2+(ry-svy(ks))**2 )
      else
      rdist=sqrt(  (sx-svx(ks))**2+(sy-svy(ks))**2 )
      endif
c revision 01Sep2012
c     are shot and receiver at same location?
c     if yes, constrain delay times to be equal
      if(rdist.lt.eps1) then
      irow(ks)=9
      irow(nshots+jrec)=-9
      itest(jrec)=1+itest(jrec)
c
c...output a row of system matrix
      write(card1,'(300(1x,i2))') (irow(kk),kk=1,nstop)
      kstart=nstop*3+3
      write(card1(kstart:kstart+10),'(f10.3)') zero
c revision 01Sep2012 write constraint equation to scratch
      write(9,'(a)') card1(1:kstart+10)
c      write(3,'(a)') card1(1:kstart+10)
c
c...output an element of data vector
c     revision 01Sep2012
      write(10,'(1x,f10.5,2x,i5)') zero,int(zero)
c      write(4,'(1x,f10.5,2x,i5)') zero,int(zero)
c
c...reset irow
      irow(ks)=0
      irow(nshots+jrec)=0
      endif
c     end of constraint block
   35 continue
c     next shot
      endif
c     end of refraction case block
c
c...test if within offset range (bad picks are set to zero)
      if(range.le.rmax .and. range.ge.rmin .and. pick.ne.0.) then
c
c...set matrix elements, this row
      irow(jshot)=1
      irow(nshots+jrec)=1
      itest(jrec)=1+itest(jrec)
c
c...output a row of system matrix
      write(card1,'(300(1x,i2))') (irow(kk),kk=1,nstop)
      kstart=nstop*3+3
      write(card1(kstart:kstart+10),'(f10.3)') range
      write(3,'(a)') card1(1:kstart+10)
c
c...output an element of data vector
      if(irecip.eq.0) then
      write(4,'(1x,f10.5,3x,a4)') pick,rstnam
      else
      write(4,'(1x,f10.5,3x,a4)') pick,spname
      endif
c
c...reset irow
      irow(jshot)=0
      irow(nshots+jrec)=0
      endif
c...end of offset range test
c
  750 continue
c     next trace
      close(iunit1)
  800 continue
c     next shot
c
c...END LOOP========================================....
600   continue
c
c...add constraint equations if a whole column is zero
c...set delay time to zero
      if(isw1.eq.0) then
      pick=0.
      range=0.
      do 900 jrec=1,ntrace
      if(itest(jrec).eq.0) then
      irow(nshots+jrec)=9
      write(card1,'(300(1x,i2))') (irow(kk),kk=1,nstop)
      kstart=nstop*3+3
      write(card1(kstart:kstart+10),'(f10.3)') range
      write(3,'(a)') card1(1:kstart+10)
      write(4,'(1x,f10.5,2x,i5)') pick, int(zero)
c
c...reset irow
      irow(nshots+jrec)=0
      endif
  900 continue
c     next trace
      endif
c
c...Rewind and then output shot loc = rec loc constraints
      rewind(9)
      rewind(10)
  901 continue
      write(card1,'(240(1h ))')
      read(9,'(a)',end=902) card1 
      write(3,'(a)') card1
      write(card1,'(240(1h ))')
      read(10,'(a)') card1 
      write(4,'(a)') card1
  902 continue
c
      write(*,'('' '')')
      write(6,240) ntr,outfil,outlst
240   format(' Traces Processed = ',i5/
     +5x,'====> output in ',a/'====> listing in ',a)
c
c
      close(11)
      close(iunit1)
      close(iunit2)
      close(3)
      close(4)
      close(8)
      close(9,status='delete')
      close(10,status='delete')
      stop
      end
c
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: getprm.f                                     |
c |------------------------------------------------------------|
c | get input parameters or help                               |
c --------------------------------------------------------------
c
      subroutine GETPRM(nargsx,infil,outfil,
     +outfil2,outfil3,nshots,rmin,rmax,infilx,isw1,irecip,nst)     
c
      character*40      infil,outfil
      character*40      infilx(*),outfil2,outfil3
      character*40      arg1,arg2,arg3,arg4,arg5,arg6
c
c
c...get input/output file names...
c   |--------------------------------------------|
c   | file naming convention                     |
c   | application prefix 4 characters "bxxx"     |
c   | data set suffix 4 characters "dddd"        |
c   | file name = bxxxdddd.seg                   |
c   |--------------------------------------------|
      write(*,*) '|---------------------------------|'
      write(*,*) '| Copyright (C) 2017  P. Michaels |'
      write(*,*) '|       All rights reserved       |'
      write(*,*) '|see GNU General Public License   |'
      write(*,*) '|---------------------------------|'
c
ccc---LINUX VERSION
      nargsx=iargc()
      karg=-1
c...test for help
      if(nargsx.ge.(2+karg)) then
      call getarg(1,arg1)
      if(arg1(1:2).eq.'-h') then
      write(*,*) ' '
      write(*,*) '|------------------------------------------|'
      write(*,*) '| Basic Seismic Utilities     FORTRAN      |'
      write(*,*) '| ONLINE HELP:                             |'
      write(*,*) '|------------------------------------------|'
      write(*,*) '| bref: Delay Time Refraction Analysis     |'
      write(*,*) '| Builds matrices for input to Scilab      |'
      write(*,*) '|------------------------------------------|'
      write(*,*) ' '
      write(*,*)
     +'bref line# nshots rmin rmax irefdir irecip infil1 infil2 infil3 '
      write(*,*)' '
      write(*,*) 'line#:  =line number for refraction/direct analysis'
      write(*,*) 'nshots  =number of shot records to use (<=10)'
      write(*,*) 'rmin    =minimum offset to include'
      write(*,*) 'rmax    =maximum offset to include'
      write(*,*) 'irefdir =0 refraction analysis '
      write(*,*) '        =1 direct wave analysis '
      write(*,*) 'irecip  =0 normal shooting, shot fixed '
      write(*,*) '        =1 reciprocal shooting, receiver fixed'
      write(*,*) 'infil1  =input file 1'
      write(*,*) 'infil2  =input file 2'
      write(*,*) 'infil3  =input file 3 etc....'
      write(*,*) ' '
      stop
      endif
      endif
      if(nargsx.lt.(2+karg)) then
      write(*,*) '  enter: line number (4 characters)'
      read(*,'(a13)') infil
      else
      call getarg(1,infil)
      endif
      call namechk(12,infil)

      write(outfil,'(''G'',a)') infil(1:4)
      write(outfil2,'(''D'',a)') infil(1:4)
      write(outfil3,'(''E'',a)') infil(1:4)
c
c...get parameters
      if(nargsx.lt.(3+karg)) then
      write(*,'(''  Enter: number of shots'')')
      read(*,*) nshots
      else
      call getarg(2,arg2)
      read(arg2,'(i10)') nshots
      endif
c
      if(nshots.gt.nst) then
      write(*,*) '  ABORT!! shot dimension exceeded'
      endif
c
      if(nargsx.lt.(4+karg)) then
      write(*,'(''  Enter: minimum offset to include'')')
      read(*,*) rmin
      else
      call getarg(3,arg3)
      read(arg3,'(f10.0)') rmin
      endif
      if(rmin.lt.0.) rmin=0.
      if(nargsx.lt.(5+karg)) then
      write(*,'(''  Enter: maximum offset to include'')')
      read(*,*) rmax
      else
      call getarg(4,arg4)
      read(arg4,'(f10.0)') rmax
      endif
c
c...will output be for refraction or direct wave analysis?
      if(nargsx.lt.(6+karg)) then
      write(*,*)'  Is output for refraction or direct analysis?'
      write(*,*)'  0=refraction  1=direct '
      read(*,*) isw1
      else
      call getarg(5,arg5)
      read(arg5,'(i1)') isw1
      endif
c
c...reciprocal shooting?
      if(nargsx.lt.(7+karg)) then
      write(*,*)'  irecip  0=normal  1=reciprocal shooting'
      read(*,*) irecip
      else
      call getarg(6,arg6)
      read(arg6,'(i1)') irecip
      endif
c
c...read file names
      do 22 jshot=1,nshots
      k=(8+karg)+jshot
      if(nargsx.lt.(k+karg)) then
      write(*,*)'  Enter: input file name'
      read(*,'(a13)') infilx(jshot)
      else
      k=k-1
      call getarg(k,arg6)
      read(arg6,'(a)') infilx(jshot)
      arg6=' '
      endif
      call namechk(12,infilx(jshot))
   22 continue
c
      return
      end
c
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: lstprm.f                                     |
c |------------------------------------------------------------|
c | echo check of input parameters, writes to a history file   |
c --------------------------------------------------------------
c
      subroutine LSTPRM(io11,ntrace,outfil,eps1,arcvsp,
     +infil,outfil2,outfil3,nshots,rmin,rmax,isw1,irecip)     
      character*40      outfil,infil,outfil2,outfil3
c
c...open output listing file...
      write(io11,*) '|---------------------------------|'
      write(io11,*) '| Copyright (C) 2017  P. Michaels |'
      write(io11,*) '|       All rights reserved       |'
      write(io11,*) '|see GNU General Public License   |'
      write(io11,*) '|---------------------------------|'
      write(io11,'('' Program bref'')')
      write(io11,'('' Line Number: '',a)') infil
      write(io11,'('' System matrix output File: '',a)') outfil
      write(io11,'('' Data vector output File: '',a)') outfil2
      write(io11,'('' Elevations file:         '',a)') outfil3
      write(io11,'(''  Number of traces='',i5)') ntrace
      write(io11,'(''  Parameters:  '')')
      write(io11,'(''     Number of shots='',i5)') nshots
      write(io11,'(''     Minimum offset ='',f10.1)') rmin
      write(io11,'(''     Maximum offset ='',f10.1)') rmax
      write(io11,'(''     isw1= '',i5)') isw1
      write(io11,'(''     irecip= '',i5)') irecip
      if(isw1.eq.0) then
      write(io11,'(''     Refraction analysis'')')
      else
      write(io11,'(''     Direct Wave analysis'')')
      endif
      write(io11,'(''     Average Receiver Spacing='',f10.4)') arcvsp
      eps1=0.75*arcvsp
      write(io11,'(''     Ground Consistent Zone='',f10.4)') eps1
      write(io11,'(''     Input File Names:'')')
      write(io11,'(50(1h-))')
      write(io11,'(''    '')')
c
      return
      end


