c |$Id: btor.f,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $          |
c --------------------------------------------------------------
c | Basic Seismic Utilities (BSU)                              |
c | LANGUAGE: Fortran77                                        |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: btor.f                                       |
c | DIRECTORY: bsu-2.0.2/src/Fort/bsegy                        |
c |------------------------------------------------------------|
c | btor reads hodo.lst and inserts azimuth and vertical angles|
c |            into trace headers (from hodogram analysis)     |
c |            for r-and t-components (channels 2 and 3)       |
c |                                                            |
c | Designed for VSP use only:  Assumes 6 channel record       |
c | with channel definition as follows                         |
c | Down Hole Tool 1=vertical 2=radial 3=transverse            |
c | Reference Phone 4=vertical 5=radial 6=transverse           |
c | Only channels 2 and 3 headers affected by btor             |
c |------------------------------------------------------------|
c |  NOTICE:                                                   |
c |  Copyright (C) 2017 Paul Michaels                          |
c |  <pm@cgiss.boisestate.edu>                                 |
c |  This program is free software; you can                    |
c |  redistribute it and/or modify it under the terms          |
c |  of the GNU General Public License as published            |
c |  by the Free Software Foundation; either version           |
c |  3 of the License, or (at your option) any later           |
c |  version.  This program is distributed in the              |
c |  hope that it will be useful, but                          |
c |  WITHOUT ANY WARRANTY; without even the implied            |
c |  warranty of MERCHANTABILITY or FITNESS FOR A              |
c |  PARTICULAR PURPOSE.  See the GNU General Public           |
c |  License for more details.                                 |
c |  You should have received a copy of the GNU                |
c |  General Public License along with this program;           |
c |  if not, write to the Free Software Foundation,            |
c |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
c |------------------------------------------------------------|
c |  PARAMETERS:                                               |
c |  ndim = maximum number of samples in a seismic trace       |
c |------------------------------------------------------------|
c |  INCLUDE FILE REQUIRED:                                    |
c |  ../include/bsegy.inc    (modified segy header definitions)|
c |                (no reel header, floating point data)       |
c |------------------------------------------------------------|
c |  SUBROUTINES CALLED:                                       |
c | In-line===============                                     |
c |     GETPRM  get input parameters from command line or      |
c |             interactive query.  Should be inline with      |
c |             main program, specific to application.         |
c |     LSTPRM  echo check of input parameters from GETPRM.    |
c |             Values written to a history file (*.lst)       |
c |     READLS  Reads triplets from bhod.lst file              |
c | Located in bsu-2.0.2/src/sublibF4===========               |
c |     CHKTRC  get number of samples, sample interval, and    |
c |             number of traces in the input data file.       |
c |             Also returns values for progress bar, stdout   |
c |     BSEGIN  Reads a seismic trace.                         |
c |     BSEGOUT Write a seismic trace.                         |
c |     PLTBAR  Writes progress bar on stdout                  |
c |                                                            |
c --------------------------------------------------------------
c
      include '../include/bsegy.inc'
      parameter (ndim=32767,ndimx=1000)
c
      integer*4         ifnum(ndimx)
      real*4            raz(ndimx),taz(ndimx)
c
      real*4            s1(ndim)
      character*4       aplc
      character*40      infil, outfil, outlst, prfx, lstfil
      common /header/ hd
c
c...assign I/O units, initialize....
      jrec=1
      iunit1=1
      iunit2=2
      icall1=0
      icall2=0
      write(infil,'('' '')')
      write(outfil,'('' '')')
c
c...define application prefix
      write(aplc,'(''btor'')')
c
c...get parameters-------------------------------------------
      call GETPRM(nargsx,prfx,lstfil,
     +isw1,maxtr)
c
c...count number of files and check dimensions----------------
      call READLS(lstfil,ifnum,raz,taz,
     +bar,invbar,ibar,nrecrd,ndimx)
c
c...name outputfile
      write(outfil,'(a,a,''.seg'')') aplc(1:4),infil(1:4)
c
c...output listing-------------------------------------------
      call LSTPRM(io11,infil,nrecrd,aplc,outlst,
     +lstfil,prfx,isw1,maxtr)
c
      icall1=0
      icall2=0
      ntr=0
c
c...data loop=======================================....
c
      write(io11,*) '  '
      write(io11,*) '     Input file   R-azi   T-azi'
c
      do 100 irec=1,nrecrd
c
c...assign input file name
      ifn=ifnum(irec)
c
c            begin updown loop
c...modification for vertical impact source
      if(isw1.eq.0) then
      judmx=1
      else
      judmx=2
      endif
      do 110 jupdwn=1,judmx
      if(jupdwn.eq.2) then
      ifn=ifn+isw1
      endif
      write(infil,'(a,i3.3,''.seg'')') prfx(1:1),ifn
c
      write(io11,'(5x,a,5x,i5,2x,i5)') infil(1:8),int(raz(irec)),
     +int(taz(irec))
c
c...clear output file...
      write(outfil,'(a,a,''.seg'')') aplc(1:4),infil(1:4)
      open(10,file=outfil,access='direct',recl=240)
      close(10,status='delete')
c
      icall1=0
      icall2=0
c
c///////////////////////begin trace loop
      do 105 jrec=1,maxtr
c
c...input a trace
      CALL BSEGIN(iunit1,ndim,
     +icall1,npts,s1,fsamin,infil,jrec,iexit)
c
      if(jrec.eq.2) then
      geoazi=int(raz(irec))
      geover=90
      endif
      if(jrec.eq.3) then
      geoazi=int(taz(irec))
      geover=90
      endif
c
c...output trace
      iunit2=2
      CALL BSEGOUT(iunit2,
     +icall2,npts,s1,fsamin,outfil,jrec)
  105 continue
c---------------------------end of trace loop
  110 continue
c-----------------------end of updown loop
c
c...display progress....
      call pltbar(bar,invbar,ibar,irec,nrecrd)
c
c
c...END LOOP========================================....
  100 continue
c
c
      write(*,'('' '')')
      write(6,240) nrecrd,outfil,outlst
240   format(/' Records Processed = ',i5/
     +5x,'====> output in ',a/'====> listing in ',a)
c
      close(io11)
      close(iunit1)
      close(iunit2)
      stop
      end
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: getprm.f                                     |
c |------------------------------------------------------------|
c | get input parameters or help                               |
c --------------------------------------------------------------
c
      subroutine GETPRM(nargsx,prfx,lstfil,
     +isw1,maxtr)
c
      character*40      prfx,lstfil
      character*40      arg1,arg3,arg4
c
c
c...get input/output file names...
c   |--------------------------------------------|
c   | file naming convention                     |
c   | application prefix 4 characters "bxxx"     |
c   | data set suffix 4 characters "dddd"        |
c   | file name = bxxxdddd.seg                   |
c   |--------------------------------------------|
      write(*,*)'|---------------------------------|'
      write(*,*)'| Copyright (C) 2017  P. Michaels |'
      write(*,*)'|       All rights reserved       |'
      write(*,*)'|see GNU General Public License   |'
      write(*,*)'|---------------------------------|'
c
ccc---LINUX VERSION
      nargsx=iargc()
      karg=-1
c...test for help
      if(nargsx.ge.(2+karg)) then
      call getarg(1,arg1)
      if(arg1(1:2).eq.'-h') then
      write(*,*)' '
      write(*,*) '|------------------------------------------|'
      write(*,*) '| Basic Seismic Utilities     FORTRAN      |'
      write(*,*) '| ONLINE HELP:                             |'
      write(*,*) '|------------------------------------------|'
      write(*,*) '| btor: Applies azimuth and vertical angles|'
      write(*,*) '| to geophone trace headers (from a file)  |'
      write(*,*) '|------------------------------------------|'
      write(*,*) ' '
      write(*,*)
     +'btor lstfil, prfx, isw1 maxtr'
      write(*,*) ' '
      write(*,*) 'lstfil  =input list file name (ex. bhod.lst) '
      write(*,*) 'prfx    =*.seg file prefix (one character) '
      write(*,*) 'isw1    =up/down switch '
      write(*,*) '      -1=apply to *.lst file and one less '
      write(*,*) '      +1=apply to *.lst file and one more '
      write(*,*) '       0=IF VERTICAL IMPACT source '
      write(*,*) 'maxtr   =maximum number of traces in shot record '
      write(*,*) '       6= 3 components down-hole, 3 ref-phone'
      write(*,*) '       7= 3 down, 3 ref-phone, 1 load cell'
      write(*,*) ' '
      write(*,*) ' '
      write(*,*) 'EXAMPLES: '
      write(*,*) '       [if file number (col. 1 of bhod.lst) is 005 '
      write(*,*) '       and isw1=-1, the azimuth and vertical angle '
      write(*,*) '       also applied to file 004] '
      write(*,*) ' '
      write(*,*) '       [if file is 005 and isw1=+1, azi and vert '
      write(*,*) '       applied to 006 also] '
      write(*,*) ' '
      stop
      endif
      endif
      if(nargsx.lt.(2+karg)) then
      write(*,*) '  enter:name of input list file(ie. bhod.lst)'
      read(*,'(a13)') lstfil
      else
      call getarg(1,lstfil)
      endif
      call namechk(12,lstfil)
c
      if(nargsx.lt.(3+karg)) then
      write(*,*) '  enter: 1 char prefix, prfx'
      read(*,'(a)') prfx(1:1)
      else
      call getarg(2,prfx)
      endif
c
      if(nargsx.lt.(4+karg)) then
      write(*,*) 'Enter isw1    =up/down switch '
      write(*,*) '      -1=apply to *.lst file and one less '
      write(*,*) '      +1=apply to *.lst file and one more '
      write(*,*) '       0=IF VERTICAL IMPACT source '
      read(*,*) isw1
      else
      call getarg(3,arg3)
      read(arg3,'(i5)') isw1
      endif
      if(isw1.lt.-1) isw1=-1
      if(isw1.gt.+1) isw1=+1
c
c
      if(nargsx.lt.(5+karg)) then
      write(*,*) 'maxtr   =maximum number of traces in shot record '
      write(*,*) '       6= 3 components down-hole, 3 ref-phone'
      write(*,*) '       7= 3 down, 3 ref-phone, 1 load cell'
      read(*,*) maxtr
      else
      call getarg(4,arg4)
      read(arg3,'(i5)') maxtr
      endif
      if(maxtr.lt.6) maxtr=6
      if(maxtr.gt.7) maxtr=7
c
      return
      end
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: readls.f                                     |
c |------------------------------------------------------------|
c | read list file and load azimuth of r and t-components      |
c | (channels 2 and 3 in 6 channel vsp recording)              |
c --------------------------------------------------------------
c
      subroutine READLS(lstfil,ifnum,raz,taz,
     +bar,invbar,ibar,nrecrd,ndimx)
c
      character*40      lstfil
      integer*4         ifnum(*)
      real*4            raz(*),taz(*)
c
      open(unit=1,file=lstfil,access='sequential')
c
      nrecrd=0
  500 nrecrd=nrecrd+1
      if(nrecrd.gt.ndimx) then
      write(*,*)'  ABORT--too many entries in *.lst file'
      stop
      endif
      read(1,*,end=600) ifnum(nrecrd),raz(nrecrd),taz(nrecrd)
      go to 500
  600 continue
      nrecrd=nrecrd-1
      bar=float(nrecrd)/50.
      ibar=int(bar+0.5)
      invbar=int(1./bar+0.5)
      close(1)
      return
      end
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: lstprm.f                                     |
c |------------------------------------------------------------|
c | echo check of input parameters, writes to a history file   |
c --------------------------------------------------------------
c
      subroutine LSTPRM(io11,infil,nrecrd,aplc,outlst,
     +lstfil,prfx,isw1,maxtr)
      character*4       aplc
      character*40      outlst,lstfil,prfx,infil
c
c...open output listing file...
      io11=11
      write(outlst,'(a,a,''.lst'')') aplc,lstfil(1:4)
      open(io11,file=outlst,access='sequential')
      write(io11,*)'|---------------------------------|'
      write(io11,*)'| Copyright (C) 2017  P. Michaels |'
      write(io11,*)'|       All rights reserved       |'
      write(io11,*)'|see GNU General Public License   |'
      write(io11,*)'|---------------------------------|'
c
      write(io11,'('' Program btor  '')')
      write(io11,'('' Input File: '',a)') infil
c
      write(io11,'(''  Number of records='',i5)') nrecrd
      write(io11,'(''  Parameters:  '')')
      write(io11,'(''   lstfil= '',a)') lstfil
      write(io11,'(''   prfx= '',a)')   prfx(1:1)
      write(io11,'(''   isw1= '',i5)')  isw1
      write(io11,'(''   maxtr= '',i5)') maxtr
      write(io11,'(50(1h-))')
      write(io11,'(''    '')')
c
      return
      end
