c |$Id: bvel.f,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $          |
c --------------------------------------------------------------
c | Basic Seismic Utilities (BSU)                              |
c | LANGUAGE: Fortran77                                        |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: bmst.f                                       |
c | DIRECTORY: bsu-2.0.2/src/Fort/bsegy                        |
c |------------------------------------------------------------|
c | Apply a phase velocity alignment (either in the frequency  |
c | domain or in the time domain)                              |
c |------------------------------------------------------------|
c |  NOTICE:                                                   |
c |  Copyright (C) 2017 Paul Michaels                          |
c |  <pm@cgiss.boisestate.edu>                                 |
c |  This program is free software; you can                    |
c |  redistribute it and/or modify it under the terms          |
c |  of the GNU General Public License as published            |
c |  by the Free Software Foundation; either version           |
c |  3 of the License, or (at your option) any later           |
c |  version.  This program is distributed in the              |
c |  hope that it will be useful, but                          |
c |  WITHOUT ANY WARRANTY; without even the implied            |
c |  warranty of MERCHANTABILITY or FITNESS FOR A              |
c |  PARTICULAR PURPOSE.  See the GNU General Public           |
c |  License for more details.                                 |
c |  You should have received a copy of the GNU                |
c |  General Public License along with this program;           |
c |  if not, write to the Free Software Foundation,            |
c |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
c |------------------------------------------------------------|
c |  PARAMETERS:                                               |
c |  ndim = maximum number of samples in a seismic trace       |
c |------------------------------------------------------------|
c |  INCLUDE FILE REQUIRED:                                    |
c |  ../include/bsegy.inc    (modified segy header definitions)|
c |                (no reel header, floating point data)       |
c |------------------------------------------------------------|
c |  SUBROUTINES CALLED:                                       |
c | In-line===============                                     |
c |     GETPRM  get input parameters from command line or      |
c |             interactive query.  Should be inline with      |
c |             main program, specific to application.         |
c |     LSTPRM  echo check of input parameters from GETPRM.    |
c |             Values written to a history file (*.lst)       |
c |     APLYV   Apply velocity time shifts in time domain      |
c | Located in bsu-2.0.2/src/sublibF4===========               |
c |     CHKTRC  get number of samples, sample interval, and    |
c |             number of traces in the input data file.       |
c |             Also returns values for progress bar, stdout   |
c |     BSEGIN  Reads a seismic trace.                         |
c |     BSEGOUT Write a seismic trace.                         |
c |     PLTBAR  Writes progress bar on stdout                  |
c |     NRAD2   Determine number samples radix 2 FFT           |
c |     NLOGN   Radix 2 FFT                                    |
c |     VSHFT   Called by APLYV, applies shift by interpolation|
c |                                                            |
c --------------------------------------------------------------
c
      include '../include/bsegy.inc'
      parameter (ndim=32767)
      parameter (mdim=256)
c
      real*4            depth(mdim),angle(mdim),cosa(mdim)
      complex           x(2*ndim)
c
      real*4            s1(ndim),s2(ndim)
      character*40      infil , outfil, aplc, outlst
      EXTERNAL cshift
      common /header/ hd
c
c...assign I/O units, initialize....
      jrec=1  
      iunit1=1
      iunit2=2
      icall1=0
      icall2=0
c
c...define application prefix
      write(aplc,'(''bvel'')')
c
c...get parameters-------------------------------------------
      call GETPRM(nargsx,infil,vel,ifast)
c
c...count traces and check dimensions------------------------
      call CHKTRC(iunit1,icall1,npts,s1,fsamin,infil,
     +bar,invbar,ibar,ntrace,ndim)
c
c...name outputfile
      write(outfil,'(a,a,''.seg'')') aplc(1:4),infil(1:4)
c
c...output listing-------------------------------------------
      call LSTPRM(io11,ntrace,aplc,outlst,outfil,
     +infil,vel,ifast)
c
c...clear output file... 
      open(10,file=outfil,access='direct',recl=240)
      close(10,status='delete')
c
      icall1=0
      icall2=0
      ntr = 0
c
c...data loop=======================================....
c
      do 100 jrec=1,ntrace
c
c...input a trace
      CALL BSEGIN(iunit1,ndim,
     +icall1,npts,s1,fsamin,infil,jrec,iexit)
c
      ntr = ntr + 1
c
      call findxyz(sx,sy,sz,sd,rx,ry,rz,frange,rangex)
c
c...compute geometry for each trace, jrec: depth,angle,cos(angle)
      if(jrec.gt.mdim) then
      write(*,*)' ABORT--number of traces>mdim dimension'
      stop
      endif
c
      depth(jrec)=sz-sd-rz
      angle(jrec)=acos(depth(jrec)/frange)
      angle(jrec)=angle(jrec)*(180./3.1415926)
      cosa(jrec)=depth(jrec)/frange
 100  continue  
c
c...END LOOP========================================....
c
c...compute depth increment
      deltaz=depth(1)-depth(2)
      write(io11,'(''  Depth increment= '',f10.2)') deltaz
c
c...apply phase velocity shift
c============================slow option=================
      if(ifast.eq.0) then
c
      call NRAD2(npts,npr2)
      npr2=2*npr2
c
c...compute bias shift, dtbias
      jrec=1
      dtbias=(depth(jrec)/(vel))/cosa(jrec)
c
      do 200 jrec=1,ntrace
c
c...input a trace
      CALL BSEGIN(iunit1,ndim,
     +icall1,npts,s1,fsamin,infil,jrec,iexit)
c
      do 205 jt=1,npts
 205  x(jt)=cmplx(s1(jt),0.0)
      nptp1=npts+1
      do 210 jt=nptp1,npr2
 210  x(jt)=cmplx(0.0,0.0)
c
c...compute time shift
      if(angle(jrec).lt.88.) then
      dt=-(depth(jrec)/(vel))/cosa(jrec) + dtbias
c
      write(io11,989) jrec,depth(jrec),angle(jrec),cosa(jrec),dt
  989 format(5x,i5,5x,f6.2,5x,f6.1,5x,f6.4,5x,f6.4)
c
c...take forward fft
      sign=-1.0
      call NLOGN(npr2,x,sign)
c
c...apply shift (phase rotation)
      call CSHIFT(npr2,x,dt,fsamin)
c
c...take inverse fft
      sign=+1.0
      call NLOGN(npr2,x,sign)
c
c...load output trace
      do 215 jt=1,npts
  215 s2(jt)=real(x(jt))
      else
      do 220 jt=1,npts
  220 s2(jt)=0.
      endif
c
c...output trace
      CALL BSEGOUT(iunit2,
     +icall2,npts,s2,fsamin,outfil,jrec)
c
c...display progress....
      call pltbar(bar,invbar,ibar,jrec,ntrace)
c
c...end traces loop
  200 continue
c
c=====================fast option========================
      else
c
c...compute bias shift, dtbias
      jrec=1
      dtbias=(depth(jrec)/(vel))/cosa(jrec)
c
      do 300 jrec=1,ntrace
c
c...input a trace
      CALL BSEGIN(iunit1,ndim,
     +icall1,npts,s1,fsamin,infil,jrec,iexit)
c
c
c...compute time shift
      if(angle(jrec).lt.88.) then
      dt=-(depth(jrec)/(vel))/cosa(jrec) + dtbias
      else
      dt=9999.
      endif
c
      call APLYV(npts,s1,s2,jrec,vel,depth,cosa,angle,fsamin,1,ntrace)
c
      write(io11,989) jrec,depth(jrec),angle(jrec),cosa(jrec),dt
c
c...output trace
      CALL BSEGOUT(iunit2,
     +icall2,npts,s2,fsamin,outfil,jrec)
c
c...display progress....
      call pltbar(bar,invbar,ibar,jrec,ntrace)
c
c...end traces loop
  300 continue
      endif
c
      write(*,'('' '')')
      write(6,240) ntrace,outfil,outlst
240   format(' Traces Processed = ',i5/
     +5x,'====> output in ',a/'====> listing in ',a)
c
      close(io11)
      close(iunit1)
      close(iunit2)
      stop
      end
c
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: getprm.f                                     |
c |------------------------------------------------------------|
c | get input parameters or help                               |
c --------------------------------------------------------------
c
      subroutine GETPRM(nargsx,infil,
     +vel,ifast)
c
      character*40      infil
      character*40      arg1,arg2,arg3
c
c...get input/output file names...
c   |--------------------------------------------|
c   | file naming convention                     |
c   | application prefix 4 characters "bxxx"     |
c   | data set suffix 4 characters "dddd"        |
c   | file name = bxxxdddd.seg                   |
c   |--------------------------------------------|
      write(*,*)'|---------------------------------|'
      write(*,*)'| Copyright (C) 2017  P. Michaels |'
      write(*,*)'|       All rights reserved       |'
      write(*,*)'|see GNU General Public License   |'
      write(*,*)'|---------------------------------|'
c
ccc---LINUX VERSION
      nargsx=iargc()
      karg=-1
c...test for help
      if(nargsx.ge.(2+karg)) then
      call getarg(1,arg1)
      if(arg1(1:2).eq.'-h') then
      write(*,*)' '
      write(*,*) '|------------------------------------------|'
      write(*,*) '| Basic Seismic Utilities     FORTRAN      |'
      write(*,*) '| ONLINE HELP:                             |'
      write(*,*) '|------------------------------------------|'
      write(*,*) '| bvel: Apply velocity correction to data  |'
      write(*,*) '|        (linear wrt source-receiver range)|'
      write(*,*) '|------------------------------------------|'
      write(*,*)' '
      write(*,*)' USE FOR DOWN-HOLE DATA (use bred for surface data) '
      write(*,*)
     +'bvel infile  vel  ifast '
      write(*,*)'infile:   =input file name '
      write(*,*)'vel       =phase velocity to apply '
      write(*,*)'ifast     0=slow option (fft phase rotation'
      write(*,*)'          1=fast option (sample shift) '
      write(*,*)' '
      stop
      endif
      endif
      if(nargsx.lt.(2+karg)) then
      write(*,*)'  enter: name of input file'
      read(*,'(a13)') infil
      else
      call getarg(1,infil)
      endif
      call namechk(12,infil)
c
      if(nargsx.lt.(3+karg)) then
      write(*,*)'  Enter velocity'
      read(*,*) vel
      else
      call getarg(2,arg2)
      read(arg2,'(f8.0)') vel
      endif
c
      if(nargsx.lt.(4+karg)) then
      write(*,*)'ifast     0=slow option (fft phase rotation'
      write(*,*)'          1=fast option (sample shift) '
      write(*,*)'  Enter ifast option'
      read(*,*) ifast
      else
      call getarg(3,arg3)
      read(arg3,'(i5)') ifast
      endif
c
      return
      end
c
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: lstprm.f                                     |
c |------------------------------------------------------------|
c | echo check of input parameters, writes to a history file   |
c --------------------------------------------------------------
c
      subroutine LSTPRM(io11,ntrace,aplc,outlst,outfil,
     +infil,vel,ifast)
      character*4       aplc
      character*40      outlst,outfil,infil
c
c...open output listing file...
      io11=11
      write(outlst,'(a,a,''.lst'')') aplc,infil(1:4)
      open(io11,file=outlst,access='sequential')
      write(io11,*)'|---------------------------------|'
      write(io11,*)'| Copyright (C) 2017  P. Michaels |'
      write(io11,*)'|       All rights reserved       |'
      write(io11,*)'|see GNU General Public License   |'
      write(io11,*)'|---------------------------------|'
c
      write(io11,'('' Program bvel  '')')
      write(io11,'('' Input File: '',a)') infil
      write(io11,'('' Output File: '',a)') outfil
      write(io11,'(''  Number of traces='',i5)') ntrace
      write(io11,'(''  Parameters:  '')')
c
      write(io11,'(''  '')')
      write(io11,'(''  Velocity= '',f10.2)') vel
      write(io11,'(''  ifast =   '',i5)') ifast
      write(io11,'(''  '')')
      write(io11,'(50(1h-))')
      write(io11,'(''    '')')
      write(io11,901)
  901 format(5x,'Trace ',5x,'Depth ',5x,'Angle ',5x,'Cos(0)',
     +5x,'Shift',/)
      return
      end
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: aplyv.f                                      |
c |------------------------------------------------------------|
c |  NOTICE:                                                   |
c |  Copyright (C) 2017 Paul Michaels                          |
c |  <pm@cgiss.boisestate.edu>                                 |
c |  This program is free software; you can                    |
c |  redistribute it and/or modify it under the terms          |
c |  of the GNU General Public License as published            |
c |  by the Free Software Foundation; either version           |
c |  3 of the License, or (at your option) any later           |
c |  version.  This program is distributed in the              |
c |  hope that it will be useful, but                          |
c |  WITHOUT ANY WARRANTY; without even the implied            |
c |  warranty of MERCHANTABILITY or FITNESS FOR A              |
c |  PARTICULAR PURPOSE.  See the GNU General Public           |
c |  License for more details.                                 |
c |  You should have received a copy of the GNU                |
c |  General Public License along with this program;           |
c |  if not, write to the Free Software Foundation,            |
c |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
c |------------------------------------------------------------|
c | Apply velocity shift to trace                              |
c | Note: argument angle is taken between the vertical and a   |
c | line connecting the source and receiver. cosa is the cosine|
c | of angle (see stage1).                                     |
c | If angle > about 88 degrees, cosa -->zero, and so, to      |
c | avoid a divide by zero, skip those cases.  In essence, the |
c | geophone would be at the same elevation as the source.     |
c | The cosa correction is to make this a vertical correction. |
c --------------------------------------------------------------
c
      subroutine APLYV(npts,s1,s2,jrec,vel,depth,cosa,angle,fsamin,
     +jmin,jmax)
      real*4            s1(*),s2(*),depth(*),cosa(*),angle(*)
c
c...compute bias shift, dtbias
      dtbias=(depth(jmin)/vel)/cosa(jmin)
      dtbs2=(depth(jmax)/vel)/cosa(jmax)
      if(dtbs2.gt.dtbias) dtbias=dtbs2
c
c...compute time shift/////////////
      if(angle(jrec).lt.88.) then
      dt=((depth(jrec)/vel)/cosa(jrec))-dtbias
c
      call VSHFT(npts,s1,s2,dt,fsamin)
c
c/////over 88 degrees
      else
      do 222 jt=1,npts
  222 s2(jt)=0.
      endif
      return
      end
