c |$Id: bwht.f,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $          |
c --------------------------------------------------------------
c | Basic Seismic Utilities (BSU)                              |
c | LANGUAGE: Fortran77                                        |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: bwht.f                                       |
c | DIRECTORY: bsu-2.0.2/src/Fort/bsegy                        |
c |------------------------------------------------------------|
c | Spectral whitening by applying AEC to band-pass filtered   |
c | subsets of the data and summing.                           |
c |------------------------------------------------------------|
c |  NOTICE:                                                   |
c |  Copyright (C) 2017 Paul Michaels                          |
c |  <pm@cgiss.boisestate.edu>                                 |
c |  This program is free software; you can                    |
c |  redistribute it and/or modify it under the terms          |
c |  of the GNU General Public License as published            |
c |  by the Free Software Foundation; either version           |
c |  3 of the License, or (at your option) any later           |
c |  version.  This program is distributed in the              |
c |  hope that it will be useful, but                          |
c |  WITHOUT ANY WARRANTY; without even the implied            |
c |  warranty of MERCHANTABILITY or FITNESS FOR A              |
c |  PARTICULAR PURPOSE.  See the GNU General Public           |
c |  License for more details.                                 |
c |  You should have received a copy of the GNU                |
c |  General Public License along with this program;           |
c |  if not, write to the Free Software Foundation,            |
c |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
c |------------------------------------------------------------|
c |  PARAMETERS:                                               |
c |  ndim = maximum number of samples in a seismic trace       |
c |------------------------------------------------------------|
c |  INCLUDE FILE REQUIRED:                                    |
c |  ../include/bsegy.inc    (modified segy header definitions)|
c |                (no reel header, floating point data)       |
c |------------------------------------------------------------|
c |  SUBROUTINES CALLED:                                       |
c | In-line===============                                     |
c |     GETPRM  get input parameters from command line or      |
c |             interactive query.  Should be inline with      |
c |             main program, specific to application.         |
c |     LSTPRM  echo check of input parameters from GETPRM.    |
c |             Values written to a history file (*.lst)       |
c | Located in bsu-2.0.2/src/sublibF4===========               |
c |     CHKTRC  get number of samples, sample interval, and    |
c |             number of traces in the input data file.       |
c |             Also returns values for progress bar, stdout   |
c |     BSEGIN  Reads a seismic trace.                         |
c |     BSEGOUT Write a seismic trace.                         |
c |     PLTBAR  Writes progress bar on stdout                  |
c |     AGC     Automatic Gain Control (box car, energy envlp) |
c |     BOXIT   Called by AGC (box car smoother)               |
c |     CONV    Convolution, filter and trace                  |
c |     CFILT   Designs filters (zero phase) raised cosine env.|
c |                                                            |
c --------------------------------------------------------------
c
c
      include '../include/bsegy.inc'
      parameter (ndim=32767, im=1001)
      real*4            s1(ndim),s2(ndim),s3(ndim),s4(ndim),s5(ndim)
      real*4            s6(ndim)
      character*40      infil , outfil, aplc, outlst
c
      real*4 fil(im,im/50),env(im)
c
      namelist /filter/ npf,nzph,fil
c
      common /header/ hd
c
c...assign I/O units, initialize....
      jrec=1
      iunit1=1
      iunit2=2
      icall1=0
      icall2=0
c
c...define application prefix
      write(aplc,'(''bwht'')')
c
c...get parameters-------------------------------------------
      call GETPRM(nargsx,infil,
     +twide,freqc,bwdth,deltf,nfilt,im)
c
      hw=deltf/2.0
c
c...count traces and check dimensions------------------------
      call CHKTRC(iunit1,icall1,npts,s1,fsamin,infil,
     +bar,invbar,ibar,ntrace,ndim)
c
c...name outputfile
      write(outfil,'(a,a,''.seg'')') aplc(1:4),infil(1:4)
c
c...clear output file...
      open(10,file=outfil,access='direct',recl=240)
      close(10,status='delete')
c
c...compute number of points in filter
c---------------------------------------------------------
      dt=fsamin
      fln=deltf
      npf=int((1.0/fln)/dt)
      npf=int(float(npf)/2.0)*2 +1
      if(npf.gt.im) then
      write(*,*)' ABORT !!!! filter too long'
      write(*,'(''  number of points in filter =  '',i5)') npf
      stop
      endif
      nzph=npf/2+1
c
c...output listing-------------------------------------------
      freq1=freqc-(bwdth/2.0)
      call LSTPRM(io11,ntrace,aplc,outlst,outfil,
     +infil,twide,freqc,bwdth,deltf,nfilt,npf,freq1)
c
c...compute filters---------------------------------
      freq1=freqc-(bwdth/2.0)
      call cfilt(fil,nfilt,npf,deltf,freq1,hw,dt,env,im)
c
c...apply zero phase reference to operator so that
c   new center (npf/2+1) becomes zero reference
      nlead=npf-2*nzph+1
      if(nlead.ne.0) then
c--------------------------
      if(nlead.gt.0) then
      npf1=npf
      npf=npf+nlead
      if(npf.gt.im) then
      write(*,*) ' ABORT !!!! shifted filter too long'
      stop
      endif
c
      do 23 ifilt=1,nfilt
      index=npf
      do 89 jf=1,npf1
      index2=index-nlead
      fil(index,1)=fil(index2,1)
      index=index-1
   89 continue
      do 90 jf=1,nlead
   90 fil(jf,ifilt)=0.0
   23 continue
c--------------------------
      else
      do 24 ifilt=1,nfilt
      npf1=npf+1
      npf=npf-nlead
      if(npf.gt.im) then
      write(*,*) ' ABORT !!!! shifted filter too long'
      stop
      endif
      do 91 jf=npf1,npf
   91 fil(jf,ifilt)=0.00
   24 continue
      endif
c-------------------------
      write(io11,'(''  number of points in shifted filter='',i5)') npf
      endif

c
      icall1=0
      icall2=0
      ntr=0
c
c...data loop=======================================....
c
      do 100 jrec=1,ntrace
c
c...input a trace
      call BSEGIN(iunit1,ndim,
     +icall1,npts,s1,fsamin,infil,jrec,iexit)
c
      ntr = ntr + 1
c
c...filter the trace
c
      do 42 jt=1,npts
   42 s6(jt)=0.
c
      do 57 ifil=1,nfilt
      call conv(npts,s1,npf,fil,s2,ifil,im)
      call AGC(npts,s2,s3,s4,s5,twide,fsamin)
      do 58 jt=1,npts
   58 s6(jt)=s6(jt)+s5(jt)
   57 continue
c
c...output trace
      iunit2=2
      call BSEGOUT(iunit2,
     +icall2,npts,s6,fsamin,outfil,jrec)
c
c...display progress....
      call pltbar(bar,invbar,ibar,jrec,ntrace)
c
c
c...END LOOP========================================....
  100 continue
c
c
      write(*,'('' '')')
      write(6,240) ntrace,outfil,outlst
240   format(' Traces Processed = ',i5/
     +5x,'====> output in ',a/'====> listing in ',a)
c
      close(io11)
      close(iunit1)
      close(iunit2)
      stop
      end
c
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: getprm.f                                     |
c |------------------------------------------------------------|
c | get input parameters or help                               |
c --------------------------------------------------------------
c
      subroutine GETPRM(nargsx,infil,
     +twide,freqc,bwdth,deltf,nfilt,im)
c
      character*40      infil
      character*40      arg1,arg2,arg3,arg4,arg5
c
c
c...get input/output file names...
c   |--------------------------------------------|
c   | file naming convention                     |
c   | application prefix 4 characters "bxxx"     |
c   | data set suffix 4 characters "dddd"        |
c   | file name = bxxxdddd.seg                   |
c   |--------------------------------------------|
      write(*,*) '|---------------------------------|'
      write(*,*) '| Copyright (C) 2017  P. Michaels |'
      write(*,*) '|       All rights reserved       |'
      write(*,*) '|see GNU General Public License   |'
      write(*,*) '|---------------------------------|'
c
ccc---LINUX VERSION
      nargsx=iargc()
      karg=-1
c...test for help
      if(nargsx.ge.(2+karg)) then
      call getarg(1,arg1)
      if(arg1(1:2).eq.'-h') then
      write(*,*) ' '
      write(*,*) '|------------------------------------------|'
      write(*,*) '| Basic Seismic Utilities     FORTRAN      |'
      write(*,*) '| ONLINE HELP:                             |'
      write(*,*) '|------------------------------------------|'
      write(*,*) '| bwht: Non linear whitening program       |'
      write(*,*) '| AGC of bandpass filters, sum to output   |'
      write(*,*) '|------------------------------------------|'
      write(*,*) ' '
      write(*,*)
     +'bwht  infil  twide  fcent  bwdth  froll '
      write(*,*) ' '
      write(*,*) 'infil:    =input file to be filtered '
      write(*,*) 'twide:    =AGC window length in sec.  '
      write(*,*) 'fcent:    =center frequency (Hz)'
      write(*,*) 'bwdth:    =bandwidth (Hz)'
      write(*,*) 'froll:    =roll off (Hz) '
      write(*,*) ' '
      stop
      endif
      endif
      if(nargsx.lt.(2+karg)) then
      write(*,*) '  enter: name of input file'
      read(*,'(a13)') infil
      else
      call getarg(1,infil)
      endif
      call namechk(12,infil)
c
c...read input parameters
      if(nargsx.lt.(3+karg)) then
      write(*,*) '  AGC length in sec.'
      write(*,*) '  enter twide'
      read(*,*) twide
      else
      call getarg(2,arg2)
      read(arg2,'(f10.4)') twide
      endif
c
      if(nargsx.lt.(4+karg)) then
      write(*,'(''  enter center frequency'')')
      read(*,*) freqc
      else
      call getarg(3,arg3)
      read(arg3,'(f8.0)') freqc
      endif
c
      if(nargsx.lt.(5+karg)) then
      write(*,'(''  enter bandwidth'')')
      read(*,*) bwdth
      else
      call getarg(4,arg4)
      read(arg4,'(f8.0)') bwdth
      endif
c
      if(nargsx.lt.(6+karg)) then
      write(*,'(''  enter roll off bandwidth'')')
      read(*,*) deltf
      else
      call getarg(5,arg5)
      read(arg5,'(f8.0)') deltf
      endif
c
      nfilt=int(bwdth/deltf+0.5)+1
      write(*,'(''  Number of filters='',i5)') nfilt
c...check dimensions
      if(nfilt.gt.im/50) then
      write(*,*) ' ABORT--Too many filters'
      stop
      endif
c
      return
      end
c
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: lstprm.f                                     |
c |------------------------------------------------------------|
c | echo check of input parameters, writes to a history file   |
c --------------------------------------------------------------
c
      subroutine LSTPRM(io11,ntrace,aplc,outlst,outfil,
     +infil,twide,freqc,bwdth,deltf,nfilt,npf,freq1)
      character*4       aplc
      character*40      outlst,outfil,infil
c
c...open output listing file...
      io11=11
      write(outlst,'(a,a,''.lst'')') aplc,infil(1:4)
      open(io11,file=outlst,access='sequential')
      write(io11,*) '|---------------------------------|'
      write(io11,*) '| Copyright (C) 2017  P. Michaels |'
      write(io11,*) '|       All rights reserved       |'
      write(io11,*) '|see GNU General Public License   |'
      write(io11,*) '|---------------------------------|'
c
      write(io11,'('' Program bwht  '')')
      write(io11,'('' Input File: '',a)') infil
      write(io11,'('' Output File: '',a)') outfil
c
      write(io11,'(''  Number of traces='',i5)') ntrace
      write(io11,'(''  Parameters:  '')')
      write(io11,'(''  twide= '',f10.2)') twide
      write(io11,'(''  freqc= '',f10.2)') freqc
      write(io11,'(''  bwdth= '',f10.2)') bwdth
      write(io11,'(''  deltf= '',f10.2)') deltf
      write(io11,'(''  nfilt= '',i5)')    nfilt    
      write(io11,'(''  number of points in filter= '',i10)') npf
      write(io11,*)'     J    F_Center (Hz)'
      fff=freq1
      do 12 ifilt=1,nfilt
      write(io11,'(2x,i5,5x,f10.2)') ifilt,fff
      fff=fff+deltf
   12 continue

      write(io11,'(50(1h-))')
      write(io11,'(''    '')')
c
      return
      end

